#include "winglib/wgWindow.h"
//////////////////////// wgDialog class


#ifndef _WG_DIALOG
#define _WG_DIALOG


#define _UNSPECIFIED	0
#define _MODAL			1
#define _MODELESS		2

#define SUTE_BAD_RESOURCE_ERROR -1  // problem with resource code, most likely wrong code or clash
#define SUTE_UNDEFINED_ERROR -2  // call GetLastError for more accurate information from win32
#define SUTE_NO_SUCH_DIALOG_ERROR -3
#define SUTE_USER_STUPIDITY_ERROR -4




/// wgDialog class is the base class used to create a DialogBox. 
/// This Dialogbox may be Modal or Non-Modal depending upon the method of Windows 
/// DialogBox Instantiation. Mulitple DialogBoxes may be Instantiated over the life of 
/// a single instance of a wgDialog class, but only one at a time may be instantiated. 
/// DialogBoxes are instantiated through the wgDialog::DoModal() function or 
/// the wgDialog::Create() function. <br><br>
/// To create an wgDialog Box, simply inherit this function as the base class for a 
/// Singular DialogBox class. Override the appropriate Message Handler Member functions 
/// to provide the required functionality. The 'OnOther' Message handler function is 
/// provided for all those Messages that are not already provided for by an in-built 
/// inline function. 
class wgDialog : public wgWinMakerBase, public virtual wgWinBase{

	public:

		wgDialog(int DIALOG_RESOURCE_CODE, HWND hwndParent=NULL);
		wgDialog(int DIALOG_RESOURCE_CODE, HINSTANCE _hInstance, HWND hwndParent=NULL);
		wgDialog(LPCTSTR lpTemplate, HINSTANCE _hInstance, HWND hwndParent=NULL);

		virtual ~wgDialog()=0;

	// regular kinda functions

		HWND Create();
		BOOL Show(int cmd_show = SW_SHOW);
		BOOL Hide();
		int DoModal(); 


		unsigned int getType();

			/// return 0 is good
			/// return non-zero bad(see above for details)
		int setDlgItemText(int DlgItem, LPCTSTR lpText); 

		int DlgItemAppendText(int DlgItem, LPCTSTR lpText);

		HWND GetDlgItem(int nIDDlgItem);   // identifier of control


	protected:
		
		// various control functions to override in implemented derived class

		virtual inline void OnInitDialog(){}	///< unique to dialog

			/// OnCommand gets passed -->(LOWORD(wParam), HIWORD (wParam)) from the base class message handler
		virtual inline BOOL OnCommand(int IDCode, int notifyCode){return FALSE;}

			/// OnNotify  gets passed -->(wParam, (NMHDR*)lParam) from the base class message handler
		virtual inline BOOL OnNotify(int IDCode, NMHDR *hdr){return FALSE;}
	
		virtual inline BOOL OnOther(UINT message, WPARAM wParam, LPARAM lParam){return FALSE;}

		// override to control WM_DESTROY
		virtual void OnDestroy(); // Default action (ONLY IF MODELESS) is to PostQuitMessage 
		// type gets automatically set to _UNSPECIFIED OnDestroy, or return of DoModal
	
	// protected member variables

		HWND parent;

	private:


	// private base member functions
		
		static BOOL CALLBACK DialogProcedure(HWND _hWnd, UINT message, WPARAM wparam, LPARAM lparam);

		LPCTSTR MakeResource();

		void _OnShowWindow(BOOL fShow, int fnStatus);
		void _OnSize(WPARAM wParam, LPARAM lParam);
		void _OnMove(WPARAM wParam, LPARAM lParam);
		inline void _OnInitDialog(HWND _hWnd);
		void CloseWindow(int result);
		inline void _OnClose();
		inline void _OnDestroy();
		inline void Cleanup();

	// private member variables

		unsigned char type; // this is either _MODELESS, _MODAL, or _UNSPECIFIED
		unsigned int dialog_resource_code; // stores the code only if the appropriate constructor 
										// is used,.otherwise is null to indicated (used string)

		void* resource; // pointer to either string or converted resource_code LPCTSTR
};

#endif //_WG_DIALOG