

#include "winglib/wgWinStatic.h"
#include <winutils/wuException.h>
#include <string.h>
#include <stdio.h>
#include <commdlg.h>
#define _WIN32_IE 0x0400
#include <commctrl.h>



#ifndef _WIN_BASE
#define _WIN_BASE






// forward definition for friend
class wgWinMenuBar;


typedef  int (*FileDialogProc)(OPENFILENAME& ofn, void* param);

typedef LRESULT (__stdcall *WindowProcessor)(HWND hWin, UINT Msg, WPARAM wParam, LPARAM lParam);


#define CWM_UNKNOWN_MESSAGE		WM_USER
#define CWM_ATTACH_WINDOW		WM_USER+1
#define CWM_CAPTURECHANGED		WM_USER+2
#define CWM_LBUTTONDBLCLK		WM_USER+3 
#define CWM_LBUTTONDOWN			WM_USER+4
#define CWM_LBUTTONUP			WM_USER+5
#define CWM_MBUTTONDBLCLK		WM_USER+6
#define CWM_MBUTTONDOWN			WM_USER+7
#define CWM_MBUTTONUP			WM_USER+8
#define CWM_MOUSEACTIVATE		WM_USER+9
#define CWM_MOUSEMOVE			WM_USER+10
#define CWM_MOUSEWHEEL			WM_USER+11
#define CWM_NCHITTEST			WM_USER+12
#define CWM_NCLBUTTONDBLCLK		WM_USER+13
#define CWM_NCLBUTTONDOWN		WM_USER+14  
#define CWM_NCLBUTTONUP			WM_USER+15 
#define CWM_NCMBUTTONDBLCLK		WM_USER+16  
#define CWM_NCMBUTTONDOWN		WM_USER+17  
#define CWM_NCMBUTTONUP			WM_USER+18  
#define CWM_NCMOUSEMOVE			WM_USER+19  
#define CWM_NCRBUTTONDBLCLK		WM_USER+20  
#define CWM_NCRBUTTONDOWN		WM_USER+21  
#define CWM_NCRBUTTONUP			WM_USER+22  
#define CWM_RBUTTONDBLCLK		WM_USER+23  
#define CWM_RBUTTONDOWN			WM_USER+24  
#define CWM_RBUTTONUP			WM_USER+25   





///
/// wgWinBase is the base class for all 'Simple' Windows found in the Simple Foundation Class. 
/// It contains the basic functionality that ALL Simple Windows should have, such as 
/// appropriately overloaded MessageBox functions, repaint, and commonly used message handlers.
/// 
class wgWinBase{

	public:

	// public member functions

		wgWinBase();
		virtual ~wgWinBase()=0;

		int MessageBox(LPCTSTR lpText, LPCTSTR lpCaption=NULL, UINT uType=MB_OK);

		int MessageBox(HWND _hWnd, LPCTSTR lpText, LPCTSTR lpCaption=NULL, UINT uType=MB_OK);

		int MessageBox(int code, LPCTSTR lpCaption=NULL, UINT uType=MB_OK);

		inline void repaint(BOOL Erase){ // erase background before repaint ?

			InvalidateRect(hWnd, NULL, Erase);
		}

		inline void MoveWindow(int x, int y){

			SetWindowPos(hWnd, HWND_NOTOPMOST, x, y, 0, 0, SWP_NOZORDER|SWP_NOSIZE);
		}

		inline void ReSizeWindow(int _Width, int _Height){
			
			ReSizeWindow(hWnd, _Width, _Height);
		}

		inline static void ReSizeWindow(HWND _hWnd, int _Width, int _Height){

			SetWindowPos(_hWnd, HWND_NOTOPMOST, 0, 0, _Width, _Height, SWP_NOOWNERZORDER |SWP_NOMOVE |SWP_NOZORDER);
		}

		inline HWND GetHwnd(){return hWnd;}
		inline HINSTANCE getHINSTANCE(){return hInstance;}

		inline BOOL PostMessage(UINT Msg, WPARAM wParam, LPARAM lParam){
			return ::PostMessage(hWnd, Msg, wParam, lParam);
		}

		inline LRESULT SendMessage(UINT Msg, WPARAM wParam, LPARAM lParam){
			return ::SendMessage(hWnd, Msg, wParam, lParam);
		}	
	

		static int  SaveDialogBox(FileDialogProc _Proc,			///< 
								  void* param,					///<
								  LPTSTR _lpstrFile,			///<
								  DWORD _nMaxFile,				///<
								  LPCTSTR _lpstrFilter=NULL,	///<
								  DWORD _nFilterIndex=0,		///<
								  LPCTSTR _lpstrInitialDir=NULL,///<
								  LPCTSTR _lpstrTitle=NULL,		///<
								  DWORD _nMaxFileTitle=0,		///<
								  DWORD _Flags=OFN_PATHMUSTEXIST,///<
								  HWND _Owner=NULL,				///<
								  HINSTANCE _hInstance=NULL		///<

		);

		static int  OpenDialogBox(FileDialogProc _Proc,
								  void* param,
								  LPTSTR _lpstrFile,
								  DWORD _nMaxFile,
								  LPCTSTR _lpstrFilter=NULL,
								  DWORD _nFilterIndex=0,
								  LPCTSTR _lpstrInitialDir=NULL,
								  LPCTSTR _lpstrTitle=NULL,
								  DWORD _nMaxFileTitle=0,
								  DWORD _Flags=OFN_PATHMUSTEXIST|OFN_FILEMUSTEXIST,
								  HWND _Owner=NULL, 
								  HINSTANCE _hInstance=NULL
		);


			/// Add the specified Menubar to this Window
		BOOL Add(wgWinMenuBar &MenuBar);
		BOOL DrawMenuBar();

	// public base member variables


		static int wgMessageQueue(int WindowMessage);

	protected:

	// protected base member functions

			/// If an application processes this message, it should return zero. 
		virtual inline LRESULT OnShowWindow(BOOL fShow, int fnStatus){return 1;}
			/// If an application processes this message, it should return zero. 
		virtual inline LRESULT OnSize(WPARAM wParam, LPARAM lParam){return 1;}
			/// An application should return TRUE if it processes this message
		virtual inline BOOL OnSizing(WPARAM wParam, LPARAM lParam){return FALSE;}
			/// If an application processes this message, it should return zero. 
		virtual inline LRESULT OnMove(WPARAM wParam, LPARAM lParam){return 1;}
			/// An application should return TRUE if it processes this message
		virtual inline BOOL OnMoving(WPARAM wParam, LPARAM lParam){return FALSE;}
			/// If an application processes this message, it should return zero. 
		virtual inline LRESULT OnPaint(){return 1;}
			/// If an application processes this message, it should return zero. 
		virtual inline LRESULT OnNCDestroy(){return 1;}
			/// If an application processes this message, it should return zero. 
		virtual inline LRESULT OnClose(){return 1;} // override to control Close 

		virtual inline long Width(){return SWin_Width;}
		virtual inline long Height(){return SWin_Height;}
		virtual inline long Left(){return SWin_Left;}
		virtual inline long Top(){return SWin_Top;}

	// protected base member variables

		HINSTANCE hInstance;
		HWND hWnd;

		long SWin_Width;
		long SWin_Height;
		long SWin_Left;
		long SWin_Top;
};

typedef wgWinBase* HwgWinBase;


typedef  wgWinBase* (*WindowCreatorFunc)(void* _ptrUserData);



///
/// WG_WindowHandler is a function used by WG_WindowCreator
///
DWORD WINAPI WG_WindowHandler(LPVOID lpParameter);


///
/// WG_WindowCreator is a helper function to create dialogs. It allows these dialogs to 
/// be under the control of a separate windows procedure queue.
///
HANDLE WINAPI WG_WindowCreator(WindowCreatorFunc _UserFunc, void* _ptrUserData) throw (exception);




// /////////////////   start of wgwgWinMakerBase  /////////////


#endif //_WIN_BASE

