


#include "winglib/wgWinBase.h"



////////////////////   START OF wgWinControl Class Prototype   //////////////////


#ifndef _SCONTROL
#define _SCONTROL


/// Scontrol class is the base class to use for easy subclassing of a control. Messages 
/// can be captured by simply overriding the appropriate Scontrol virtual function Message 
/// Handler, or if it doesn't exist, overriding the 'OnOther' Default Message handler. 
/// Note: After Overriding functions are called, the original Control Message Handler 
/// function, is ALWAYS called. 

class wgWinControl : public virtual wgWinBase{

	public:

		wgWinControl(const char* _pcharControlName);

		wgWinControl(	const char* _pcharControlName,
						int RESOURCE_CODE,	// RESOURCE CODE of control 
						HwgWinBase _Parent,// Handle to Parent of Control
						int _Width=0,		// Width of ZERO means don't change (actual resource)
						int _Height=0		// Height of ZERO means don't change
						);


		wgWinControl(	const char* _pcharControlName,
						HWND _Button,		//	Handle to control window
						HwgWinBase _Parent,//	Handle to Parent of Control
						int _Width=0,		// Width of ZERO means don't change
						int _Height=0		// Height of ZERO means don't change
						);

		virtual ~wgWinControl()=0;


		void Associate(HWND _hWnd);
		void Attach(); // final --- don't touch
		void UnAttach();
		inline char IsCreated(){return blnCreated;}

		char* GetControlID(char* pcharBuffer, unsigned int pcharLen);


		virtual inline LRESULT OnRedirected(UINT message, WPARAM wParam, LPARAM lParam){return 1;}

	protected:

		void SetControlID(const char* _pcharControlID);

	// protected member functions

		virtual inline void OnInit(){}

		virtual inline LRESULT OnDestroy(){return 1;}
		virtual inline LRESULT OnOther(UINT message, WPARAM wParam, LPARAM lParam){return 1;}
		virtual inline LRESULT OnMouseMove(WPARAM wParam, LPARAM lParam){return 1;}
		virtual inline LRESULT OnMouseLButtonDown(WPARAM wParam, LPARAM lParam){return 1;}
		virtual inline LRESULT OnMouseLButtonUp(WPARAM wParam, LPARAM lParam){return 1;}
		virtual inline LRESULT OnMouseRButtonDown(WPARAM wParam, LPARAM lParam){return 1;}
		virtual inline LRESULT OnMouseRButtonUp(WPARAM wParam, LPARAM lParam){return 1;}
		virtual inline LRESULT OnCaptureChanged(HWND hwndNewCapture){return 1;}	
														// sent to window loosing
														// capture. It recieves HWND of window
														// gaining the mouse capture
	// protected member variables

		HwgWinBase Parent;


	private:

	// private member functions

		static LRESULT APIENTRY ControlProcedure(HWND _hWnd, UINT message, WPARAM wparam, LPARAM lparam);
		void _OnUser(); // this sets up the window, fixed initialization before call to OnInit
		LRESULT _OnNCDestroy();

	// private member variables

		LONG OldProc;
		char blnCreated;
		char* pcharControlID;
};

#endif // _SCONTROL

