#include "winglib/wgWinMakerBase.h"
#include <yvals.h>              // warning numbers get enabled in yvals.h 
#pragma warning(disable: 4786)  // identifier was truncated to 'number' characters in 
                                // the debug information
	// BUG: C4786 Warning Is Not Disabled with #pragma Warning
	// STATUS: Microsoft has confirmed this to be a bug in the Microsoft product. 
	// This warning can be ignored. This occured only in the <map> container.
#include <map>




#ifndef _WINMAKER
#define _WINMAKER




typedef std::map<HWND, void*> UserParamMapType;


/// WinMaker class is used to create and manage Windows, simplifying the development of a 
/// WIN CLASS (WNDCLASSEX), registration with windows, and the actual creation of the window.
/// It also provides the capability for Window related memory storage through two functions, 
/// get_param, and put_param. Additionally, WinMaker class inherits from class WinMaker_BASE 
/// which means that it automatically registers Windows created with this class with the system 
/// registry. in doing this it provides a means to keep track of opened windows, and closed 
/// windows through the close member function of WinMaker. By keeping track of windows in this 
/// manner, we are able (through the GetNumOpenWindows member function of WinMaker_BASE of which 
/// WinMaker inherits from, check the number of windows opened in an application. With this 
/// knowledge we can quit the application when necessary. 

class wgWinMaker : public wgWinMakerBase{

	public:

		wgWinMaker(HINSTANCE,	///< (hInstance) handle to instance of window
				 UINT,		///< cbsize
				 LPCTSTR,	///< lpszClassName
				 WNDPROC,	///< window process function
				 HBRUSH,	///< hbrBackground
				 HCURSOR,	///< hCursor
				 HICON,		///< hIcon
				 HICON,		///< hIconSm
				 LPCTSTR,	///< lpszMenuName
				 UINT,		///< style
				 int,		///< cbClsExtra
				 int		///< cbWndExtra
		);

		wgWinMaker(WNDCLASSEX aclass);

		virtual ~wgWinMaker();

		ATOM register_class();

	
			/// The WinMaker::create_window member function creates a non-modal Dialog Box. 

		HWND create_window( char* window_title,			///< A String of characters containing text to place in the Window Title-bar
							DWORD style,				///< Specifies the style of the window being created. This parameter can be a combination of the following window styles, plus the control styles indicated in the Remarks section.
							HMENU menu,					///< Handle to a menu, or specifies a child-window identifier depending on the window style. For an overlapped or pop-up window, hMenu identifies the menu to be used with the window; it can be NULL if the class menu is to be used. For a child window, hMenu specifies the child-window identifier, an integer value used by a dialog box control to notify its parent about events. The application determines the child-window identifier; it must be unique for all child windows with the same parent window. 
							void* mdic_lparam,			///< A pointer to an instance of LPARAMS is passed to the created window's WM_CREATE Message. mdic_lparam is stored as the ' user_param' member of this LPARAMS. A pointer to this WinMaker is stored in the 'hwinmaker' parameter of LPARAMS. 
							int x = CW_USEDEFAULT,		///< Specifies the initial horizontal position of the window. For an overlapped or pop-up window, the x parameter is the initial x-coordinate of the window's upper-left corner, in screen coordinates. For a child window, x is the x-coordinate of the upper-left corner of the window relative to the upper-left corner of the parent window's client area. 
														///< If this parameter is set to CW_USEDEFAULT, the system selects the default position for the window's upper-left corner and ignores the y parameter. CW_USEDEFAULT is valid only for overlapped windows; if it is specified for a pop-up or child window, the x and y parameters are set to zero. 
							int y = CW_USEDEFAULT,		///< Specifies the initial vertical position of the window. For an overlapped or pop-up window, the y parameter is the initial y-coordinate of the window's upper-left corner, in screen coordinates. For a child window, y is the initial y-coordinate of the upper-left corner of the child window relative to the upper-left corner of the parent window's client area. For a list box, y is the initial y-coordinate of the upper-left corner of the list box's client area relative to the upper-left corner of the parent window's client area. 
														///< If an overlapped window is created with the WS_VISIBLE style bit set and the x parameter is set to CW_USEDEFAULT, the system ignores the y parameter. 
							int nWidth = CW_USEDEFAULT, ///< Specifies the width, in device units, of the window. For overlapped windows, nWidth is either the window's width, in screen coordinates, or CW_USEDEFAULT. If nWidth is CW_USEDEFAULT, the system selects a default width and height for the window; the default width extends from the initial x-coordinate to the right edge of the screen, and the default height extends from the initial y-coordinate to the top of the icon area. CW_USEDEFAULT is valid only for overlapped windows; if CW_USEDEFAULT is specified for a pop-up or child window, nWidth and nHeight are set to zero. 
							int nHeight = CW_USEDEFAULT,///< Specifies the height, in device units, of the window. For overlapped windows, nHeight is the window's height, in screen coordinates. If nWidth is set to CW_USEDEFAULT, the system ignores nHeight. 
							DWORD dwExStyle=NULL,		///< Extra Window Style
							HWND _Parent=NULL			///< Parent window to this window
		);
		
		BOOL show(HWND _hwnd, int cmd_show);			///< The WinMaker::Show member function affects the Show state of a window. 
		HWND get_running_window();						///< The WinMaker::get_running_window member function returns a handle to a currently running window of the same Window-class. 
		void* get_param(HWND ahwnd);					///< The WinMaker::get_running_window member function returns a pointer to user created memory related to a specified HWND, and of this Window-Class. 
		void put_param(HWND ahwnd, void* param);		///< The WinMaker::put_running_window member function stores user created memory related to a specified HWND, and of this Window-Class. 
		void close(HWND ahwnd);							///< The WinMaker::close function cleans up memory used for storage for a Window created through the WinMaker class, or any other subclass of WinMaker_BASE, Destroys the window (through the call DestroyWindow, and unregisters the specified window with the system registry.
		void unregister(HWND _hWnd);					///< unregisters a given window if it exists in registry
		void update_registry(HWND _hWnd);				///< registers a given window if it exists ....
		void unregister_wgWinMaker();
		void register_wgWinMaker();

				

		// member variables

	protected:

		UserParamMapType UserParamMap;
		WNDCLASSEX wclassex;
		HANDLE WMSem; // semaphore to synchronize access to the SLinkedList
};
// end of class wgWinMaker




struct LPARAMS{

	wgWinMaker* hwinmaker;
	void* user_param;
};



#endif //_WINMAKER
