// wgWindows  --- amalgamation of wgWinMaker.h wgWinController.h 
//				and wgWinLayout.h, wgDialog, wgWindow, and all other related classes 
//				and functions ---

// Written by Sean Van Buggenum
// 25/06/03
// Email: sean@wg.com.au
#include "winglib/wgWinBase.h"
#include "winglib/wgWinMaker.h"
#include <stdio.h>

#ifndef _WG_WINDOW
#define _WG_WINDOW

#define NO_MENU -1
 



/// wgWindow class is the base class used to create a regular window. Mulitple Windows 
/// may be Instantiated over the life of a single instance of a wgWindow class, 
/// but only one at a time may be instantiated. Windows are instantiated through the 
/// wgWindow::Create() function. <br><br>
/// To create an wgWindow, simply inherit this function as the base class for a Singular 
/// Window class. Override the appropriate Message Handler Member functions to provide the 
/// required functionality. The 'OnOther' Message handler function is provided for all 
/// those Messages that are not already provided for by an in-built inline function.<br><br>
/// If a Menu is not required, it is recommended to set the argument MenuID to NO_MENU, 
/// as defined in WinSute.h:<br><br>
/// #define NO_MENU -1<br><br>
/// To easily assign icons to the window (both arguments LargeIcon, and SmallIcon), 
/// at is recommended you use the Global Function call MAKE_HICON. 


class wgWindow : public virtual wgWinBase{

	public:

		wgWindow(HINSTANCE _hInstance, 
				LPCTSTR ClassName, 
				UINT style=CS_VREDRAW|CS_HREDRAW,
				int Menu=NO_MENU, 
				HICON LargeIcon=NULL, 
				HICON SmallIcon=NULL, 
				HBRUSH brush=NULL, 
				HCURSOR cursor=NULL, 
				int ClassExtra=0,
				bool _blnUseDialogDefaultProc=false
		) throw (wuException);
	
		wgWindow(HINSTANCE _hInstance, bool _blnUseDialogDefaultProc) throw (wuException);


		virtual ~wgWindow()=0;

		HWND Create(char* window_title, 
					DWORD style, 
					int x = CW_USEDEFAULT, 
					int y = CW_USEDEFAULT, 
					int nWidth = CW_USEDEFAULT, 
					int nHeight = CW_USEDEFAULT,
					HMENU menu=NULL,
					DWORD dwExStyle=NULL,
					HWND _Parent=NULL) throw (wuException);
		
		BOOL Show(int nCmdShow=SW_SHOW);

		unsigned int GetNumOpenWindows();

	protected:

	// protected member functions

	// WINDOWS MESSAGES

			/// If an application processes this message, it should return 0 to 
			/// continue creation of the window. If the application returns -1, 
			/// the window is destroyed and the CreateWindowEx or CreateWindow function returns a NULL handle. 
		virtual inline LRESULT OnCreate(){return 1;}
		virtual inline LRESULT OnCommand(WPARAM wParam, LPARAM lParam){return 1;}
		virtual inline LRESULT OnOther(UINT message, WPARAM wParam, LPARAM lParam){return 1;}
		virtual inline LRESULT OnNotify(WPARAM wParam, LPARAM lParam){return 1;}
		virtual inline LRESULT OnDestroy();

	// KEYBOARD MESSAGES

		virtual inline LRESULT OnChar(WPARAM wParam, LPARAM lParam){return 1;}
		virtual inline LRESULT OnKeyDown(WPARAM wParam, LPARAM lParam){return 1;}
		virtual inline LRESULT OnKeyUp(WPARAM wParam, LPARAM lParam){return 1;}
		virtual inline LRESULT OnSysKeyDown(WPARAM wParam, LPARAM lParam){return 1;}
		virtual inline LRESULT OnSysKeyUp(WPARAM wParam, LPARAM lParam){return 1;}

	// protected member variables

		static const UINT CREATE_APP_SUBCLASSING;
		wgWinMaker* winmaker;

		WindowProcessor GetWindowMessageHandler();

	private:

	// private base member functions
		
		static LRESULT CALLBACK _WinProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam);
		static LRESULT CALLBACK _DlgWinProc(HWND _hWnd, UINT message, WPARAM wParam, LPARAM lParam);
		LRESULT CALLBACK CommonControlSwitch(HWND _hWnd, UINT message, WPARAM wParam, LPARAM lParam);

		LRESULT _OnShowWindow(BOOL fShow, int fnStatus);
		LRESULT _OnSize(WPARAM wParam, LPARAM lParam);
		LRESULT _OnMove(WPARAM wParam, LPARAM lParam);
		inline LRESULT _OnCreate(HWND ahwnd);
		inline LRESULT _OnClose();
		inline LRESULT _OnDestroy();
		inline void Cleanup();

		char created;	
		bool blnUseDialogDefaultProc;
		WindowProcessor hDefWinProc;
		WindowProcessor hWinProc; 

};


#endif // _WG_WINDOW









