#include "winutils/wuFile.h"
#include <vector>
#include "winutils/wuFile.h"
#include <string>
#include "winutils/wuSysLogger.h"


#ifndef _WU_CSV_LOG_FILE
#define _WU_CSV_LOG_FILE




//--------------------------------------------------------------

///
/// wuCSVLogFile is a utility to assist in the logging of information to a CSV file. 
///
class wuCSVLogFile : public wuFile
{ 
	public:

			/// Default constructor
		wuCSVLogFile(void);

			/// Constructor
		wuCSVLogFile(	const std::string&		strAppName,					///< Application name
						const std::string&		strLogFileUniqueName,		///< Unique Log filename string
						const std::string&		strFilePath,				///< File path, absolute or relative
						const std::string&		strDelim,					///< Delimeter to be used 
						unsigned long			lngMaxFileSize,				///< Max file size per file (bytes)
						unsigned int			intMaxNumFiles,				///< Maximum number of files at any one time.
						bool					blnCacheLineIfWriteFail		///< Cache the current built line if a log write fails.
					);

			/// Destructor
		virtual ~wuCSVLogFile(void);

			/// initialise settings
			/// Note: max file size is inputed in bytes
		void InitialiseSettings(const std::string&			strAppName,				///< Application name
								const std::string&			strLogFileUniqueName,	///< Unique Log filename string
								const std::string&			strFilePath,			///< File path, absolute or relative
								const std::string&			strDelim,				///< Delimeter to be used 
								unsigned long				lngMaxFileSize,			///< Max file size per file (bytes)
								unsigned int				intMaxNumFiles,			///< Maximum number of files at any one time.
								bool						blnCacheLineIfWriteFail	///< Cache the current built line if a log write fails.
							);

			/// Open log file
		virtual bool Open(bool blnNewFile = false);

			/// Log the currently built log to file. 
			/// \param blnEndLine if true ensures an endline character is written at the end of the log line.
			/// otherwise, no endline, but a comment will end the current log.
		bool Log(bool blnEndLine);

			/// Add a value followed by a delimeter to the current log string.
		void AddValue(const std::string& strValue);

			/// Clear the log string that has been built up so far. 
		void ClearLogString();
	
			/// Return the log string that has been built up so far. 
		const std::string GetLogString();

			/// Set the log string to the string provided. 
		void SetLogString(const std::string& strLine);

			/// Set the delimeter to be used with the log object. 
		void SetDelimeter(const std::string& strDelim);

			/// Roll to next file
		void RollToNextFile(void);

	// HELPER FUNCTIONS

	protected:

		bool Log(const char* LogString);

		void SetFileName(void);

		void LimitNumLogFiles(void);

	private:

		std::string				mstrLine;
		std::string				mstrDelim;

		std::string				mstrAppName;			// application name
		std::string				mstrLogFileUniqueName;
		SYSTEMTIME				mSYSTIMELastWriteDate;		// last write date
		unsigned long			mlngMaxFileSize;		///< Size in bytes
		unsigned int			mintMaxNumFiles;
		unsigned int			mintFileCount;			// counts number of files today

		bool					mblnCacheLineIfWriteFail;
};


//--------------------------------------------------------------



#endif//_WU_CSV_LOG_FILE



