#include <yvals.h>              // warning numbers get enabled in yvals.h 
#pragma warning(disable: 4786)  // identifier was truncated to 'number' characters in 
                                // the debug information
	// BUG: C4786 Warning Is Not Disabled with #pragma Warning
	// STATUS: Microsoft has confirmed this to be a bug in the Microsoft product. 
	// This warning can be ignored. This occured only in the <map> container.
#include <map>
#include <hash_map>
#include "winutils/wuCalendarDefinitions.h"
#include "winutils/wuSYSTEMTIME.h"

#ifndef _WU_CALENDAR
#define _WU_CALENDAR



struct ltcmp
{
  bool operator()(const wuSYSTEMTIME& t1, const wuSYSTEMTIME& t2) const
  {
    return (t1<t2);
  }
};


typedef std::map <wuSYSTEMTIME, unsigned char, ltcmp> HolidayMapType;
typedef std::hash_map<int/*region code*/, HolidayMapType*> RegionHMapType;
typedef std::hash_map<int/*case ccy Code*/, void*> CountryHMapType;



///
/// wuCalendar maintains a list of non-work days and also provides calendar type functionality
/// such as determining leap year, day of the week, and time based arithmetic. 
/// Using this calendar one can determine (once initialised) if a given date is a work day.
///
class wuCalendar
{
	public:

		  /// Constructor
		wuCalendar();

		  /// Destructor
		~wuCalendar();

		  /// Add a non working day. 
		void AddNonWorkDay(const wuSYSTEMTIME& Date, a2CountryCode Country, int _intRegionCode);
		  /// Add a non working day. 
		void AddNonWorkDay(const wuSYSTEMTIME& Date, a3CountryCode Country, int _intRegionCode);
		 /// Add a non working day. 
		void AddNonWorkDay(const wuSYSTEMTIME& Date, ccyCurrencyCode Country, int _intRegionCode);

		  /// Add a country to the list. A country is not required to have holidays.
		void AddRegion(a2CountryCode Country, int _intRegionCode);
		  /// Add a country to the list. A country is not required to have holidays.
		void AddRegion(a3CountryCode Country, int _intRegionCode);
		  /// Add a country to the list. A country is not required to have holidays.
		void AddRegion(ccyCurrencyCode Country, int _intRegionCode);

		  /// Returns true, based on standard weekends and supplied non-working days for
		  /// the correct country and region, if the wuSYSTEMTIME is a work day.
		bool IsWorkingDay(const wuSYSTEMTIME& wDate, a2CountryCode Country, int _intRegionCode) throw (exception);
		bool IsWorkingDay(const wuSYSTEMTIME& wDate, a3CountryCode Country, int _intRegionCode) throw (exception);
		bool IsWorkingDay(const wuSYSTEMTIME& wDate, ccyCurrencyCode Country, int _intRegionCode) throw (exception);

	// static functions

		  /// Get the day of the week, given the year, month, and day of the month. 
		  /// This function returns an integer specifying the day of the week, where sunday = 0, 
		  /// monday = 1, ...., saturday = 6. This function is based on the Gregorian calendar.
		static unsigned char GetDayOfWeek(unsigned int uintYear, unsigned int uintMonth, unsigned int uintDayInMonth);

		  /// Returns true if the given date is a weekend.
		static bool IsWeekend(unsigned int uintYear, unsigned int uintMonth, unsigned int uintDayInMonth);

		  /// returns true if the given year is a leap year.
		static bool IsLeapYear(unsigned int uintYear);

		  /// returns the number of 100 Nano second blocks in a given Time Span.
		static const ULARGE_INTEGER GetTimeSpanIn100Nano(wuTimeSpan SpanConstant);

		  /// Adds an amount of time to the supplied wuSYSTEMTIME, specified by the TimeSpan multiplied by num.
		  /// The result is returned also in a wuSYSTEMTIME object.
		static wuSYSTEMTIME SYSAddTimeDiff(const wuSYSTEMTIME& DateTime,		///< valid datetime which we want change
											wuTimeSpan SpanConstant,		///< TimeSpan constant enum
											const __int64 num				///< Multiplier for the SpanConstant.
										);

		static unsigned int DaysInMonth(wuSYSTEMTIME objDate);

		  /// Adds a certain number of months to the supplied wuSYSTEMTIME. If the current dates day of the month
		  /// is more than the number of days in the resulting month, the difference is added to the number
		  /// of days in the resulting month, and wrapped to the next month. For example:
		  /// Add 1 month to the 31st of Jan, will result in a date of March 3rd for a non-leap year, and 
		  /// March 2nd for a leap year.
		  /// The result is returned also in a wuSYSTEMTIME object.
		static wuSYSTEMTIME SYSAddMonths(	const wuSYSTEMTIME& pst,		///< valid datetime which we want change
										int intAddMonths
										);


		static wuSYSTEMTIME SYSAddYears(	const wuSYSTEMTIME& pst,		///< valid datetime which we want change
										int intAddYears
										);

		  /// returns the number of seconds, or minutes etc depending on the SpanConstant provided
		static ULARGE_INTEGER GetAbsDiff(const wuSYSTEMTIME& Left, const wuSYSTEMTIME& Right, wuTimeSpan SpanConstant);

		  /// ConformDate sets the time values components to this DateTime to the zero hour (12 midnight).
		static wuSYSTEMTIME ConformDate(const wuSYSTEMTIME& _DateTime);

		  /// Remove all non-working days and Countries from the Holiday list.
		void CleanCalendar();

	protected:

		  /// used only by public AddNonWorkDay(const wuSYSTEMTIME& Date, a3CountryCode Country, int _intRegionCode);
		void AddNonWorkDay(const wuSYSTEMTIME& Date, int _intRegionCode, RegionHMapType* RegionalHMap);
	
		  /// Returns a pointer to the stored HolidayMapType which relates to the Country and region specifically. 
		HolidayMapType* GetHolidayMap(a3CountryCode Country, int _intRegionCode);


	private:

		void FreeNationalHolidays(RegionHMapType* RegionalMap);

		CountryHMapType mIntHolidaysMap;
};




#endif//_WU_CALENDAR


