//////////////////////// wuSocket class header //////////////////////////////

// ifdefines to put in.. friends and includes to take out.
// the following line gets rid of winsock (1) which avoids the nasty microsoft winsock compilation redefinition errors
#include <winsock2.h>
  // we want to include winsock2 before including winsock1
#ifndef _WINSOCKAPI_	
#define _WINSOCKAPI_	
#endif// _WINSOCKAPI_	

#include <winsock2.h>
#include <stdio.h>
#include "winutils/wuSocketException.h"

 

 
#ifndef _WU_SOCKET_IMPL_H_
#define _WU_SOCKET_IMPL_H_


#pragma warning( disable : 4290 ) // warning C4290: C++ Exception Specification ignored

#define _WU_SOCKETS_NOOFERRORCODES 48
#define MAXLINE 4096


/// wuSocket is the base class for all socket based objects. It contains functionality
/// which provides
class wuSocket
{
	public:

	// public member Functions

			/// Main Constructor. 
		wuSocket();

			/// Copy Constructor required for ServerSocket
		wuSocket(const wuSocket & other);

		wuSocket(int port) throw (wuSocketException); // only for serversocket

		virtual ~wuSocket();

			/// Reconnect simply closes and reconnects to the currently chosen address using the 
			/// same LocalPort and RemotePort parameters.
		void Reconnect() throw (wuSocketException);

			// Reconnect will only reconnect if the socket is known to be closed by the class. 
			// ForceReconnect will force a reconnect even if it is open. It will close the socket first.
		void ForceReconnect() throw (wuSocketException);


			/// Connect should be used as a client connect to a server or listening port.
			/// This function accepts I.P address OR Host-Name.
			/// This function will close the existing Socket before opening another.
		virtual void Connect(const char* host, unsigned int RemotePort) throw (wuSocketException, wuSocketAddressInUse);

			/// Connect should be used as a client connect to a server or listening port.
			/// This function accepts I.P address OR Host-Name.
			/// It enables the user to choose the local port that is allocated for this socket connection.
			/// This function will close the existing Socket before opening another.
		virtual void Connect(const char* ahost, unsigned int intLocalPort, unsigned int intRemotePort) throw (wuSocketException, wuSocketAddressInUse)=0;

			/// Connect should be used as a client connect to a server or listening port.
			/// This function accepts I.P address OR Host-Name.
			/// It enables the user to choose the local port that is allocated for this socket connection.
			/// This function will close the existing Socket before opening another.
		virtual void Connect(const char* ahost, unsigned int intLocalPort, unsigned int intRemotePort, unsigned int uintReceiveTimeout) throw (wuSocketException, wuSocketAddressInUse);

		void CreateSocket(const char* ahost, unsigned int uintLocalPort, unsigned int uintRemotePort, int intType) throw (wuSocketException, wuSocketAddressInUse);

		void ConnectOnSocket() throw (wuSocketException, wuSocketAddressInUse);

			/// Copy constructor. This is not often required as it is recommended 
			/// that references are passed around instead of objects. However this function
			/// is required for ServerSocket::Accept
		wuSocket& operator=(const wuSocket & other);
		

		int GetListenFD();
		int GetConnectedFD();
		bool IsConnected();

			/// GetListeningPort returns the value of the port used to listen to client connections by a server.
			/// This is only valid if the socket is associated with a server socket.
		inline unsigned int GetListeningPort(){return muintUsedLocalPort;}
			/// GetRemotePort returns the remote port connected to by a client machine.
		inline unsigned int GetRemotePort(){return muintRemotePort;}
			/// GetLocalPort returns the local port used by a client machine.
		inline unsigned int GetLocalPort(){return muintUsedLocalPort;}
		int Close();

	// new functions using write and read for binary transfer with pthread data controls

		virtual int Receive(char FAR* buf, int len);

			/// Possible values for flags are 0 (just do a read and remove that from the buffer)
			/// MSG_PEEK (read and do not remove that data read from the buffer),
			/// MSG_OOB (process out of band data). 
		virtual int Receive(char FAR* buf, int len, int flags);	
		
		virtual int Send(const char FAR* buf, int len);	
		virtual int Send(const char FAR* buf, int len, int flags);

		char* GetPeerName(char* pcharPeerNameBuffer, int intSizePeerName) throw(wuSocketException);
		unsigned short GetPeerPortAddress() throw(wuSocketException);

			/// Set the Read timeout for the socket. Setting to zero removes the timeout.
			/// In other-words, the read will never timeout when set to zero
		int SetReceiveTimeout(int intTimeMilli) throw (wuSocketException);

			/// Set the Write timeout for the socket. Setting to zero removes the timeout.
			/// In other-words, the Write will never timeout when set to zero
		int SetSendTimeout(int intTimeMilli) throw (wuSocketException);

			/// Set the total per-socket buffer space reserved for receives. (SO_RCVBUF)
		void SetReceiveBufferSize(int intSize);

			/// Get the total per-socket buffer space reserved for receives. (SO_RCVBUF)
		int GetReceiveBufferSize();


		static const char* TranslateWinsockStatus(int intCode);

		int CloseConnector();  // used by ServerSocket (friend)

	protected:

	// member functions

		int ActOnLastError();
		void RecordSuccessfulAction();
		bool WasLastActionSuccessful();

		void SetAddress(struct sockaddr_in* an_addr, int intPortNum);

		void Establish() throw (wuSocketException); // only for serversocket
	
			/// Socket::Accept accepts a client connection. This function can fail, partially fail
			/// or succeed. A partial fail means that for some reason the default Recieve Timeout
			/// could not be set. The socket however should still be in a useable state. 
			/// No exception is thrown if the recieve timeout default fails to be set.
		int Accept() throw(wuSocketException); // used by ServerSocket (friend)

	// member variables
  
		char* mpcharHost;

		SOCKET Sockfd;  // note: Win32 = typedef u_int SOCKET;
		SOCKET Connfd;
		unsigned int muintRemotePort;
		unsigned int muintLocalPort; // this is the requested local port.... can be zero
		unsigned int muintUsedLocalPort;	// this is the actual local port. Will never be zero unless connect failed.
		int mintClilen;
		bool mblnLastActionSuccessful;
		
		struct sockaddr_in local_addr, remote_addr;
		struct hostent *host_ptr;

		static const int smintErrorCodes[_WU_SOCKETS_NOOFERRORCODES];
		static const char* smstrErrorMessages[_WU_SOCKETS_NOOFERRORCODES];
};

// need to work out when to call cleanup on winsock

#endif //_WU_SOCKET_IMPL_H_
