
#include "winutils/wuException.h"




#ifndef _WU_SOCKET_EXCEPTION
#define _WU_SOCKET_EXCEPTION






///
/// wuSocketException should be used in Socket class. It inherits from wuException class
///

class wuSocketException : public wuException
{

	public:	

			/// Main constructor
		wuSocketException(const char* amessage) throw(): wuException(amessage){}
			/// Copy constructor
		wuSocketException(const wuSocketException& other) throw() : wuException(other){}

		static const char* NOTCONNECTED;	///< NOTCONNECTED is a code that indicates that the socket is not connected.
		static const char* NOHOST;			///< NOHOST is a code that indicates that the socket is not connected.
		static const char* NOSOCK;			///< NOSOCK is a code that indicates that the socket is not connected.
		static const char* REFUSE;			///< REFUSE is a code that indicates that the socket is not connected.
		static const char* BADWRITE;		///< BADWRITE is a code that indicates that the socket is not connected.
		static const char* BADREAD;			///< BADREAD is a code that indicates that the socket is not connected.
		static const char* BADCLIENT;		///< BADCLIENT is a code that indicates that the socket is not connected.
		static const char* EOF_ERROR;		///< EOF_ERROR is a code that indicates that the socket is not connected.
		static const char* UNDEF_ERROR;		///< UNDEF_ERROR is a code that indicates that the socket is not connected.
		static const char* WINSOCK_NOT_STARTED;
};
 


/// wuSocketInterupt = WSAEINTR
/// Blocking function was canceled.
/// This error is generated whenever a blocking socket operation is canceled 
/// by setting the Action property to the appropriate value or by using the Cancel method. 
/// Note that the Cancel event is fired when a blocking operation is canceled, and the 
/// generation of this error depends on how the application responds to this event.
class wuSocketInterrupt : public wuSocketException
{
	public:	

			/// Main constructor
		wuSocketInterrupt(const char* amessage) throw(): wuSocketException(amessage){}
			/// Copy constructor
		wuSocketInterrupt(const wuSocketInterrupt& other) throw() : wuSocketException(other){}
};


///
/// wuSocketTimeout should be used in Socket class. It inherits from wuSocketException class
///
class wuSocketWaitTimeout : public wuSocketException
{
	public:	

			/// Main constructor
		wuSocketWaitTimeout(const char* amessage) throw(): wuSocketException(amessage){}
			/// Copy constructor
		wuSocketWaitTimeout(const wuSocketWaitTimeout& other) throw() : wuSocketException(other){}
};


/// wuSocketAddressInUse = WSAEADDRINUSE
class wuSocketAddressInUse : public wuSocketException
{
	public:	

			/// Main constructor
		wuSocketAddressInUse(const char* amessage) throw(): wuSocketException(amessage){}
			/// Copy constructor
		wuSocketAddressInUse(const wuSocketAddressInUse& other) throw() : wuSocketException(other){}
};




///
/// wuSocketUndefError should be used in Socket class. It inherits from wuSocketException class
///

class wuSocketUndefError : public wuSocketException{

	public:	

			/// Main constructor
		wuSocketUndefError(const char* amessage) throw(): wuSocketException(amessage){}
			/// Copy constructor
		wuSocketUndefError(const wuSocketUndefError& other) throw() : wuSocketException(other){}
};



///
/// wuSocketEOFError should be used in Socket class. It inherits from wuSocketException class
///

class wuSocketEOFError : public wuSocketException{

	public:	

			/// Main constructor
		wuSocketEOFError(const char* amessage) throw(): wuSocketException(amessage){}
			/// Copy constructor
		wuSocketEOFError(const wuSocketEOFError& other) throw() : wuSocketException(other){}
};


///
/// wuSocketBadClient should be used in Socket class. It inherits from wuSocketException class
///

class wuSocketBadClient : public wuSocketException{

	public:	

			/// Main constructor
		wuSocketBadClient(const char* amessage) throw(): wuSocketException(amessage){}
			/// Copy constructor
		wuSocketBadClient(const wuSocketBadClient& other) throw() : wuSocketException(other){}
};


///
/// wuSocketBadRead should be used in Socket class. It inherits from wuSocketException class
///

class wuSocketBadRead : public wuSocketException{

	public:	
			/// Main constructor
		wuSocketBadRead(const char* amessage) throw(): wuSocketException(amessage){}
			/// Copy constructor
		wuSocketBadRead(const wuSocketBadRead& other) throw() : wuSocketException(other){}
};


///
/// wuSocketBadWrite should be used in Socket class. It inherits from wuSocketException class
///

class wuSocketBadWrite : public wuSocketException{

	public:	

			/// Main constructor
		wuSocketBadWrite(const char* amessage) throw(): wuSocketException(amessage){}
			/// Copy constructor
		wuSocketBadWrite(const wuSocketBadWrite& other) throw() : wuSocketException(other){}
};



///
/// wuSocketRefused should be used in Socket class. It inherits from wuSocketException class
///

class wuSocketRefused : public wuSocketException{

	public:	

			/// Main constructor
		wuSocketRefused(const char* amessage) throw(): wuSocketException(amessage){}
			/// Copy constructor
		wuSocketRefused(const wuSocketRefused& other) throw() : wuSocketException(other){}
};



///
/// wuSocketNoSocket should be used in Socket class. It inherits from wuSocketException class
///

class wuSocketNoSocket : public wuSocketException{

	public:	

			/// Main constructor
		wuSocketNoSocket(const char* amessage) throw(): wuSocketException(amessage){}
			/// Copy constructor
		wuSocketNoSocket(const wuSocketNoSocket& other) throw() : wuSocketException(other){}
};



///
/// wuSocketNoHost should be used in Socket class. It inherits from wuSocketException class
///

class wuSocketNoHost : public wuSocketException{

	public:	

			/// Main constructor
		wuSocketNoHost(const char* amessage) throw(): wuSocketException(amessage){}
			/// Copy constructor
		wuSocketNoHost(const wuSocketNoHost& other) throw() : wuSocketException(other){}
};



///
/// wuSocketNotConnected should be used in Socket class. It inherits from wuSocketException class
///

class wuSocketNotConnected : public wuSocketException{

	public:	

			/// Main constructor
		wuSocketNotConnected(const char* amessage) throw(): wuSocketException(amessage){}
			/// Copy constructor
		wuSocketNotConnected(const wuSocketNotConnected& other) throw() : wuSocketException(other){}
};




#endif//_WU_SOCKET_EXCEPTION


