
#include "winutils/wuSocketException.h"


#pragma warning( disable : 4290 ) // warning C4290: C++ Exception Specification ignored


#ifndef _WU_SYS_LOG_GENERATOR
#define _WU_SYS_LOG_GENERATOR


#define _SYSLOG_MAX_TAG					32
#define _SYSLOG_MAX_MESSAGE_CONTENT		920	




static const char* scpcharSysLogMonth[12] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};



// important!!  Do not change order. Numbering is significant
typedef enum
{
	syslgKernelMessage,
	syslgUserLevelMessage,
	syslgMailSystem,
	syslgSystemDaemon,
	syslgSecurityAuthMessage1,
	syslgInternallyGenerated,
	syslgLinePrinterSubSystem,
	syslgNetworkNewsSubSystem,
	syslgUUCPSubSystem,
	syslgClockDaemon1,
	syslgSecurityAuthMessage2,
	syslgFTPDaemon,
	syslgNTPSubSystem,
	syslgLogAudit,
	syslgLogAlert,
	syslgClockDaemon2,
	syslgLocalUse0,
	syslgLocalUse1,
	syslgLocalUse2,
	syslgLocalUse3,
	syslgLocalUse4,
	syslgLocalUse5,
	syslgLocalUse6,
	syslgLocalUse7

} SysLogFacilityCode;





// important!!  Do not change order. Numbering is significant
typedef enum
{
	syslgEmergency, 	// System is unusable
	syslgAlert,		// Action must be taken immediately
	syslgCritical,		// Critical conditions
	syslgError,		// Error conditions
	syslgWarning,		// Warning conditions
	syslgNotice,		// normal but significant condition
	syslgInformational,	// Informational messages
	syslgDebug		// Debug-level messages	

} SysLogSeverityCode;


/*************************************    SEVERITY USAGE GUIDE    **************************************


Severity			Description
-------------------|------------------------------------------------------------------------------------|
Emergency			Something unthinkably bad has happened, and you probably haven't even thought to 
                    log this error as it probably seemed so far fetched and catastrophic. 
					Example: 

Alert               Something has failed which could imply that something is wrong system wide. 
                    This needs to be checked ASAP.
					Example:

Critical            Something in the application has failed, and this will cause the application to 
                    close.
					Example:

Error               Something has not worked (and it was hoped that it would work). This means that
                    an important operation will not, or has not worked.
                    Example:

Warning             Something has not worked (and it was hoped that it would work), but operations 
                    will probably not suffer for this. or we can recover to some respect.
                    Example: "Warning - Unable to use profiles, settings will not be saved"

Notice              Important events, system startup, major module startup, closure. 
                    Example: "Client disconnected. Input Handler closing now!"

Informational       Very important process information within an operation. Something that is important
                    to know has occurred. More important that debug, but not as significant as notice. 

Debug               Doing messages, positional messages. Used to track the program counter through the 
                    application and to indicate the occurance of events. 

***********************************************************************************************************/




static const char* cpcharSeverityCodeString[] = {	"syslgEmergency",
													"syslgAlert",
													"syslgCritical",
													"syslgError",
													"syslgWarning",
													"syslgNotice",
													"syslgInformational",
													"syslgDebug",
													""
											};




///
/// wuSysLogStringGenerator class provide the functionality necessary to create a log string that
/// conforms to the RFC 3164 - The BSD Syslog Protocol.
///
class wuSysLogStringGenerator
{
	public:

			/// Constructor
		wuSysLogStringGenerator();

			/// Deconstructor
		~wuSysLogStringGenerator();

			/// initialise the Syslogger.
		void Initialise(const char* cpcharProcessName,					///< The process name. This argument is to be used as the TAG field of the MSG part of the syslog as defined in RFC 3164.
						unsigned int uintWinsockSignificantVersNum=2,	///< version of winsock (significant digit)	
						unsigned int uintWinsockInsignificantVersNum=2	///< version of winsock (insignificant digit)	
					) throw (wuSocketException);

			/// Unitialise the syslogger. 
		void UnInitialise();

			/// report on the initialisation status of this syslogger. 
		bool IsInitialised();

			/// Create a Syslog string with the arguments as the message information.
		size_t CreatelogString (char* pcharLogStringBuffer,				///< The buffer into which to copy the Syslog String.
								unsigned int uintLogStringBufferLen,	///< The length of the buffer provided in pcharLogStringBuffer.
								SysLogFacilityCode Facility,			///< Facility indicates the Facility code as defined in RFC 3164.
								SysLogSeverityCode Severity,			///< param Severity indicates the Severity code as defined in RFC 3164.
								const char* cpcharMessageContent		///< The user defined message to log to the file. This is used as the CONTENT of the MSG part of a syslog as defined in  RFC 3164.	
							);

			/// converts a string to the appropriate SysLogSeverityCode enum (case insensitive).
		static SysLogSeverityCode StringToSeverityCode(const char* cpcharSeverity);

	private:

		char mpcharLocalHostName[1024];
		char mpcharProcessName[32];

		bool blnWSAInitialised;

};








#endif//_WU_SYS_LOG_GENERATOR