#include "winutils/wuSysLogStringGenerator.h"



#ifndef _WU_LOGGER
#define _WU_LOGGER


 
static const char* cpcharFacility[] = {	"Kernel Message",
									"User Level Message",
									"Mail System",
									"System Daemon",
									"Security Auth Message 1",
									"Internally Generated",
									"Line Printer SubSystem",
									"Network News SubSystem",
									"UUCP SubSystem",
									"Clock Daemon 1",
									"Security Auth Message 2",
									"FTP Daemon",
									"NTP SubSystem",
									"Log Audit",
									"Log Alert",
									"Clock Daemon 2",
									"Local Use 0",
									"Local Use 1",
									"Local Use 2",
									"Local Use 3",
									"Local Use 4",
									"Local Use 5",
									"Local Use 6",
									"Local Use 7",
									""
								};



static const char* cpcharSeverity[] = {	"Emergency",
										"Alert",
										"Critical",
										"Error",
										"Warning",
										"Notice",
										"Informational",
										"Debug"
								};





///
/// wuSysLogger is an abstract class and must be inherited from to use its functionality. 
/// The implementor must override the pure virtual function bool Log(const char* cpcharLogString)
/// in order to inherit from this class. 
///
class wuSysLogger
{
	public:
	
			/// Constructor
		wuSysLogger();

			/// destructor
		virtual ~wuSysLogger();

			/// initialise the Syslogger.
		virtual void Initialise(const char* cpcharProcessName,						///< The process name. This argument is to be used as the TAG field of the MSG part of the syslog as defined in RFC 3164.
								unsigned int uintWinsockSignificantVersNum=2,		///< version of winsock (significant digit)		
								unsigned int uintWinsockInsignificantVersNum=2		///< version of winsock (insignificant digit)
							);

			/// Unitialise the syslogger. 
		void UnInitialise();

			/// report on the initialisation status of this syslogger. 
		bool IsInitialised();

			/// the Log function organises the creation of the syslog, and provides the syslog string to 
			/// the implementor via the overridden function bool Log(const char* cpcharLogString).
			/// \param Facility indicates the Facility code as defined in RFC 3164.
			/// \param Severity indicates the Severity code as defined in RFC 3164.
			/// \param cpcharMessageContent is the user defined message to log to the file. This is
			/// used as the CONTENT of the MSG part of a syslog as defined in  RFC 3164.			
		virtual bool Log(	SysLogFacilityCode Facility, 
							SysLogSeverityCode Severity, 
							const char* cpcharMessageContent);

	protected:

		virtual	bool Log(const char* cpcharLogString)=0;

	private:

		wuSysLogStringGenerator StringGenerator;

		
};



#endif//_WU_LOGGER


