
#define _WINSOCKAPI_
#ifndef STRICT
	#define STRICT
#endif
#ifndef _WIN32_WINNT 
	#define _WIN32_WINNT 0x0400
#endif//_WIN32_WINNT
// fix for mfc
#if defined(WIN32)
	#ifdef _AFXDLL
		#include <afx.h>
	#else
		#include<windows.h>
	#endif
#endif	

#include "winutils/wuThread.h"


#ifndef _WU_WIN_SERVICE
#define _WU_WIN_SERVICE


///
/// wuWinService is a helper class to set up your application as a service. 
/// An application that uses wuWinService may also choose not to run as a service, simply by
/// calling the Run function (in a non-multithreading mode) or the Start function (multithreading). 
/// wuWinService inherits from class wuThread, and as such, in calling Start, will automatically clean 
/// its own memory. It should not then be created on the stack, but on the heap, and then NOT delete'd unless
/// the application is started in non-multithreading mode (i.e. started via the Run function). 
/// In this way we negate the need for seperate builds for Service, and Application. We need to just
/// setup the command arguments and call the appropriate functions in each case.  
///
class wuWinService : public wuThread
{
	public:

		wuWinService();

			/// The Service must always be initialised before any other service operations. Initialisation
			/// of the service will usually occur at the beginning of the main function. 
		void Initialise(	const char* pcharServiceName, 
							const char* pcharDisplayName, 
							const char* pcharDescription) throw ();

		virtual ~wuWinService();

			/// override this pure virtual function, and have the actual service application start from here. 
			/// If required, the application can call this function directly to have this not run as a service. 
			/// If, as an application only, it is required that the main application be started on a seperate thread, 
			/// call the Start function (inheritied from wuThread). Only in the case of directly calling the 
			/// Run function should be manually do any memory deallocation of instances of this class.
		virtual void Run()=0;

		static void WINAPI XControlHandler(DWORD dwControl);
		static void WINAPI XServiceMain(DWORD dwNumServiceArgs, LPTSTR *lpServiceArgs);

		void ControlHandler(DWORD dwControl);
		void ServiceMain(DWORD dwNumServiceArgs, LPTSTR *lpServiceArgs);

		bool ServiceInstalled(void);
		void UninstallService(void) throw();
		void InstallService(void) throw();
		int QueryServiceStatus() throw();
		void StartServiceCtrlDispatcher() throw();

		unsigned int GetServiceName(char* pcharBuffer, unsigned int len);
		unsigned int GetDisplayName(char* pcharBuffer, unsigned int len);
		unsigned int GetServiceDescription(char* pcharBuffer, unsigned int len);

	protected:

		virtual int ShutDown(){ return 0;}
		virtual int Restart(){ return 0;}
	

	private:

		SERVICE_TABLE_ENTRY mSrvTable[2];

		SERVICE_STATUS_HANDLE	mhCtrlStatus;
		SERVICE_STATUS			mStatus;

		char* mpcharDisplayName;
		char* mpcharServiceDescription;

		static wuWinService* mThis;
};




#endif//_WU_WIN_SERVICE


