#include "Simple.h"
#include "TextWindow.h"

LRESULT Simple::OnCreate(){

	LoadSkin();		// load the skin (specified in constructor) as bitmap
	MakeWindow();	// now create other child windows
					// this window will clean up automatically (see OnNCDestroy)

	return 0;
}
	

LRESULT Simple::OnPaint(){

	PAINTSTRUCT ps;

	HDC dc = BeginPaint(hWnd, &ps);	// get a device context
	DrawSkin(dc);					// draw the skin (bitmap) onto the window
	EndPaint(hWnd, &ps);			// end the paint session and give back dc

	return 0;
}



LRESULT Simple::OnCommand(WPARAM wParam, LPARAM lParam){ // all the menu options

	HWND TopWindow=NULL;
	char buffer[100]={0};

	switch(LOWORD(wParam))
	{
		case IDM_QUIT:
			PostMessage(WM_CLOSE, 0, 0); // tell the window to close and cleanup
			break;

		case IDM_NEW:
			MakeWindow();
			break;

		case IDM_CLOSE:
			// send to appropriate window
			TopWindow = GetTopWindow(hWnd); // get handle to top window
			::SendMessage(TopWindow, WM_CLOSE, 0, 0); // should send here so job
												// is fully done by next step (no post)
			TopWindow = GetTopWindow(hWnd); // get new top if there is one
			if (TopWindow==NULL) HideCloseMenuItem();
			break;

		case IDM_SAVEAS:
			// send to appropriate window
			TopWindow = GetTopWindow(hWnd); // get handle to top window
			::PostMessage(TopWindow, WM_SAVEFILE, 0, 0);
			break;

		case IDM_OPEN:
			OpenDialogBox(OpenFileFunc, this, buffer, 99, 
					"Text Files (*.txt)\0*.txt\0All Files (*.*)\0*.*\0\0", 1);
			break;
	};

	return 0;
}



HWND Simple::MakeWindow(){ // create new window on heap, which will clean itself up.
							//	see TextWindow::OnNCDestroy
	TextWindow* w = new TextWindow(hInstance);
	HWND hwnd2 = w->Create("TextWindow", WS_OVERLAPPEDWINDOW|WS_CHILD|WS_CLIPSIBLINGS,
							CW_USEDEFAULT, CW_USEDEFAULT, 300,
							200, NULL, NULL, hWnd);	//need to give it the parent HWND
											// because this is a child window
	w->Show();
	ShowCloseMenuItem(); // make sure that the close menu item is selectable

	return hwnd2;
}




void Simple::HideCloseMenuItem(){ // hid close option when there are no open windows

	HMENU hMenu = GetMenu(hWnd); // get our menu
	EnableMenuItem(hMenu, IDM_CLOSE, MF_GRAYED); // disable close menu item
}



void Simple::ShowCloseMenuItem(){ 

	HMENU hMenu = GetMenu(hWnd); // get our menu
	EnableMenuItem(hMenu, IDM_CLOSE, MF_ENABLED); // make it selectable
}


int Simple::OpenFileFunc(OPENFILENAME& _ofn, void* _SWnd){

	// got here, that means found a valid file to open
	HWND NewWindow = ((Simple*)_SWnd)->MakeWindow();
	::SendMessage(NewWindow, WM_OPENFILE, (WPARAM)&_ofn, 0);
	// must be 'Send' not 'Post' or _ofn might be out of scope here, 
	// while still needed in other thread
	return 1;
}

