#include "TextWindow.h"


LRESULT TextWindow::OnCreate(){

	myLayout = new Layout(hWnd); // give it it's parent
	myLayout->setup(hWnd, hInstance, NULL);

	return 0;
}


LRESULT TextWindow::OnNCDestroy(){

	hWnd = 0;	// do this so it's already nice and clean for destroying
	delete this; // nothing gets done after OnNCDestroy, so it's ok to do this here
			// but we must make sure that nothing gets even looked at for
			// this class after this line
	return 0;
}


LRESULT TextWindow::OnSize(WPARAM wParam, LPARAM lParam){

	myLayout->update();

	return 0;
}



LRESULT TextWindow::OnOther(UINT message, WPARAM wParam, LPARAM lParam){ 
							// messages sent from the parent window
	switch(message)
	{
		case WM_SAVEFILE:
			return OnSaveFile();
			break;

		case WM_OPENFILE:
			if (OnOpenFile((LPOPENFILENAME)wParam)==0) {
				MessageBox("error opening file", "File Error");
				PostMessage(WM_CLOSE, 0, 0);
			}
			break;
	};

	return 1; // nothing done, just say so (VERY IMPORTANT!!!)
				// this is so for the other important messages, 
				// that we don't need to touch, they can still get done
				// if we return 0 here, it is saying WE have done them!!
}


int TextWindow::OnSaveFile(){

	char buffer[100]={0};

	return SaveDialogBox(SaveFileFunc, this, buffer, 99, 
					"Text Files (*.txt)\0*.txt\0All Files (*.*)\0*.*\0\0", 1);
}

// actual File operations
int TextWindow::OnOpenFile(LPOPENFILENAME lpofn){ // do the actual open of file

	FILE* file = NULL;
	fpos_t length = 0; // Used to record the length of the file
	char* buff = NULL; // A character buffer, used to hold ALL the text loaded in

	file = fopen(lpofn->lpstrFile,"rb");  // DOES NOT create a new file if can't open one
	if(!file) return 0;

	// Seek to the end of the file
	fseek(file,0,SEEK_END); // find size of file
	fgetpos(file,&length);

	buff = new char[(int)length + 1]; // Allocate memory
	if(!buff) return 0;	// Error check
	
	fseek(file,0,SEEK_SET); // Move the file pointer back to the beginning of the file
	fread(buff,(int)length,1,file);	// Read in the ENTIRE file, into buff
	buff[(int)length] = NULL; 
	fclose(file);
	SetWindowText(myLayout->text_area, buff);
	delete[] buff; // Free up "buff"

	return 1; // good code
}


// actual File operations
int TextWindow::SaveFileFunc(OPENFILENAME& _ofn, void* _SWnd){ // do the actual save file

	HWND text_area = ((TextWindow*)_SWnd)->myLayout->text_area;
	FILE *file = NULL;

	int length=GetWindowTextLength(text_area);
	char* buf = new char[length+1];
	GetWindowText(text_area, buf, (length+1));

	file = fopen(_ofn.lpstrFile, "wb");
	fwrite(buf,sizeof(char),(length),file);
	fclose(file);
	delete[] buf;
	return 1;
}



