

#include "winglib/wgWindow.h"

/////////////////       wgSkin Class      //////////////////////


#ifndef _SKIN
#define _SKIN

	#define FULL_SKIN 0
	#define TILE_SKIN 1
///
/// Skin class is the base class to use when you desire to 'Skin' your window or control with a 
/// simple bitmap image, in either a tile or stretch manner.
/// To Use the Skin class, create a Window class (eg a SWindow, SDialog, or SControl), 
/// and also Inherit class Skin. Give the Skin class constructor the Image resource code 
/// located in the resource.h file usually, and set the Skin Type to either FULL_SKIN, or 
/// TILE_SKIN; as defined in WinSute.h:<br>
/// #define FULL_SKIN 0<br>
/// #define TILE_SKIN 1<br>
/// If the Skin Type is set to FULL_SKIN, the imagex and imagey dimensions will be ignored 
/// and the image will be set to the dimensions of the window.
/// If the Skin Type is set to TILE_SKIN, the imagex, and imagey dimensions will 
/// specify the size that the bitmap image should appear on the screen. For example, 
/// Specifying smaller dimensions will see the image tiled more times across the window; 
/// Specifying larger dimensions will see the image tiled LESS number of times across the window. 

class wgSkin : public virtual wgWinBase{

	public:
									// either FULL_SKIN or TILE_SKIN. 
									// imageX, imageY do nothing if wgSkinType==FULL_SKIN
		wgSkin(int IMAGE_RESOURCE_CODE, int _SkinType = FULL_SKIN, int imageX=0, int imageY=0);
		virtual ~wgSkin()=0;

		int LoadSkin();
	
	protected:

	// protected member functions

		void DrawSkin(HDC dc, HBITMAP _hwgSkinBitmap=NULL);

	// protected member variables

		HDC dcSkin;	//	skin device context

		HBITMAP hSkinBmp, hOldBmp; // skin bitmap and old device context bitmap

		int image; // resource code
		int SkinType;
		BITMAP _bitmap;	// bitmap structure to be associated with actual image
		char isInitialized;  // test and store if wgSkin has been loaded, only draw if true
		RECT WindowRect;
};


#endif // _SKIN




///////////// end of wgSkin class prototype  ////////////////////////

