#include <yvals.h>              // warning numbers get enabled in yvals.h 
#pragma warning(disable: 4786)  // identifier was truncated to 'number' characters in 
                                // the debug information
	// BUG: C4786 Warning Is Not Disabled with #pragma Warning
	// STATUS: Microsoft has confirmed this to be a bug in the Microsoft product. 
	// This warning can be ignored. This occured only in the <map> container.
#include <map>
#include "winglib/wgWinBase.h"


#ifndef _WINDOW_TIMER
#define _WINDOW_TIMER


class wgWindowTimer;

struct wgWindowTimerInfo{
	
	unsigned int CurrentElapsePeriod;
	unsigned int NextElapsePeriod;
	wgWindowTimer* AssociatedObject;
	unsigned int idEvent;
	int IsActive;

};


typedef std::map<unsigned int, wgWindowTimerInfo*> TimerMapType;



/// wgWindowTimer is a wrapper class for WM_TIMER call back win32 functionality.
/// It's aim is to make managing timed events as thread-safe as possible, orderly
/// easy to use. It tries to enforce 'sequentality' of the events. Eg, When one 
/// event occurs, another one will not occur until that one is processed. Not matter
/// how long it takes to process the first event.<br><br>
/// wgWindowTimer has one assumption. That is that it is used with a wg style window,
/// and that a valid window handle is obtained and stored within the member variable: HWND hWnd.

class wgWindowTimer : public virtual wgWinBase
{

	public:

			/// Constructor
		wgWindowTimer();

			/// virtual destructor
		virtual ~wgWindowTimer();

			/// Kills the existing timer identified by idEvent
		BOOL KillTimer(UINT idEvent);
			/// Sets the timer using a chosen identifier 'idEvent'.
			/// The timer event will trigger every 'ElapsePeriod' milliseconds.
		UINT SetTimer(UINT idEvent, UINT ElapsePeriod);

			/// OnwgWindowTimer is an empty virtual function that you may override
			/// to catch the timer events.
		virtual void OnwgWindowTimer(unsigned int idEvent){}

			/// If your window still has associated timers at OnClose, you should
			/// call this function to make a clean exit 
		int Cleanup(); 

	private:

	// private member functions

		static VOID CALLBACK TimerWindowProc(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime);

	// private member variables

		unsigned int CurrentElapsePeriod;
		unsigned int NextElapsePeriod;
		TimerMapType mTimerMap;
		HANDLE wgwgWindowTimerMutex;
		HANDLE wgwgWindowTimerEventMutex;
};



#endif //_WINDOW_TIMER

