#include "winglib/wgDialog.h"



//////////////// start of code to wgDialog base class


wgDialog::wgDialog(int DIALOG_RESOURCE_CODE, HWND hwndParent){

	parent = hwndParent;
	if (parent!=0)
		wgWinBase::hInstance = (HINSTANCE)GetWindowLong(parent, GWL_HINSTANCE);
	else wgWinBase::hInstance = (HINSTANCE)GetModuleHandle(NULL);
			
	dialog_resource_code = DIALOG_RESOURCE_CODE;
	resource = 0;
	wgWinBase::hWnd=0;
	type = _UNSPECIFIED;
}


wgDialog::wgDialog(int DIALOG_RESOURCE_CODE, HINSTANCE _hInstance, HWND hwndParent){

	wgWinBase::hInstance = _hInstance;
	parent = hwndParent;
	dialog_resource_code = DIALOG_RESOURCE_CODE;
	resource = 0;
	wgWinBase::hWnd=0;
	type = _UNSPECIFIED;
}


wgDialog::wgDialog(LPCTSTR lpTemplate, HINSTANCE _hInstance, HWND hwndParent){

	wgWinBase::hInstance = _hInstance;
	parent = hwndParent;
	dialog_resource_code = 0;
	resource = (void*)lpTemplate; // store before using, ok to do this
	wgWinBase::hWnd=0;
	type = _UNSPECIFIED;
}


// should take care of window, by sending close and destroy the proper way
// then unregister stuff
wgDialog::~wgDialog(){

	if (type!=_UNSPECIFIED && hWnd!=0) ::SendMessage(hWnd, WM_CLOSE,0, 0);
	// now do unregister stuff
}


unsigned int wgDialog::getType(){

	return (unsigned int)type;
}


// returns FALSE if NOT handled message
BOOL CALLBACK wgDialog::DialogProcedure(HWND _hWnd, UINT message, WPARAM wParam, LPARAM lParam){

	wgDialog* control = (wgDialog*)GetWindowLong (_hWnd, GWL_USERDATA);

	switch (message)
    {
	    case WM_INITDIALOG:
			SetWindowLong (_hWnd, GWL_USERDATA, (long)lParam);
			((wgDialog*)lParam)->_OnInitDialog(_hWnd);
			break;

		case WM_SHOWWINDOW:
			control->_OnShowWindow((BOOL)wParam , (int)lParam);
			break;

		case WM_SIZE:
			control->_OnSize(wParam, lParam);
			break;

		case WM_SIZING:
			control->OnSizing(wParam, lParam);
			break;

		case WM_MOVE:
			control->_OnMove(wParam, lParam);
			break;

		case WM_MOVING:
			control->OnMoving(wParam, lParam);
			break;			

	    case WM_COMMAND:
			return control->OnCommand(LOWORD(wParam), HIWORD (wParam));
			
	    case WM_NOTIFY:
			return control->OnNotify(wParam, (NMHDR*)lParam);
	
		case WM_PAINT:
			control->OnPaint();
			break;

		case WM_CLOSE: // this would be done before WM_DESTROY (example cause: hit the corner x box)
			control->_OnClose();
			break;

		case WM_DESTROY:
			control->_OnDestroy();
			break;

		case WM_NCDESTROY:
			control->OnNCDestroy();
			break;

		default: if (control!=0) return control->OnOther(message, wParam, lParam);
    };

    return FALSE;	// default message handler 
}



HWND wgDialog::Create(){

	if (type==_UNSPECIFIED){ // can't create the same object twice
		type = _MODELESS;
		hWnd = CreateDialogParam(hInstance, (LPCTSTR)MakeResource(), 
									parent, (DLGPROC)DialogProcedure, (LPARAM)this);

		return hWnd;
	}
	else return NULL;
}



int wgDialog::DoModal(){
	
	type = _MODAL;
	int temp = DialogBoxParam(hInstance, (LPCTSTR)MakeResource(), parent,
							(DLGPROC)DialogProcedure, (LPARAM)this);

	
	type = _UNSPECIFIED;
	return temp;
}




void wgDialog::_OnInitDialog(HWND _hWnd){

	hWnd = _hWnd;
	// here initialize the registry information
	RECT MyRect;
	GetWindowRect(hWnd, &MyRect);

	SWin_Width = MyRect.right - MyRect.left;
	SWin_Height= MyRect.bottom - MyRect.top;
	SWin_Left= MyRect.left;
	SWin_Top= MyRect.top;
	
	update_registry(hWnd);

	OnInitDialog();
}




void wgDialog::_OnSize(WPARAM wParam, LPARAM lParam){

	// set size info for this window
	SWin_Width = LOWORD(lParam);
	SWin_Height = HIWORD(lParam);

	OnSize(wParam, lParam);
}

void wgDialog::_OnMove(WPARAM wParam, LPARAM lParam){

	// set size info for this window
	SWin_Left = LOWORD(lParam);
	SWin_Top  = HIWORD(lParam);

	OnMove(wParam, lParam);
}


void wgDialog::_OnShowWindow(BOOL fShow, int fnStatus){

	RECT rect;
	GetClientRect(hWnd,  &rect);
	SWin_Width = rect.right - rect.left;
	SWin_Height = rect.bottom - rect.top;

	OnShowWindow(fShow, fnStatus);
}



void wgDialog::_OnClose(){
	// room to setup any MUST DO actions
	CloseWindow(OnClose());
}


void wgDialog::CloseWindow(int rcode){

	unregister_window(hWnd);

	if (type==_MODAL){
		EndDialog(hWnd, rcode);
	}
	else if (type == _MODELESS){
		DestroyWindow(hWnd);
	}
}



void wgDialog::_OnDestroy(){

	if (type==_MODELESS) type = _UNSPECIFIED;
	Cleanup();  // nothing more happens that user see's now, 
				// so cleanup for possible next use
	OnDestroy();
}




void wgDialog::OnDestroy(){ // override to control WM_DESTROY
	
	if (type!=_MODAL && GetNumOpenWindows()<1)
	{
		unregister_wgWinMaker();
		PostQuitMessage(0);
	}	
}




LPCTSTR wgDialog::MakeResource(){

	LPCTSTR TempResourceString = 0;

	if (dialog_resource_code==0)
	{
		return (LPCTSTR) resource;
	}
	else
	{
		TempResourceString = (LPCTSTR)MAKEINTRESOURCE(dialog_resource_code); 
		return TempResourceString;
	}
}



void wgDialog::Cleanup(){
// do NOT set to _UNSPECIFIED here!!

		// here i call this again, just in case it was bypassed before
	unregister_window(hWnd); // last use of hWnd before remove reference.
		// we should do this unregister here because of the chance of OnClose being
		// bypassed
}


BOOL wgDialog::Show(int cmd_show){

	if (type==_MODELESS && hWnd!=0) return ShowWindow(hWnd, cmd_show);
	else return FALSE;
}

		

BOOL wgDialog::Hide(){

	if (type==_MODELESS && hWnd!=0) return ShowWindow(hWnd, SW_HIDE);
	else return FALSE;
}



int wgDialog::setDlgItemText(int DlgItem, LPCTSTR lpText){

	if (hWnd==0) return SUTE_NO_SUCH_DIALOG_ERROR;
	HWND item = ::GetDlgItem(hWnd, DlgItem);
	if (item==0) return SUTE_BAD_RESOURCE_ERROR;
	if (SetWindowText(item, lpText)==0) return SUTE_UNDEFINED_ERROR;
	else return 0;
}


int wgDialog::DlgItemAppendText(int DlgItem, LPCTSTR lpText){

	if (hWnd==0) return SUTE_NO_SUCH_DIALOG_ERROR;
	else if (lpText==0) return SUTE_USER_STUPIDITY_ERROR;

	HWND item = ::GetDlgItem(hWnd, DlgItem);
	if (item==0) return SUTE_BAD_RESOURCE_ERROR;

	int len_first = GetWindowTextLength(item);
	int len_second = strlen(lpText);
	char* hbuff = new char[len_first + len_second+1];
	memset((void*)hbuff, 0, len_first + len_second+1);
	GetWindowText(item, hbuff, len_first);
	strcat(hbuff, (const char*)lpText);
	if (SetWindowText(item, hbuff)==0) return SUTE_UNDEFINED_ERROR;
	delete[] hbuff; // SetWindowText COPIES MEMORY, so no longer needed

	return 0;
}



HWND wgDialog::GetDlgItem(int nIDDlgItem){

	return ::GetDlgItem(hWnd, nIDDlgItem);
} 


/////////////// end of wgDialog class code


