#include "winglib/wgOleDispatchInfo.h"
#include <stdio.h>

// Using Array


using namespace std;


wgOleDispatchInfo::wgOleDispatchInfo(){

	miArgCount = 0;
	miNamedArgCount =0;
}



wgOleDispatchInfo::~wgOleDispatchInfo(){

	ClearAllArgs();
}




void wgOleDispatchInfo::ClearVariant(VARIANTARG *pvarg)
  // Zeros a variant structure without regard to current contents
{
	pvarg->vt = VT_EMPTY;
	pvarg->wReserved1 = 0;
	pvarg->wReserved2 = 0;
	pvarg->wReserved3 = 0;
	pvarg->lVal = 0;
}

void wgOleDispatchInfo::ReleaseVariant(VARIANTARG *pvarg)
  // Clears a particular variant structure and releases any external objects
  // or memory contained in the variant.  Supports the data types listed above.
{
	VARTYPE vt;
	VARIANTARG _huge *pvargArray;
	long lLBound, lUBound, l;
	
	vt = pvarg->vt & 0xfff;		// mask off flags
	
	// check if an array.  If so, free its contents, then the array itself.
	if (V_ISARRAY(pvarg)) 
	{
		// variant arrays are all this routine currently knows about.  Since a
		// variant can contain anything (even other arrays), call ourselves
		// recursively.
		if (vt == VT_VARIANT) 
		{
			SafeArrayGetLBound(pvarg->parray, 1, &lLBound);
			SafeArrayGetUBound(pvarg->parray, 1, &lUBound);
			
			if (lUBound > lLBound) 
			{
				lUBound -= lLBound;
				
				SafeArrayAccessData(pvarg->parray, (void**)&pvargArray);
				
				for (l = 0; l < lUBound; l++) 
				{
					ReleaseVariant(pvargArray);
					pvargArray++;
				}
				
				SafeArrayUnaccessData(pvarg->parray);
			}
		}
		else 
		{
			// ErrorMessage("ReleaseVariant: Array contains non-variant type");
		}
		
		// Free the array itself.
		SafeArrayDestroy(pvarg->parray);
	}
	else 
	{
		switch (vt) 
		{
			case VT_DISPATCH:
				pvarg->pdispVal->Release();
				break;
				
			case VT_BSTR:
				SysFreeString(pvarg->bstrVal);
				break;
				
			case VT_I2:
			case VT_BOOL:
			case VT_R8:
			case VT_ERROR:
			case VT_EMPTY:	// to avoid erroring on an error return from Excel
				// no work for these types
				break;
				
			default:
			//	ErrorMessage("ReleaseVariant: Unknown type");
				break;
		}
	}
	
	ClearVariant(pvarg);
}



void wgOleDispatchInfo::ClearAllArgs()
  // Clears the existing contents of the arg array in preparation for
  // a new invocation.  Frees argument memory if so marked.
{
	int i;
	
	for (i = 0; i < miArgCount; i++) 
	{
		if (mawFlags[i] & DISPARG_NOFREEVARIANT)
			// free the variant's contents based on type
			ClearVariant(&maVargs[i]);
		else
			ReleaseVariant(&maVargs[i]);
	}

	miArgCount = 0;
	miNamedArgCount = 0;
}


/*******************************************************************
 *
 *					   ARGUMENT CONSTRUCTOR FUNCTIONS
 *
 *  Each function adds a single argument of a specific type to the list
 *  of arguments for the current invoke.  If appropriate, memory may be
 *  allocated to represent the argument.  This memory will be
 *  automatically freed the next time ClearAllArgs() is called unless
 *  the NOFREEVARIANT flag is specified for a particular argument.  If
 *  NOFREEVARIANT is specified it is the responsibility of the caller
 *  to free the memory allocated for or contained within the argument.
 *
 *  Arguments may be named.  The name string must be a C-style string
 *  and it is owned by the caller.  If dynamically allocated, the caller
 *  must free the name string.
 *
 *******************************************************************/

void wgOleDispatchInfo::AddArgumentCommon(	LPOLESTR	lpszArgName, 
								WORD		wFlags, 
								VARTYPE		vt)
  // Common code used by all variant types for setting up an argument.
{
	ClearVariant(&maVargs[miArgCount]);
	
	maVargs[miArgCount].vt = vt;
	mawFlags[miArgCount] = wFlags;
	
	if (lpszArgName != NULL) 
	{
		malpszArgNames[miNamedArgCount + 1] = lpszArgName;
		miNamedArgCount++;
	}
}	
	

bool wgOleDispatchInfo::AddArgumentDispatch(LPOLESTR	lpszArgName, 
								WORD		wFlags, 
								IDispatch*	pdisp)
{
	AddArgumentCommon(lpszArgName, wFlags, VT_DISPATCH);
	maVargs[miArgCount++].pdispVal = pdisp;
	return true;
}


bool wgOleDispatchInfo::AddArgumentInt2(LPOLESTR	lpszArgName, 
							WORD		wFlags, 
							int			i)
{
	AddArgumentCommon(lpszArgName, wFlags, VT_I2);
	maVargs[miArgCount++].iVal = i;
	return true;
}


bool wgOleDispatchInfo::AddArgumentInt4(LPOLESTR	lpszArgName, 
							WORD		wFlags, 
							int			i)
{
	AddArgumentCommon(lpszArgName, wFlags, VT_I4);
	maVargs[miArgCount++].iVal = i;
	return true;
}


bool wgOleDispatchInfo::AddArgumentBool(LPOLESTR	lpszArgName, 
							WORD		wFlags, 
							BOOL		b)
{
	AddArgumentCommon(lpszArgName, wFlags, VT_BOOL);
	// Note the variant representation of True as -1
	maVargs[miArgCount++].boolVal = b ? -1 : 0;
	return true;
}

bool wgOleDispatchInfo::AddArgumentDouble(LPOLESTR	lpszArgName, 
							  WORD		wFlags, 
							  double	d)
{
	AddArgumentCommon(lpszArgName, wFlags, VT_R8);
	maVargs[miArgCount++].dblVal = d;
	return true;
}

bool wgOleDispatchInfo::AddArgumentError(	LPOLESTR	lpszArgName,
								WORD		wFlags, 
								SCODE       enuError)
  // create an variant with an error code - for skipping optional arguments in 
  // Invoke, use this function with error code DISP_E_PARAMNOTFOUND
{
	AddArgumentCommon(lpszArgName, 0, VT_ERROR);
	maVargs[miArgCount++].scode = enuError;
	return true;
}



bool wgOleDispatchInfo::AddArgumentVariant(	LPOLESTR	lpszArgName, 
								WORD		wFlags, 
								VARIANT&	lpsz)

{
	AddArgumentCommon(lpszArgName, 0, VT_BYREF|VT_VARIANT);

	maVargs[miArgCount++].pvarVal = &lpsz;
	return true;
}





bool wgOleDispatchInfo::AddArgumentSafeArray(	LPOLESTR	lpszArgName, 
												WORD		wFlags, 
												wgSafeArray&	lpsz)

{
	AddArgumentCommon(lpszArgName, 0, VT_ARRAY|VT_VARIANT);

	maVargs[miArgCount++].pvarVal = (VARIANT*)&lpsz;
	return true;
}






bool wgOleDispatchInfo::AddArgumentCString(	LPOLESTR	lpszArgName, 
								WORD		wFlags, 
								LPOLESTR	lpsz)
  // OLE and IDispatch use a BSTR as the representation of strings.
  // This constructor automatically copies the passed-in C-style string
  // into a BSTR.  It is important to not set the NOFREEVARIANT flag
  // for this function, otherwise the allocated BSTR copy will probably
  // get lost and cause a memory leak.
{
	BSTR b;
	
	b = SysAllocString(lpsz);
	if (!b)
		return FALSE;
	AddArgumentCommon(lpszArgName, wFlags, VT_BSTR);
	maVargs[miArgCount++].bstrVal = b;
	return true;
}

bool wgOleDispatchInfo::AddArgumentBSTR(LPOLESTR	lpszArgName, 
							WORD		wFlags, 
							BSTR		bstr)
  // BSTRs are the OLE and IDispatch representation of strings.  If you
  // already happen to have a BSTR, use this argument constructor
  // to avoid an extra string copy.  Be sure to use the NOFREEVARIANT flag
  // if you want to use the BSTR following the Invoke call with this argument.
{
	AddArgumentCommon(lpszArgName, wFlags, VT_BSTR);
	maVargs[miArgCount++].bstrVal = bstr;
	return true;
}


bool wgOleDispatchInfo::AddArgumentCStringArray(LPOLESTR	lpszArgName, 
									WORD		wFlags, 
									LPOLESTR*	paszStrings, 
									int			iCount)
  // Constructs an 1-dimensional array containing variant strings.  The strings
  // are copied from an incoming array of C-Strings.
{
	SAFEARRAY *psa;
	SAFEARRAYBOUND saBound;
	VARIANTARG *pvargBase;
	VARIANTARG *pvarg;
	int i, j;
	
	saBound.lLbound = 0;
	saBound.cElements = iCount;
	
	psa = SafeArrayCreate(VT_VARIANT, 1, &saBound);
	if (psa == NULL)
		return false;
	
	SafeArrayAccessData(psa, (void**)&(VARIANTARG _huge *) pvargBase);
	
	pvarg = pvargBase;
	for (i = 0; i < iCount; i++) 
	{
		// copy each string in the list of strings
		ClearVariant(pvarg);
		pvarg->vt = VT_BSTR;
		if ((pvarg->bstrVal = SysAllocString(*paszStrings++)) == NULL) 
		{
			// memory failure:  back out and free strings alloc'ed up to
			// now, and then the array itself.
			pvarg = pvargBase;
			for (j = 0; j < i; j++) 
			{
				SysFreeString(pvarg->bstrVal);
				pvarg++;
			}
			SafeArrayDestroy(psa);
			return false;
		}
		pvarg++;
	}
	
	SafeArrayUnaccessData(psa);

	// With all memory allocated, setup this argument
	AddArgumentCommon(lpszArgName, wFlags, VT_VARIANT | VT_ARRAY);
	maVargs[miArgCount++].parray = psa;
	return true;
}





string wgOleDispatchInfo::WideCharStringToString(wchar_t strOriginal[])
  // convert the specified wide character string into a regular string and return it
{
	int		intChar=0;
	string	strReturn;

	  // convert the wide-character string to a single-byte string
	while (strOriginal[intChar]!='\0')
		strReturn += (char)strOriginal[intChar++];

	return (strReturn);
}

string wgOleDispatchInfo::BSTRToString(BSTR strOriginal)
  // convert a BSTR into a string. A BSTR is a wide-character, length-prefixed string
{
	int		intChar=0;
	string	strReturn;

	if (strOriginal!=NULL)
	{
		long lngLength = SysStringLen(strOriginal);

		  // convert the member to a character string
		for (intChar=0; intChar<lngLength; intChar++)
			strReturn += (char)strOriginal[intChar];
	}
	return (strReturn);
}

  




// convert the specified value into a wide character string and return it in the buffer
void wgOleDispatchInfo::WideCharacterString(	wchar_t strBuffer[], 
										int intBufferSize, 
										const string& strValue){

	unsigned int uintChar=0;

	for (uintChar=0; uintChar<strValue.size() && (int)uintChar<(intBufferSize-1); uintChar++)
		strBuffer[uintChar] = strValue[uintChar];

	strBuffer[uintChar]='\0';
}




std::string wgOleDispatchInfo::GetValueAsString(VARIANT* vRet)
{

	char strTemp[1024]={0};

	if (vRet->vt==VT_UI1) sprintf(strTemp,"%c", vRet->bVal);
	else if (vRet->vt==VT_I2) sprintf(strTemp,"%d", vRet->iVal);
	else if (vRet->vt==VT_I4) sprintf(strTemp,"%ld", vRet->lVal);
	else if (vRet->vt==VT_R4) sprintf(strTemp,"%f", vRet->fltVal);
	else if (vRet->vt==VT_R8) sprintf(strTemp,"%f", vRet->dblVal);
	else if (vRet->vt==VT_DATE) sprintf(strTemp,"%f", vRet->date);
	else if (vRet->vt==VT_BOOL) sprintf(strTemp,"%d", vRet->boolVal);
	else if (vRet->vt==VT_BSTR) strcpy(strTemp, (wgOleDispatchInfo::BSTRToString(vRet->bstrVal).c_str()));
	//else if (vRet->vt==VT_VARIANT|VT_ARRAY) ;
	else strcpy(strTemp, "UNCONVERTABLE VALUE");

	return string(strTemp);
}



void wgOleDispatchInfo::TestValue(){



}



/*
		int			miArgCount;
		int			miNamedArgCount;
		VARIANTARG	maVargs[MAX_DISP_ARGS];
		LPOLESTR	malpszArgNames[MAX_DISP_ARGS + 1];	// used to hold the argnames for GetIDs
		WORD		mawFlags[MAX_DISP_ARGS];

*/