#include "winglib/wgSafeArray.h"
#include <winutils/wuException.h>
#include <winutils/wuArrayException.h>
#include <cstdarg>
#include "winglib/wgOleDispatchInfo.h"


#define _MAX_STRINGSIZE 1024



using namespace std;

wgSafeArray::wgSafeArray()
{
	ClearVariant(this);
}


wgSafeArray::wgSafeArray(SAFEARRAY* pSafeArrayBaseType)
{
	ClearVariant(this);
	VariantInit(this);		// initialise the refreshed variant

	this->parray = pSafeArrayBaseType;
	this->vt = VT_ARRAY;
}



wgSafeArray::wgSafeArray(VARTYPE  vt, unsigned int uintRows)
{
	Initialise(vt, uintRows, 0);
}



wgSafeArray::wgSafeArray(VARTYPE  vt, unsigned int uintRows, unsigned int uintColumns)
{
	Initialise(vt, uintRows, uintColumns, 0);
}




wgSafeArray::wgSafeArray(VARTYPE  vt, unsigned int uintRows, unsigned int uintColumns, unsigned int uintOutPage)
{
	Initialise(vt, uintRows, uintColumns, uintOutPage, 0);
}




wgSafeArray::wgSafeArray(VARTYPE  vt, unsigned int uintRows, unsigned int uintColumns, unsigned int uintOutPage, unsigned int uintFourthDimension)
{
	Initialise(vt, uintRows, uintColumns, uintOutPage, uintFourthDimension, 0);
}




wgSafeArray::wgSafeArray(const VARIANT& varSafeArray)
{
	ClearVariant(this);

	HRESULT hr = VariantCopy((VARIANTARG*)this, (VARIANTARG*)&varSafeArray);

	if (hr==DISP_E_BADINDEX) throw wuArrayException("Bad Index Specified", hr);
	else if (hr==E_INVALIDARG) throw wuArrayException("Invalid arg", hr);
	else if (hr==E_OUTOFMEMORY) throw wuArrayException("Out of Memory error", hr);
	else if (hr!=S_OK) throw wuException("Unknown Array Exception");
}




wgSafeArray::wgSafeArray(const wgSafeArray* other) throw()
{
	ClearVariant(this);

	HRESULT hr = VariantCopy((VARIANTARG*)this, (VARIANTARG*)&other);

	if (hr==DISP_E_BADINDEX) throw wuArrayException("Bad Index Specified", hr);
	else if (hr==E_INVALIDARG) throw wuArrayException("Invalid arg", hr);
	else if (hr==E_OUTOFMEMORY) throw wuArrayException("Out of Memory error", hr);
	else if (hr!=S_OK) throw wuException("Unknown Array Exception");
}





wgSafeArray::~wgSafeArray()
{
	ReleaseVariant(this);
}




/*
VARIANTARG* wgSafeArray::XloperToVariant(VARIANTARG* ppvDatam, XLOPER* other) throw()
{

	switch(other->xltype)
	{
		case xltypeNum:
			SetVariantDouble(*ppvDatam, other->val.num);
			break;

		case xltypeStr:
			SetVariantXloperString(*ppvDatam, other->val.str);
			break;

		case xltypeBool:
			SetVariantBool(*ppvDatam, other->val.bool);
			break;

		case xltypeInt:
			SetVariantInt2(*ppvDatam, other->val.w);
			break;
	};

	return ppvDatam;
}

*/





// this should handle the possibility of a single value type as well! but doesn't yet. Fix it.
// should copy from one array to another array of any size
void wgSafeArray::SetArray(const VARIANTARG* other) throw()
{
	HRESULT hr = 0, hr2 = 0;
	VARIANTARG HUGEP FAR*  ppvDatao=0;  
	VARIANTARG HUGEP FAR*  ppvDatam=0;

	long LowerBoundOther =0;
	long LowerBoundMe = 0;
	long UpperBoundOther = 0;
	long UpperBoundMe = 0;
	long ElementsOther = 1;	// start this at 1 because we multiply out later
	long ElementsMe = 1;	// start this at 1 because we multiply out later
	unsigned int uintCount = 0;
	unsigned int uintSizeOther =0;
	unsigned int uintDimsOther =0;

	unsigned int uintSizeMe = SafeArrayGetElemsize(this->parray);
	unsigned int uintDimsMe	= SafeArrayGetDim(this->parray);

	hr = SafeArrayAccessData(this->parray, (void HUGEP* FAR*)&ppvDatam);	// Access My private data

	if (V_ISARRAY(other))
	{
		uintSizeOther = SafeArrayGetElemsize(other->parray);
		uintDimsOther =	SafeArrayGetDim(other->parray);

		if (uintSizeOther!=uintSizeMe) throw wuArrayException("Element Size Differs", 0);

		hr = SafeArrayAccessData(other->parray, (void HUGEP* FAR*)&ppvDatao);		// if it's an array, access it's private data

		for (uintCount = 0;uintCount<uintDimsOther;uintCount++){

			hr2 = SafeArrayGetLBound(other->parray, uintCount+1, &LowerBoundOther);
			hr2 = SafeArrayGetUBound(other->parray, uintCount+1, &UpperBoundOther);
			ElementsOther *= (UpperBoundOther - LowerBoundOther +1);
		}

		for (uintCount = 0;uintCount<uintDimsMe;uintCount++){

			hr2 = SafeArrayGetLBound(this->parray, uintCount+1, &LowerBoundMe);
			hr2 = SafeArrayGetUBound(this->parray, uintCount+1, &UpperBoundMe);
			ElementsMe *= (UpperBoundMe - LowerBoundMe +1);
		}

		unsigned int MaxElements = (ElementsMe>ElementsOther)?ElementsOther:ElementsMe;

		if (hr==S_OK){

			for (uintCount = 0;uintCount<MaxElements;uintCount++){

				VariantCopy(ppvDatam++, ppvDatao++);
			}
		}
		
		hr =	SafeArrayUnaccessData(other->parray);
	}	// end of "if (V_ISARRAY(&other)"
	else
	{
		VariantCopy((VARIANTARG*)ppvDatam, (VARIANT*)other);
	}

	hr = SafeArrayUnaccessData(this->parray);

	if (hr==DISP_E_BADINDEX) throw wuArrayException("Bad Index Specified", hr);
	else if (hr==E_INVALIDARG) throw wuArrayException("Invalid arg", hr);
	else if (hr==E_OUTOFMEMORY) throw wuArrayException("Out of Memory error", hr);
	else if (hr!=S_OK) throw wuException("Unknown Array Exception");
}





 

// MAX DIMENSIONS 
// Dimension1 = Row; Dimension2 = Column
void wgSafeArray::Initialise(VARTYPE  vtArg, unsigned int uintDim1, ...)
{
	va_list ap;
	va_start(ap, uintDim1);
	ClearVariant(this);
//	ReleaseVariant(this);	// make sure that any existing memory is cleaned up
	VariantInit(this);		// initialise the refreshed variant

	int intVarArgs = 0;
	int intDimCounter = 1;
	int intCounter = 0;

	// first find out how many dimensions
	for (intCounter = 0;;intCounter++)
	{
		intVarArgs = va_arg(ap, unsigned int);		
		if (intVarArgs==0) break;	
		intDimCounter++;	// increment the count of dimensions	
	}

	va_end(ap);

	intVarArgs = 0;
	va_start(ap, uintDim1);

	// Create pSafeBound array. Envision each bond holds one type, therefore
	// in a spreadsheet situation, each column has one type, so we say each bound is a column
	SAFEARRAYBOUND* pSafeBound = new SAFEARRAYBOUND[intDimCounter];

	intVarArgs = uintDim1;

	for (intCounter = 0;intCounter<intDimCounter;intCounter++)
	{
		pSafeBound[intCounter].cElements = intVarArgs;    
		pSafeBound[intCounter].lLbound = 0;

		intVarArgs = va_arg(ap, unsigned int);
	}

	if ( (vtArg|VT_ARRAY)==vtArg) vtArg = vtArg^VT_ARRAY;	// delete the VT_ARRAY
	if ( (vtArg|VT_BYREF )==vtArg) vtArg = vtArg^VT_BYREF ;	// delete the VT_BYREF 

	this->parray = SafeArrayCreate(vtArg, intDimCounter, pSafeBound); // the intDimCounter signifies 2 dimensions sort of 
	this->vt = vtArg | VT_ARRAY;

	// Cleanup	
	delete[] pSafeBound;
	va_end(ap);

	if (this->parray==NULL) throw wuArrayException("Array Could not be created", 0);
}





// Dimension1 = Row; Dimension2 = Column
void wgSafeArray::AddElement(void FAR* pv, unsigned int uintDim1, ...) throw(){

	va_list ap;
	va_start(ap, uintDim1);
	int intVarArgs = 0;

	const int intMaxDimensions = SafeArrayGetDim(this->parray);

	long* lDimension = new long[intMaxDimensions];

	intVarArgs = uintDim1;

	for (int intCounter=0;intCounter<intMaxDimensions;intCounter++)
	{
		lDimension[intCounter]=intVarArgs;

		intVarArgs = va_arg(ap, unsigned int);	
	}

	HRESULT hr = SafeArrayPutElement(this->parray,lDimension,pv);
	delete[] lDimension;
	va_end(ap);

	if (hr==DISP_E_BADINDEX) throw wuArrayException("Bad Index Specified", hr);
	else if (hr==E_INVALIDARG) throw wuArrayException("Invalid arg", hr);
	else if (hr==E_OUTOFMEMORY) throw wuArrayException("Out of Memory error", hr);
	else if (hr!=S_OK) throw wuException("Unknown Array Exception");
}






void wgSafeArray::GetElement(void FAR* pv, unsigned int uintDim1, ...) const throw()
{

	va_list ap;
	va_start(ap, uintDim1);
	int intVarArgs = 0;

	const int intMaxDimensions = SafeArrayGetDim(this->parray);

	long* lDimension = new long[intMaxDimensions];

	intVarArgs = uintDim1;

	for (int intCounter=0;intCounter<intMaxDimensions;intCounter++)
	{
		lDimension[intCounter]=intVarArgs;

		intVarArgs = va_arg(ap, unsigned int);	
	}

	HRESULT hr = SafeArrayGetElement(this->parray, lDimension, pv);
	delete[] lDimension;
	va_end(ap);

	if (hr==DISP_E_BADINDEX) throw wuArrayException("Bad Index Specified", hr);
	else if (hr==E_INVALIDARG) throw wuArrayException("Invalid arg", hr);
	else if (hr==E_OUTOFMEMORY) throw wuArrayException("Out of Memory error", hr);
	else if (hr!=S_OK) throw wuException("Unknown Array Exception");
}


string wgSafeArray::GetElementAsString(unsigned int uintDim1, ...) const throw()
{
	va_list ap;
	va_start(ap, uintDim1);
	int intVarArgs = 0;
	VARIANT pv;

	const int intMaxDimensions = SafeArrayGetDim(this->parray);

	long* lDimension = new long[intMaxDimensions];

	intVarArgs = uintDim1;

	for (int intCounter=0;intCounter<intMaxDimensions;intCounter++)
	{
		lDimension[intCounter]=intVarArgs;

		intVarArgs = va_arg(ap, unsigned int);	
	}

	HRESULT hr = SafeArrayGetElement(this->parray, lDimension, &pv);
	delete[] lDimension;
	va_end(ap);

	if (hr==DISP_E_BADINDEX) throw wuArrayException("Bad Index Specified", hr);
	else if (hr==E_INVALIDARG) throw wuArrayException("Invalid arg", hr);
	else if (hr==E_OUTOFMEMORY) throw wuArrayException("Out of Memory error", hr);
	else if (hr!=S_OK) throw wuException("Unknown Array Exception");

	return wgOleDispatchInfo::GetValueAsString(&pv);
}



int wgSafeArray::GetNumberOfDimensions() const throw()
{
	try
	{

		if (V_ISARRAY((VARIANTARG*)this)) return SafeArrayGetDim(this->parray);
		else return 0;
	}
	catch(...)
	{
		throw exception("Unknown Exception in GetNumberOfDimension");
	}
}





void wgSafeArray::GetDimensionSizes(unsigned int puintDimSizes[]) const throw()
{

	try
	{
		if (V_ISARRAY((VARIANTARG*)this))
		{

			int intCount = 0;
			long lngLowerBound = 0;
			long lngUpperBound = 0;
			int NumOfDimensions = GetNumberOfDimensions();
			HRESULT hr = 0;

			for (intCount = 0;intCount<NumOfDimensions;intCount++)
			{
				hr = SafeArrayGetLBound(this->parray, intCount+1, &lngLowerBound);
				hr = SafeArrayGetUBound(this->parray, intCount+1, &lngUpperBound);
				puintDimSizes[intCount] = (lngUpperBound - lngLowerBound +1);
			}
		}
		else throw exception("wgSafeArray not holding an Array");
	}
	catch(exception& e)
	{
		throw exception(e.what());
	}
	catch(...)
	{
		throw exception("Unknown Error at GetDimensionSizes");
	}
}


	/// This function is only useful if you have an extra handle to the SAFEARRAY and 
	/// only want to use that without this object deleting it when it reaches the end of it's scope, 
	/// or when the delete on this object is called. 
void wgSafeArray::ZeroWithoutRelease()
{
	wgSafeArray::ClearVariant(this);
	VariantInit(this);
}



	/// Returns the member parray.
SAFEARRAY* wgSafeArray::GetSafeArrayHandle()
{
	return ((VARIANT*)this)->parray;
}





//////////// UTILITIES //////////////////




  // Zeros a variant structure without regard to current contents
void wgSafeArray::ClearVariant(VARIANTARG *pvarg)
{
	pvarg->vt = VT_EMPTY;
	pvarg->wReserved1 = 0;
	pvarg->wReserved2 = 0;
	pvarg->wReserved3 = 0;
	pvarg->lVal = 0;
	pvarg->parray = 0;
}





  // Clears a particular variant structure and releases any external objects
  // or memory contained in the variant.  Supports the data types listed above.
void wgSafeArray::ReleaseVariant(VARIANTARG *pvarg)
{
	VARTYPE vt;
	VARIANTARG _huge *pvargArray;
	long lLBound, lUBound, l;
	
	vt = pvarg->vt & 0xfff;		// mask off flags
	
	// check if an array.  If so, free its contents, then the array itself.
	if (V_ISARRAY(pvarg)) 
	{
		// variant arrays are all this routine currently knows about.  Since a
		// variant can contain anything (even other arrays), call ourselves
		// recursively.
		if (vt == VT_VARIANT) 
		{
			SafeArrayGetLBound(pvarg->parray, 1, &lLBound);
			SafeArrayGetUBound(pvarg->parray, 1, &lUBound);
			
			if (lUBound > lLBound) 
			{
				lUBound -= lLBound;
				
				SafeArrayAccessData(pvarg->parray, (void**)&pvargArray);
				
				for (l = 0; l < lUBound; l++) 
				{
					ReleaseVariant(pvargArray); // temporarily commented out
					pvargArray++;
				} 
				
				SafeArrayUnaccessData(pvarg->parray);
			}
		}
		else 
		{
			// ErrorMessage("ReleaseVariant: Array contains non-variant type");
		}
		
		// Free the array itself.
		SafeArrayDestroy(pvarg->parray);
	}
	else 
	{
		switch (vt) 
		{
			case VT_DISPATCH:
				pvarg->pdispVal->Release();
				break;
				
			case VT_BSTR:
				SysFreeString(pvarg->bstrVal);
				break;
				
			case VT_I2:
			case VT_BOOL:
			case VT_R8:
			case VT_ERROR:
			case VT_EMPTY:	// to avoid erroring on an error return from Excel
				// no work for these types
				break;
				
			default:
			//	ErrorMessage("ReleaseVariant: Unknown type");
				break;
		}
	}
	
	ClearVariant(pvarg);
}



VARIANTARG* wgSafeArray::SetVariantDispatch(VARIANTARG &Var ,IDispatch* pdisp)
{
	ClearVariant(&Var);
	VariantInit(&Var);		// initialise the refreshed variant

	Var.vt = VT_DISPATCH;
	Var.pdispVal = pdisp;
	return &Var;
}



VARIANTARG* wgSafeArray::SetVariantInt2(VARIANTARG &Var, int i)
{
	ClearVariant(&Var);
	VariantInit(&Var);		// initialise the refreshed variant

	Var.vt = VT_I2;
	Var.iVal = i;
	return &Var;
}



VARIANTARG* wgSafeArray::SetVariantInt4(VARIANTARG &Var, int i)
{
	ClearVariant(&Var);
	VariantInit(&Var);		// initialise the refreshed variant

	Var.vt = VT_I4;
	Var.lVal = i;
	return &Var;
}




VARIANTARG* wgSafeArray::SetVariantBool(VARIANTARG &Var, BOOL	b)
{	
	ClearVariant(&Var);
	VariantInit(&Var);		// initialise the refreshed variant

	Var.vt = VT_BOOL;
	// Note the variant representation of True as -1
	Var.boolVal = b ? -1 : 0;
	return &Var;
}





VARIANTARG* wgSafeArray::SetVariantDouble(VARIANTARG &Var, double	d)
{
	ClearVariant(&Var);
	VariantInit(&Var);		// initialise the refreshed variant

	Var.vt = VT_R8;
	Var.dblVal = d;
	return &Var;
}



  // create an variant with an error code - for skipping optional arguments in 
  // Invoke, use this function with error code DISP_E_PARAMNOTFOUND
VARIANTARG* wgSafeArray::SetVariantError(VARIANTARG &Var,	SCODE enuError)
{
	ClearVariant(&Var);
	VariantInit(&Var);		// initialise the refreshed variant

	Var.vt = VT_ERROR;
	Var.scode = enuError;
	return &Var;
}



VARIANTARG* wgSafeArray::SetVariantVariant(VARIANTARG &Var, VARIANT& lpsz)

{
	ClearVariant(&Var);
	VariantInit(&Var);		// initialise the refreshed variant

	Var.vt = VT_BYREF|VT_VARIANT;

	Var.pvarVal = &lpsz;
	return &Var;
}



  // OLE and IDispatch use a BSTR as the representation of strings.
  // This constructor automatically copies the passed-in C-style string
  // into a BSTR.  It is important to not set the NOFREEVARIANT flag
  // for this function, otherwise the allocated BSTR copy will probably
  // get lost and cause a memory leak.
VARIANTARG* wgSafeArray::SetVariantCString(VARIANTARG &Var, const char* astring)
{
	ClearVariant(&Var);
	VariantInit(&Var);		// initialise the refreshed variant

	wchar_t 	strBuffer[_MAX_STRINGSIZE];

	// convert the argument to a wide character string
	WideCharacterString(strBuffer, _MAX_STRINGSIZE, astring);
	
	BSTR b;
	
	b = SysAllocString(strBuffer);
	if (!b) return FALSE;
	Var.vt = VT_BSTR;
	Var.bstrVal = b;
	return &Var;
}


/*
VARIANTARG* wgSafeArray::SetVariantXloperString(VARIANTARG &Var, LPSTR astring)
{
	ClearVariant(&Var);
	VariantInit(&Var);		// initialise the refreshed variant

	wchar_t 	strBuffer[_MAX_STRINGSIZE]={0};
	char cstrTemp[_MAX_STRINGSIZE]={0};

	strncpy(cstrTemp, (astring+1), astring[0]);
	cstrTemp[astring[0]] = '\0';

	// convert the argument to a wide character string
	WideCharacterString(strBuffer, _MAX_STRINGSIZE, cstrTemp);
	
	BSTR b;
	
	b = SysAllocString(strBuffer);
	if (!b) return FALSE;
	Var.vt = VT_BSTR;
	Var.bstrVal = b;
	return &Var;
}
*/




VARIANTARG* wgSafeArray::SetVariantBSTR(VARIANTARG &Var, BSTR bstr)
  // BSTRs are the OLE and IDispatch representation of strings.  If you
  // already happen to have a BSTR, use this argument constructor
  // to avoid an extra string copy.  Be sure to use the NOFREEVARIANT flag
  // if you want to use the BSTR following the Invoke call with this argument.
{
	ClearVariant(&Var);
	VariantInit(&Var);		// initialise the refreshed variant

	Var.vt = VT_BSTR;
	Var.bstrVal = bstr;
	return &Var;
}




  // convert a BSTR into a string. A BSTR is a wide-character, length-prefixed string
string wgSafeArray::BSTRToString(BSTR strOriginal)
{
	int		intChar=0;
	string	strReturn;

	if (strOriginal!=NULL)
	{
		long lngLength = SysStringLen(strOriginal);

		  // convert the member to a character string
		for (intChar=0; intChar<lngLength; intChar++)
			strReturn += (char)strOriginal[intChar];
	}
	return (strReturn);
}





// convert the specified value into a wide character string and return it in the buffer
void wgSafeArray::WideCharacterString(	wchar_t strBuffer[], 
										int intBufferSize, 
										const char* pcharValue){

	unsigned int	uintChar=0;
	unsigned int uintPCharLen = strlen(pcharValue);


	for (uintChar=0; uintChar<uintPCharLen && (int)uintChar<(intBufferSize-1); uintChar++)
		strBuffer[uintChar] = pcharValue[uintChar];

	strBuffer[uintChar]='\0';
}




