#include "winglib/wgSkin.h"


///////////////// SKIN IMPLEMENTATION ///////////////////////////////


wgSkin::wgSkin(int IMAGE_RESOURCE_CODE, int _SkinType, int _imageX, int _imageY){

	SkinType = _SkinType;
	isInitialized = 0;
	image  = IMAGE_RESOURCE_CODE;
	dcSkin = CreateCompatibleDC(0);// create a device context for the skin
	
	if (SkinType==FULL_SKIN){

		_bitmap.bmWidth = 0;	// use actual bitmap resource width
		_bitmap.bmHeight= 0;	// use actual bitmap resource height
		SetStretchBltMode(dcSkin, COLORONCOLOR); 
	}
	else {
		_bitmap.bmWidth = _imageX;	// set to users wishes if tile
		_bitmap.bmHeight = _imageY;	// set to users wishes if tile
	}
}





wgSkin::~wgSkin(){	
	
	DeleteObject(hSkinBmp);
	DeleteDC(dcSkin);
}





// return zero on good, negative on bad
int wgSkin::LoadSkin(){
	
	if (isInitialized) return 0; // only LoadwgSkin if not already loaded

	int fail=0;

	hSkinBmp  = (HBITMAP)LoadImage(hInstance, MAKEINTRESOURCE(image), IMAGE_BITMAP, _bitmap.bmWidth, _bitmap.bmHeight, LR_VGACOLOR);  
	fail = GetObject(hSkinBmp, sizeof(BITMAP), (LPVOID)&_bitmap);
	
	if (fail==0 || hSkinBmp==NULL) return -1;
	
	isInitialized = 1;
	
	GetWindowRect(hWnd, &WindowRect);

	return 0;
}




void wgSkin::DrawSkin(HDC hdc, HBITMAP _hSkinBitmap){

	if (!isInitialized) return;	// don't try to draw if not initialized
	
	if (_hSkinBitmap==NULL) hOldBmp = (HBITMAP)SelectObject(dcSkin, hSkinBmp); // store default bitmap while using other
	else hOldBmp = (HBITMAP)SelectObject(dcSkin, _hSkinBitmap); // store default bitmap while using other

	// make the clip region 
	HRGN temp = CreateRectRgn(0, 0, WindowRect.right, WindowRect.bottom);
	GetWindowRgn(hWnd, temp);
	SelectClipRgn(hdc, temp);
	
	// end make clip region -- this is so it doesn't paint the whole SQUARE just the region

	if (SkinType==FULL_SKIN){
		StretchBlt(hdc, 0, 0, SWin_Width, SWin_Height, dcSkin, 0, 0, _bitmap.bmWidth, _bitmap.bmHeight, SRCCOPY);
	}
	else {	// else tile the window with the image at default size

		int x_pos=0, y_pos=0;

		while(y_pos < SWin_Height)
		{
			while(x_pos < SWin_Width)
			{	
				BitBlt(hdc, x_pos, y_pos, _bitmap.bmWidth, _bitmap.bmHeight, dcSkin, 0,0, SRCCOPY);
				x_pos += _bitmap.bmWidth; 
			}
	
			x_pos=0;
			y_pos += _bitmap.bmHeight; // Move down to the next "row" for blitting the background tile
		}

	} // end of if else 

	SelectObject(dcSkin, hOldBmp);	// put back the default bitmap so that windows can 
	DeleteObject(temp); 	// take care of memory on quit
}


///////////////// END OF SKIN IMPLEMENTATION ///////////////////////////////

