#include "winglib/wgTabControlDialog.h"
#include "winglib/wgTabDialog.h"
#include <stdio.h>
#include <commctrl.h>
#include "winglib/wgTabCtrlHelpers.h"



#define TAB_SIDE_BUFFER 16
#define TAB_TOP_BUFFER 38


using namespace std;





HWND wgTabControlDialog::Create(DWORD _dwStyle)
{

	dwStyle = _dwStyle;
	return wgDialog::Create();
}



wgTabControlDialog::wgTabControlDialog(HINSTANCE _hInstance, int _ResourceCode)
	:wgDialog(_ResourceCode, _hInstance)
{

	MaxDialogWidth = 0;
	MaxDialogHeight = 0;
	CurrentTab = 0;
	hTabWnd = 0;
}


wgTabControlDialog::~wgTabControlDialog()
{
	// make sure all TabDialog's are closed, and then delete them

	wgTabDialog* TempDialog = NULL;

	TabDialogListType::iterator iter;
	iter = DialogList.begin();

	try
	{
		while (iter != DialogList.end())
		{
			TempDialog = (*iter).mpTab;
			if (TempDialog !=0) 
			{
				delete TempDialog;
				TempDialog = 0;
			}
			iter++;
		}
	}
	catch(exception e){	}

	DialogList.clear();
}



//////////////////// WINDOWS MESSAGE HANDLERS  //////////////////////////


void wgTabControlDialog::OnInitDialog()
{
	INITCOMMONCONTROLSEX cc;
	
	cc.dwSize = sizeof(INITCOMMONCONTROLSEX);
	cc.dwICC = ICC_TAB_CLASSES | ICC_UPDOWN_CLASS;
	InitCommonControlsEx(&cc);

	int TabWidth = (((int)Width()-TAB_SIDE_BUFFER)>50)?Width()-TAB_SIDE_BUFFER:50;
	int TabHeight = (((int)Height()-TAB_TOP_BUFFER)>80)?Height()-TAB_TOP_BUFFER:80;
	
	hTabWnd = CreateWindow(WC_TABCONTROL, "", WS_VISIBLE | WS_TABSTOP | WS_CHILD | dwStyle,
							6,8, TabWidth, TabHeight,
							hWnd, NULL, hInstance, NULL);

}


BOOL wgTabControlDialog::AddTabDialog(wgTabDialog* hDialog, const char* TabText)
{
	TCITEM tci;
	RECT DRect, TabRect, BottomRect;
	tci.mask = TCIF_TEXT|TCIF_PARAM;
	tci.iImage = -1;
	int num=0;
	int TabWidth = 0;
	int TabHeight =0;

	// first we make sure we can create the dialog
	// we will store it as well. probably by tab text or index
	HWND NewDialog = hDialog->Create(hTabWnd, this); // store NewDialog only for testing shortly later
	hDialog->Show(SW_HIDE);

	if (NewDialog==NULL) return FALSE;

	GetWindowRect(NewDialog, &DRect);
	GetWindowRect(hTabWnd, &TabRect);

	if (DRect.right-DRect.left >MaxDialogWidth) 
		MaxDialogWidth  = DRect.right-DRect.left;

	if (DRect.bottom-DRect.top>MaxDialogHeight)
		MaxDialogHeight = DRect.bottom-DRect.top;

	TabWidth = MaxDialogWidth + 4;
	TabHeight = TabRect.bottom - TabRect.top;

	SetWindowPos(hTabWnd, HWND_NOTOPMOST , 0, 0, 
				TabWidth, TabHeight, 
				SWP_NOOWNERZORDER |SWP_NOMOVE |SWP_NOZORDER);	

	tci.pszText = (char*)TabText;
	tci.lParam = (LPARAM)hDialog;
	num = TabCtrl_GetItemCount(hTabWnd);
	num = TabCtrl_InsertItem(hTabWnd, num, &tci);
	
	if (num==-1) return FALSE;

	// at this point we should be able to get the height we need
	BottomRect = GetBottomTabRect();
	TabHeight = MaxDialogHeight + BottomRect.bottom +4;

	SetWindowPos(hTabWnd, HWND_NOTOPMOST , 0, 0, 
				TabWidth, TabHeight, 
				SWP_NOOWNERZORDER |SWP_NOMOVE |SWP_NOZORDER);

	DialogList.insert(DialogList.end(), TabDialogContainer(TabText, hDialog));//	DialogList.Add(TabText, hDialog);

	hDialog->SetTabText(TabText);	// give the TabDialog his TabIndex number

	RepositionTabs(BottomRect.bottom);

	TabHeight = BottomRect.bottom + MaxDialogHeight + 2;

	// mod wgTabControlDialog
	ReSizeWindow(TabWidth+TAB_SIDE_BUFFER, TabHeight+TAB_TOP_BUFFER+2); 
	
	TabCtrl_SetCurSel(hTabWnd, num);
	SetTabDialogSelection(num);
	
	return TRUE;	
}



BOOL wgTabControlDialog::DeleteTabDialog(wgTabDialog* hDialog)
{
	if (hDialog!=NULL) return DeleteTabDialog(hDialog->GetTabText().c_str());
	else return FALSE;
}



BOOL wgTabControlDialog::DeleteTabDialog(const char* _TabText)
{
	if (_TabText!=NULL){ // extract TabIndex from TabText

		unsigned int TabIndex=0;
		if(TabCtrl_GetItemIndex(hTabWnd, _TabText, TabIndex)){

			return DeleteTabDialog(TabIndex);
		}
		else return FALSE;
	}
	else return FALSE;
	
}



BOOL wgTabControlDialog::DeleteTabDialog(const unsigned int TabIndex)
{
	RECT TabRect, BottomRect;
	wgTabDialog* hDialog= 0;
	int TabWidth = 0, TabHeight = 0, num=0, CurrentSelection=-1;
	LPARAM lParam;
	
	if (!TabCtrl_GetItemParam(hTabWnd, TabIndex, lParam)) return FALSE;

	hDialog = (wgTabDialog*)lParam;

	if (hDialog==NULL) return FALSE;

	CurrentSelection = TabCtrl_GetCurSel(hTabWnd);

	if (!TabCtrl_DeleteItem(hTabWnd, TabIndex)) return FALSE; 
	num = TabCtrl_GetItemCount(hTabWnd);
	// if deletion from tab didn't work, progress no further

	///////***BlockA
	// if currently selected tab is deleted one, select another. Quick!!
	if (CurrentSelection == (int)TabIndex && num>0)
	{
		// scan through and find an existing tab, and it's dialog and index
		unsigned int index=0;
		TCITEM tci;
		tci.mask = TCIF_PARAM;

		while(1)
		{
			if (TabCtrl_GetItem(hTabWnd, index, &tci)) break;
			index++;
			if (index>500)
			{
				break; // protect against endless loop
			}
		}
		
		TabCtrl_SetCurSel(hTabWnd, index);
		SetTabDialogSelection(index);
	}
	///////End BlockA***
	TabDialogListType::iterator iter = FindTab(hDialog->GetTabText().c_str());
	if (iter != DialogList.end())
	{
		DialogList.erase(iter);	//DialogList.Remove(hDialog->GetTabText());
	}

	GetWindowRect(hTabWnd, &TabRect);
	
	// at this point we should be able to get the height we need
	BottomRect = GetBottomTabRect();

	TabWidth = MaxDialogWidth + 4;
	TabHeight = TabRect.bottom - TabRect.top;

	TabHeight = MaxDialogHeight + BottomRect.bottom +4;

	SetWindowPos(hTabWnd, HWND_NOTOPMOST , 0, 0, 
				TabWidth, TabHeight, 
				SWP_NOOWNERZORDER |SWP_NOMOVE |SWP_NOZORDER);


	RepositionTabs(BottomRect.bottom);

	TabHeight = BottomRect.bottom + MaxDialogHeight + 2;

	// mod wgTabControlDialog
	ReSizeWindow(TabWidth+TAB_SIDE_BUFFER, TabHeight+TAB_TOP_BUFFER+2); 
	
	delete hDialog; // this also kills the specific TabDialog

	return TRUE;	
}



void wgTabControlDialog::RepositionTabs(int intPositionY)
{
	TabDialogListType::iterator iter;
	iter = DialogList.begin();
	wgTabDialog* TempDialog = NULL;

	try	// test, or fix, this whole block is duplicated. Make it a function of positioning
	{	
		while (iter != DialogList.end())
		{
			TempDialog = (*iter).mpTab;
		
			if (TempDialog !=0) 
			{
				TempDialog->MoveWindow(2, intPositionY+2);
			}
		
			iter++;
		}
	}
	catch(exception e){	}
}



TabDialogListType::iterator wgTabControlDialog::FindTab(const char* cpcharTabName)
{
	TabDialogListType::iterator iter = DialogList.begin();

	try
	{
		while (iter != DialogList.end())
		{
			if (strcmp((*iter).mstrTabName.c_str(),cpcharTabName)==0)
			{	// break out and return if we have found the entry we want
				break;
			}

			iter++;
		}
	}
	catch(exception e)
	{	
		iter = DialogList.end();
	}
	
	return iter;
}



RECT wgTabControlDialog::GetBottomTabRect()
{
	int num = TabCtrl_GetItemCount(hTabWnd);
	RECT BottomRect, TempRect;
	memset((void*)&BottomRect, 0, sizeof(RECT));

	for (int i=0;i<num;i++){

		TabCtrl_GetItemRect(hTabWnd, i, &TempRect); // zero will always be the bottom most
		if (TempRect.bottom>BottomRect.bottom) BottomRect = TempRect;
	}
	
	return BottomRect;
}


BOOL wgTabControlDialog::OnCommand(int IDCode, int notifyCode)
{
	switch(IDCode)
	{
		case IDOK:
			PostMessage(WM_CLOSE, 0, 0);
			break;
		
		default: return FALSE;
	};

	return FALSE;
}



BOOL wgTabControlDialog::OnNotify(int IDCode, NMHDR *hdr)
{
	int TabNumber =0;

	if (hdr->code == TCN_SELCHANGE){

		TabNumber = TabCtrl_GetCurSel((HWND)hdr->hwndFrom);
		SetTabDialogSelection(TabNumber);
	}
	
	return FALSE;
}




void wgTabControlDialog::SetTabDialogSelection(const unsigned int _TabIndex)
{
	wgTabDialog* NextDialog=0;
	LPARAM lParam;
	
	if (!TabCtrl_GetItemParam(hTabWnd, _TabIndex, lParam)) return;

	NextDialog = (wgTabDialog*)lParam;

	if (NextDialog==NULL) return;

	if (CurrentTab!=0)
	{
		CurrentTab->OnChangeOut();	
		CurrentTab->Show(SW_HIDE);
	}

	NextDialog->OnChangeIn();
	NextDialog->Show();

	CurrentTab=NextDialog;
}



