#include "winglib/wgWinAbstractComboBox.h"



wgWinAbstractComboBox::wgWinAbstractComboBox(const char* _pcharControlID) :	wgWinControl(_pcharControlID), 
												wgWindow(NULL, "COMBOBOX", NULL)
{
	pcharEndTextBuffer = 0;
	GetTextReturn = 0;
	intEndCursorSelectIndex = 0;
	intEndCount = 0;
	intEndCursorTextLength = 0;
}


wgWinAbstractComboBox::~wgWinAbstractComboBox()
{
	if (pcharEndTextBuffer!=0) delete[] pcharEndTextBuffer;
}




HWND wgWinAbstractComboBox::Create(const char* Label, HWND _Parent, int x, int y, int nWidth, int nHeight, UINT Style) throw (wuException)
{
	HWND Return = NULL;

	Return = wgWindow::Create((char*)Label, WS_CHILDWINDOW|Style, x, y, nWidth, nHeight, NULL, NULL, _Parent);

	Attach(); 

	return Return;
}



 

LRESULT wgWinAbstractComboBox::OnOther(UINT message, WPARAM wParam, LPARAM lParam)
{
	LRESULT lrRes = 1;

	AbsMutex.WaitForSingleObject(0);

	lrRes = GetListener().OnOtherAction(message, wParam, lParam, this);

	AbsMutex.ReleaseMutex();

	return lrRes;
}




LRESULT wgWinAbstractComboBox::OnMouseMove(WPARAM wParam, LPARAM lParam)
{
	LRESULT lrRes = 1;

	AbsMutex.WaitForSingleObject(0);

	lrRes = GetListener().OnMouseMoveAction(wParam, lParam, this);

	AbsMutex.ReleaseMutex();

	return lrRes;
}





LRESULT wgWinAbstractComboBox::OnMouseLButtonDown(WPARAM wParam, LPARAM lParam)
{
	LRESULT lrRes = 1;

	AbsMutex.WaitForSingleObject(0);

	lrRes = GetListener().OnMouseLButtonDownAction(wParam, lParam, this);

	AbsMutex.ReleaseMutex();

	return lrRes;
}



LRESULT wgWinAbstractComboBox::OnMouseLButtonUp(WPARAM wParam, LPARAM lParam)
{	
	LRESULT lrRes = 1;

	AbsMutex.WaitForSingleObject(0);

	lrRes = GetListener().OnMouseLButtonUpAction(wParam, lParam, this);

	AbsMutex.ReleaseMutex();

	return lrRes;
}




LRESULT wgWinAbstractComboBox::OnMouseRButtonDown(WPARAM wParam, LPARAM lParam)
{
	LRESULT lrRes = 1;

	AbsMutex.WaitForSingleObject(0);

	lrRes = GetListener().OnMouseRButtonDownAction(wParam, lParam, this);

	AbsMutex.ReleaseMutex();

	return lrRes;
}



LRESULT wgWinAbstractComboBox::OnMouseRButtonUp(WPARAM wParam, LPARAM lParam)
{
	LRESULT lrRes = 1;

	AbsMutex.WaitForSingleObject(0);

	lrRes = GetListener().OnMouseRButtonUpAction(wParam, lParam, this);

	AbsMutex.ReleaseMutex();

	return lrRes;
}



	// sent to window loosing
	// capture. It recieves HWND of window
	// gaining the mouse capture
LRESULT wgWinAbstractComboBox::OnCaptureChanged(HWND hwndNewCapture)
{
	LRESULT lrRes = 1;

	AbsMutex.WaitForSingleObject(0);

	lrRes = GetListener().OnCaptureChangedAction(hwndNewCapture, this);

	AbsMutex.ReleaseMutex();

	return lrRes;
}




LRESULT wgWinAbstractComboBox::OnNCDestroy()
{
	LRESULT lrRes = 1;

	AbsMutex.WaitForSingleObject(0);

	UnAttach();

	// Because window is dying, first get all End Values
	intEndCount = GetCount();
	intEndCursorSelectIndex = GetCursorSelectIndex();
	
	if (intEndCursorSelectIndex>=0)
	{
		intEndCursorTextLength = GetCursorTextLength(intEndCursorSelectIndex);// don't add null here because user gets this value

		if (intEndCursorTextLength>0)
		{
			pcharEndTextBuffer = new char[intEndCursorTextLength+1];
			GetTextReturn = GetCursorText(pcharEndTextBuffer, intEndCursorSelectIndex, intEndCursorTextLength+1);
		}
	}

	try
	{
		lrRes = GetListener().OnNCDestroyAction(this);
	}
	catch(...)
	{
	}

	AbsMutex.ReleaseMutex();

	return lrRes;
}


LRESULT wgWinAbstractComboBox::OnDestroy()
{
	LRESULT lrRes = 1;

	AbsMutex.WaitForSingleObject(0);

	lrRes = GetListener().OnDestroyAction(this);

	AbsMutex.ReleaseMutex();

	return lrRes;
}




LRESULT wgWinAbstractComboBox::OnPaint()
{
	LRESULT lrRes = 1;

	AbsMutex.WaitForSingleObject(0);

	lrRes = GetListener().OnPaintAction(this);

	AbsMutex.ReleaseMutex();

	return lrRes;
}






int wgWinAbstractComboBox::GetCursorSelectIndex()
{
	if (IsCreated())
	{
		return SendMessage(CB_GETCURSEL, 0, 0);
	}
	else
	{
		return intEndCursorSelectIndex;
	}
}





void wgWinAbstractComboBox::SetCursorSelectIndex(int index)
{
	if (IsCreated())
	{
		SendMessage(CB_SETCURSEL, index, 0);
	}
}




int wgWinAbstractComboBox::GetCount()
{	
	if (IsCreated())
	{
		return SendMessage(CB_GETCOUNT, 0, 0);
	}
	else
	{
		return intEndCount;
	}
}




int wgWinAbstractComboBox::GetCursorTextLength(const int index)
{
	if (IsCreated())
	{
		return SendMessage(CB_GETLBTEXTLEN, index, 0);
	}
	else
	{
		return intEndCursorTextLength;
	}
}




int wgWinAbstractComboBox::GetCursorText(char* pcharBuffer, const int index, const int len)
{
	int ActLen = GetCursorTextLength(index)+1;

	if (ActLen<2) return CB_ERR;

	char* pcharTempBuffer = 0;
	
	int Result = 0;

	// first we get the actual string

	if (IsCreated())
	{
		pcharTempBuffer = new char[ActLen];
		Result = SendMessage(CB_GETLBTEXT, index,(long) pcharTempBuffer);
	}
	else
	{
		pcharTempBuffer = pcharEndTextBuffer;
		Result = GetTextReturn;
	}

	if (ActLen>len) // if the buffer passed to us is smaller than the actual text size + null
	{
		ActLen = len -1;
	}

	// now we copy it to the users buffer and return the already retrieved result

	int intCount = 0;

	if (pcharTempBuffer!=0 && pcharBuffer!=0)
	{
		while (intCount<ActLen)
		{
			pcharBuffer[intCount] = pcharTempBuffer[intCount];
			intCount++;
		}
	
		pcharBuffer[intCount]=0;	// add null

		return Result;
	}
	else
	{
		return CB_ERR;
	}
}
