#include "winglib/wgWinAbstractEdit.h"
#include <string.h>


wgWinAbstractEdit::wgWinAbstractEdit(const char* _pcharButtonID) :	wgWinControl(_pcharButtonID), 
																			wgWindow(NULL, "EDIT", NULL)
{
	pcharEndTextBuffer = 0;
	GetTextReturn = 0;
	intEndTextLength = 0;
}




wgWinAbstractEdit::~wgWinAbstractEdit()
{
	if (pcharEndTextBuffer!=0) delete[] pcharEndTextBuffer;
}





HWND wgWinAbstractEdit::Create(const char* Label, HWND _Parent, int x, int y, int nWidth, int nHeight, UINT Style) throw (wuException)
{
	HWND Return = NULL;

	Return = wgWindow::Create((char*)Label, Style, x, y, nWidth, nHeight, NULL, NULL, _Parent);

	Attach(); 

	return Return;
}





 

LRESULT wgWinAbstractEdit::OnOther(UINT message, WPARAM wParam, LPARAM lParam)
{
	LRESULT lrRes = 1;

	AbsMutex.WaitForSingleObject(0);

	lrRes = GetListener().OnOtherAction(message, wParam, lParam, this);

	AbsMutex.ReleaseMutex();

	return lrRes;
}


LRESULT wgWinAbstractEdit::OnMouseMove(WPARAM wParam, LPARAM lParam)
{
	LRESULT lrRes = 1;

	AbsMutex.WaitForSingleObject(0);

	lrRes = GetListener().OnMouseMoveAction(wParam, lParam, this);

	AbsMutex.ReleaseMutex();

	return lrRes;
}



LRESULT wgWinAbstractEdit::OnMouseLButtonDown(WPARAM wParam, LPARAM lParam)
{
	LRESULT lrRes = 1;

	AbsMutex.WaitForSingleObject(0);

	lrRes = GetListener().OnMouseLButtonDownAction(wParam, lParam, this);

	AbsMutex.ReleaseMutex();

	return lrRes;
}



LRESULT wgWinAbstractEdit::OnMouseLButtonUp(WPARAM wParam, LPARAM lParam)
{	
	LRESULT lrRes = 1;

	AbsMutex.WaitForSingleObject(0);

	lrRes = GetListener().OnMouseLButtonUpAction(wParam, lParam, this);

	AbsMutex.ReleaseMutex();

	return lrRes;
}



LRESULT wgWinAbstractEdit::OnMouseRButtonDown(WPARAM wParam, LPARAM lParam)
{
	LRESULT lrRes = 1;

	AbsMutex.WaitForSingleObject(0);

	lrRes = GetListener().OnMouseRButtonDownAction(wParam, lParam, this);

	AbsMutex.ReleaseMutex();

	return lrRes;
}



LRESULT wgWinAbstractEdit::OnMouseRButtonUp(WPARAM wParam, LPARAM lParam)
{
	LRESULT lrRes = 1;

	AbsMutex.WaitForSingleObject(0);

	lrRes = GetListener().OnMouseRButtonUpAction(wParam, lParam, this);

	AbsMutex.ReleaseMutex();

	return lrRes;
}


	// sent to window loosing
	// capture. It recieves HWND of window
	// gaining the mouse capture
LRESULT wgWinAbstractEdit::OnCaptureChanged(HWND hwndNewCapture)
{
	LRESULT lrRes = 1;

	AbsMutex.WaitForSingleObject(0);

	lrRes = GetListener().OnCaptureChangedAction(hwndNewCapture, this);

	AbsMutex.ReleaseMutex();

	return lrRes;
}



LRESULT wgWinAbstractEdit::OnNCDestroy()
{
	LRESULT lrRes = 1;

	AbsMutex.WaitForSingleObject(0);

	intEndTextLength = GetTextLength();

	if (intEndTextLength>0)
	{
		pcharEndTextBuffer = new char[intEndTextLength+1];
		GetTextReturn = GetText(pcharEndTextBuffer, intEndTextLength+1);
	}

	UnAttach();

	try
	{
		lrRes = GetListener().OnNCDestroyAction(this);
	}
	catch(...)
	{
	}

	AbsMutex.ReleaseMutex();

	return lrRes;
}



LRESULT wgWinAbstractEdit::OnDestroy()
{
	LRESULT lrRes = 1;

	AbsMutex.WaitForSingleObject(0);

	lrRes = GetListener().OnDestroyAction(this);

	AbsMutex.ReleaseMutex();

	return lrRes;
}



LRESULT wgWinAbstractEdit::OnPaint()
{
	LRESULT lrRes = 1;

	AbsMutex.WaitForSingleObject(0);

	lrRes = GetListener().OnPaintAction(this);

	AbsMutex.ReleaseMutex();

	return lrRes;
}




int wgWinAbstractEdit::GetTextLength()
{
	if (IsCreated())
	{
		return GetWindowTextLength(hWnd);
	}
	else
	{
		return intEndTextLength;
	}
}




int wgWinAbstractEdit::GetText(char* pcharBuffer, const int len)
{
	int Result;

	if (IsCreated() && pcharBuffer!=0)
	{
		Result = GetWindowText(hWnd, pcharBuffer, len);
	}
	else if (pcharEndTextBuffer!=0 && pcharBuffer!=0)
	{
		int ActLen = strlen(pcharEndTextBuffer) +1;

		if (ActLen>len) // if the buffer passed to us is smaller than the actual text size + null
		{
			ActLen = len -1;
		}

		int intCount = 0;

		while (intCount<ActLen)
		{
			pcharBuffer[intCount] = pcharEndTextBuffer[intCount];
			intCount++;
		}
	
		pcharBuffer[intCount]=0;	// add null

		Result = GetTextReturn;
	}
	else
	{
		return 0;
	}

	return Result;
}


