
#include "winglib/wgWinBase.h"
#include "winglib/wgWinMenuBar.h"




////////////////// GLOBAL WG ACCESS Functions allowing use /////////////




DWORD WINAPI WG_WindowHandler(LPVOID lpParameter){ // new permanent function

	HANDLE ErrorWaitEvent = OpenEvent(EVENT_MODIFY_STATE|STANDARD_RIGHTS_ALL, FALSE, "ErrorWaitEvent");
	HANDLE OKWaitEvent = OpenEvent(EVENT_MODIFY_STATE|STANDARD_RIGHTS_ALL, FALSE, "OKWaitEvent");

	VoidDouble* FuncAUDat = (VoidDouble*)lpParameter;

	wgWinBase* UserWindow = ((WindowCreatorFunc)(FuncAUDat->ptrOne))(FuncAUDat->ptrTwo); // call the user created function

	MSG msg;
	HWND MyHwnd = 0;

	MyHwnd=UserWindow->GetHwnd();

	if (MyHwnd==NULL){
		SetEvent(ErrorWaitEvent);
		return 0;	// return zero for Something Wrong
	}
	else{
		 SetEvent(OKWaitEvent);
	}


	while(GetWindowLong(MyHwnd, GWL_HINSTANCE) && GetMessage(&msg, MyHwnd, 0 , 0)){

		if (!IsForDialog(MyHwnd, &msg)){
				TranslateMessage(&msg);
				DispatchMessage(&msg);
		}
	}

	UserWindow=0; // don't delete because this one deletes itself (should be mandatory, and controlled)

	return (1);
}


// returns the thread associated with the user created window
HANDLE WINAPI WG_WindowCreator(WindowCreatorFunc _UserFunc, void* _ptrUserData) throw(exception){ // new permanent function

	HANDLE CreationMutex = CreateMutex(NULL, FALSE, "CreationMutex"); // make multithread safe
	WaitForSingleObject(CreationMutex, 30000); // ask for ownership

	VoidDouble FuncAndUserData;
	DWORD threadID;
	HANDLE ErrorAndOK[2];
	
	ErrorAndOK[0] = CreateEvent(NULL, FALSE, FALSE, "ErrorWaitEvent");
	ErrorAndOK[1] = CreateEvent(NULL, FALSE, FALSE, "OKWaitEvent");

	FuncAndUserData.ptrOne = _UserFunc;
	FuncAndUserData.ptrTwo = _ptrUserData;

	HANDLE CR = CreateThread(NULL, NULL, WG_WindowHandler, (LPVOID)&FuncAndUserData, NULL, &threadID);

	if (CR==NULL){
		CloseHandle(ErrorAndOK[0]);
		CloseHandle(ErrorAndOK[1]);
		ReleaseMutex(CreationMutex); // give up mutex here
		throw exception("CreateThread Failed!");
	}

	DWORD EventResult =WaitForMultipleObjects(2, ErrorAndOK, FALSE, INFINITE); // wait on either Events

	CloseHandle(ErrorAndOK[0]);
	CloseHandle(ErrorAndOK[1]);	

	if (EventResult-WAIT_OBJECT_0==0){
		ReleaseMutex(CreationMutex); // give up mutex here
		throw exception("Window Not Created Error"); 
	}

	ReleaseMutex(CreationMutex); // give up mutex here (release ownership)

	return CR;
}





////////////// end of global functions /////////////////////





wgWinBase::wgWinBase(){
		
	SWin_Width = 0;
	SWin_Height = 0;

	SWin_Left = 0;
	SWin_Top = 0;

	hInstance = 0;	// temporary initialisation
	hWnd = 0;		// temporary initialisation
}




wgWinBase::~wgWinBase(){	// declare virtual destructor, so simple need to do nothing

}





int wgWinBase::MessageBox(LPCTSTR lpText, LPCTSTR lpCaption, UINT uType)
{
	return ::MessageBox(hWnd, lpText, lpCaption, uType );
}



int wgWinBase::MessageBox(HWND _hWnd, LPCTSTR lpText, LPCTSTR lpCaption, UINT uType)
{
	return ::MessageBoxEx(_hWnd, lpText, lpCaption, uType, LANG_NEUTRAL | SUBLANG_SYS_DEFAULT);
}



int wgWinBase::MessageBox(int code, LPCTSTR lpCaption, UINT uType)
{
	char hbuff[100]={0};
	sprintf(hbuff, "%d", code);
	return ::MessageBoxEx(hWnd, hbuff, lpCaption, uType, LANG_NEUTRAL | SUBLANG_SYS_DEFAULT);
} 





int  wgWinBase::SaveDialogBox(FileDialogProc _Proc,
							  void* param,
							  LPTSTR _lpstrFile,
        				      DWORD _nMaxFile,
							  LPCTSTR _lpstrFilter,
							  DWORD _nFilterIndex,
							  LPCTSTR _lpstrInitialDir,
							  LPCTSTR _lpstrTitle,
							  DWORD _nMaxFileTitle, // changed source
							  DWORD _Flags,
							  HWND _Owner, 
							  HINSTANCE _hInstance){

	OPENFILENAME ofn;
	BOOL result = FALSE;
	memset(&ofn,0,sizeof(OPENFILENAME));
	
	// set all the defaults or specified
	ofn.lpstrFile = _lpstrFile;
	ofn.nMaxFile = _nMaxFile;
	ofn.lpstrFilter = _lpstrFilter;
	ofn.nFilterIndex = _nFilterIndex;
	ofn.lpstrInitialDir = _lpstrInitialDir;
	ofn.lpstrTitle = _lpstrTitle;
	ofn.nMaxFileTitle = _nMaxFileTitle;
	ofn.hwndOwner = _Owner;
	ofn.hInstance = _hInstance;
	ofn.Flags = _Flags;

	// set the standard 
	ofn.lStructSize = sizeof(OPENFILENAME);

	result = GetSaveFileName(&ofn);
	if (result==FALSE) return result;
	else return _Proc(ofn, param);
}



int  wgWinBase::OpenDialogBox(FileDialogProc _Proc,
							  void* param,
							  LPTSTR _lpstrFile,
        				      DWORD _nMaxFile,
							  LPCTSTR _lpstrFilter,
							  DWORD _nFilterIndex,
							  LPCTSTR _lpstrInitialDir,
							  LPCTSTR _lpstrTitle,
							  DWORD _nMaxFileTitle,
							  DWORD _Flags,
							  HWND _Owner, 
							  HINSTANCE _hInstance){
	OPENFILENAME ofn;
	BOOL result = FALSE;
	memset(&ofn,0,sizeof(OPENFILENAME));
	
	// set all the defaults or specified
	ofn.lpstrFile = _lpstrFile;
	ofn.nMaxFile = _nMaxFile;
	ofn.lpstrFilter = _lpstrFilter;
	ofn.nFilterIndex = _nFilterIndex;
	ofn.lpstrInitialDir = _lpstrInitialDir;
	ofn.lpstrTitle = _lpstrTitle;
	ofn.nMaxFileTitle = _nMaxFileTitle;
	ofn.hwndOwner = _Owner;
	ofn.hInstance = _hInstance;
	ofn.Flags = _Flags;

	// set the standard 
	ofn.lStructSize = sizeof(OPENFILENAME);

	result = GetOpenFileName(&ofn);
	if (result==FALSE) return result;
	else return _Proc(ofn, param);
}





BOOL wgWinBase::Add(wgWinMenuBar &MenuBar)
{
	return SetMenu(hWnd, MenuBar.MenuBar);
}




BOOL wgWinBase::DrawMenuBar()
{
	return ::DrawMenuBar(hWnd);
}





int wgWinBase::wgMessageQueue(int WindowMessage)
{
	int intReturn = 0;

	switch(WindowMessage)
	{
		case WM_CAPTURECHANGED: 
			intReturn=  CWM_CAPTURECHANGED;
			break;

		case WM_LBUTTONDBLCLK:  
			intReturn=  CWM_LBUTTONDBLCLK;			
			break;

		case WM_LBUTTONDOWN:  
			intReturn=  CWM_LBUTTONDOWN;
			break;
							
		case WM_LBUTTONUP:  
			intReturn=  CWM_LBUTTONUP;
			break;
					
		case WM_MBUTTONDBLCLK:  
			intReturn=  CWM_MBUTTONDBLCLK;
			break;
		
		case WM_MBUTTONDOWN:  
			intReturn=  CWM_MBUTTONDOWN;
			break;
		
		case WM_MBUTTONUP:  
			intReturn=  CWM_MBUTTONUP;
			break;
	
		case WM_MOUSEACTIVATE:  
			intReturn=  CWM_MOUSEACTIVATE;
			break;
	
		case WM_MOUSEMOVE:  
			intReturn=  CWM_MOUSEMOVE;
			break;
	
		case WM_MOUSEWHEEL:  
			intReturn=  CWM_MOUSEWHEEL;
			break;
	
		case WM_NCHITTEST:  
			intReturn=  CWM_NCHITTEST;
			break;

		case WM_NCLBUTTONDBLCLK:  
			intReturn=  CWM_NCLBUTTONDBLCLK;
			break;
	
		case WM_NCLBUTTONDOWN:  
			intReturn=  CWM_NCLBUTTONDOWN;
			break;
	
		case WM_NCLBUTTONUP:  
			intReturn=  CWM_NCLBUTTONUP;
			break;
	
		case WM_NCMBUTTONDBLCLK:  
			intReturn=  CWM_NCMBUTTONDBLCLK;
			break;
	
		case WM_NCMBUTTONDOWN:  
			intReturn=  CWM_NCMBUTTONDOWN;
			break;
	
		case WM_NCMBUTTONUP:  
			intReturn=  CWM_NCMBUTTONUP;
			break;
	
		case WM_NCMOUSEMOVE:  
			intReturn=  CWM_NCMOUSEMOVE;
			break;
		
		case WM_NCRBUTTONDBLCLK:  
			intReturn=  CWM_NCRBUTTONDBLCLK;
			break;
	
		case WM_NCRBUTTONDOWN:  
			intReturn=  CWM_NCRBUTTONDOWN;
			break;
		
		case WM_NCRBUTTONUP:  
			intReturn=  CWM_NCRBUTTONUP;
			break;
		
		case WM_RBUTTONDBLCLK:  
			intReturn=  CWM_RBUTTONDBLCLK;
			break;
		
		case WM_RBUTTONDOWN: 
			intReturn=  CWM_RBUTTONDOWN;
			break;
			 
		case WM_RBUTTONUP:   
			intReturn=  CWM_RBUTTONUP;
			break;

		default:
			intReturn = CWM_UNKNOWN_MESSAGE;
	};

	return intReturn;
}


