#include "winglib/wgWinControl.h"









///////////////     START OF CONTROL CLASS IMPLEMENTATION         //////////////////



wgWinControl::wgWinControl(const char* _pcharControlName, int RESOURCE_CODE, HwgWinBase _Parent, int _Width,	int _Height){

	Parent = _Parent;
	hWnd = GetDlgItem(Parent->GetHwnd(), RESOURCE_CODE);

	if (Parent!=0)
		wgWinBase::hInstance = (HINSTANCE)GetWindowLong(Parent->GetHwnd(), GWL_HINSTANCE);
	else wgWinBase::hInstance = (HINSTANCE)GetModuleHandle(NULL);

	SWin_Width = _Width;
	SWin_Height = _Height;

	OldProc = 0;

	SetControlID(_pcharControlName);
}



wgWinControl::wgWinControl(const char* _pcharControlName, HWND _Button, HwgWinBase _Parent, int _Width, int _Height){

	Parent = _Parent;
	hWnd = _Button;

	if (Parent!=0)
		wgWinBase::hInstance = (HINSTANCE)GetWindowLong(Parent->GetHwnd(), GWL_HINSTANCE);
	else wgWinBase::hInstance = (HINSTANCE)GetModuleHandle(NULL);
	SWin_Width = _Width;
	SWin_Height = _Height;	

	OldProc = 0;

	SetControlID(_pcharControlName);
}



wgWinControl::wgWinControl(const char* _pcharControlName)
{
	OldProc = 0;
	SWin_Width = 0;	
	SWin_Height = 0;	
	SWin_Left = 0;	

	SetControlID(_pcharControlName);
}


wgWinControl::~wgWinControl()
{
	if (pcharControlID!=0)
	{
		delete[] pcharControlID;
	}
}




void wgWinControl::SetControlID(const char* _pcharControlID)
{
	if (_pcharControlID!=0)
	{
		int len = strlen(_pcharControlID) + 1;
		pcharControlID = new char[len];
		strcpy(pcharControlID, _pcharControlID);
	}
	else
	{
		pcharControlID = 0;
	}
}





char* wgWinControl::GetControlID(char* pcharBuffer, unsigned int pcharLen)
{
	if (pcharBuffer!=0 && pcharControlID!=0)
	{
		unsigned int CpLen = strlen(pcharControlID);
		unsigned int uintCount = 0;

		if (CpLen+1>pcharLen)	// if lenth of string + null is greater than the buffer provided
		{
			CpLen = pcharLen-1; // copy pcharlen-1 characters, then one more character at end is NULL terminator
		}

		while (uintCount<CpLen)	// copy the string. 
		{	
			pcharBuffer[uintCount] = pcharControlID[uintCount];
			uintCount++;
		}

		pcharBuffer[uintCount] = 0;	// add the null terminator

		return pcharBuffer;
	}
	else return 0;
}





void wgWinControl::Associate(HWND _hWnd)
{
	wgWinBase::hWnd = _hWnd;
}



void wgWinControl::Attach()	// final --- don't touch
{	
	if (OldProc==0)	// only do if not done before
	{
		OldProc =(LONG) SetWindowLong(hWnd, GWL_WNDPROC, (LONG) ControlProcedure);

		if (OldProc!=0)
		{
			::SendMessage(hWnd, CWM_ATTACH_WINDOW, (WPARAM)this, 0);
		}
	}
}



void wgWinControl::UnAttach()
{
	if (OldProc!=0)	// only do if not done before
	{
		OldProc =(LONG) SetWindowLong(hWnd, GWL_WNDPROC, (LONG) OldProc);
		OldProc = 0;
	}
}



// returns FALSE if NOT handled message
LRESULT APIENTRY wgWinControl::ControlProcedure(HWND _hWnd, UINT message, WPARAM wParam, LPARAM lParam){

	wgWinControl* controller = (wgWinControl*)GetProp(_hWnd, "wgWinControl");
	LONG OldProcess = 0;
	LRESULT lrReturn = 1; // not processed default
	int intConvertedMessage = 0;

	if (controller!=0)
	{
		OldProcess = controller->OldProc;
	}

	switch(message)	
	{
		case CWM_ATTACH_WINDOW:
			if (controller==NULL)
			{
				SetProp(_hWnd, "wgWinControl", (HANDLE)wParam);
				controller = (wgWinControl*)wParam;
				controller->_OnUser();
				lrReturn = 0;
			}
			else if (controller!=0) 
			{	
				lrReturn = controller->OnOther(message, wParam, lParam);	// if not initial setup, pass along to user function
			}
			break;

		case WM_PAINT:
			if (controller!=0)
			{
				lrReturn = controller->OnPaint();
			}
			break;

		case WM_MOUSEMOVE:	// convert
			if (controller!=0) 			
			{
				lrReturn = 1;
				::PostMessage(_hWnd, CWM_MOUSEMOVE, wParam, lParam);
			}
			break;

		case WM_LBUTTONDOWN:	// convert
			if (controller!=0) 			
			{
				lrReturn = 1;
				::PostMessage(_hWnd, CWM_LBUTTONDOWN, wParam, lParam);
			}
			break;

		case WM_LBUTTONUP:	// convert
			if (controller!=0) 			
			{
				lrReturn = 1;
				::PostMessage(_hWnd, CWM_LBUTTONUP, wParam, lParam);
			}
			break;

		case WM_RBUTTONDOWN:	// convert
			if (controller!=0) 			
			{
				lrReturn = 1;
				::PostMessage(_hWnd, CWM_RBUTTONDOWN, wParam, lParam);
			}
			break;
 
		case WM_RBUTTONUP:	// convert
			if (controller!=0) 			
			{
				lrReturn = 1;
				::PostMessage(_hWnd, CWM_RBUTTONUP, wParam, lParam);
			}
			break;

		case WM_CAPTURECHANGED:
			if (controller!=0) 			
			{
				lrReturn = 1;
				::PostMessage(_hWnd, CWM_CAPTURECHANGED, wParam, lParam);
			}
			break;

		case WM_DESTROY:
			if (controller!=0) 			
			{
				lrReturn = controller->OnDestroy();
			}
			break;

		case WM_NCDESTROY:
			if (controller!=0)
			{
				lrReturn = controller->_OnNCDestroy();
			}
			break;

// CWM messages are messages (or conversion messages) because we shouldn't use the 
// window messages directly because in doing so we can interupt some messages from activing correctly. 
// so we pass them onto the conversion messages instead
		case CWM_LBUTTONDOWN:
			if (controller!=0) 			
			{
				lrReturn = controller->OnMouseLButtonDown(wParam, lParam);
			}
			break;			

		case CWM_LBUTTONUP:
			if (controller!=0) 			
			{
				lrReturn = controller->OnMouseLButtonUp(wParam, lParam);
			}
			break;	

		case CWM_RBUTTONDOWN:
			if (controller!=0) 			
			{
				lrReturn = controller->OnMouseRButtonDown(wParam, lParam);
			}
			break;	

		case CWM_RBUTTONUP:
			if (controller!=0) 			
			{
				lrReturn = controller->OnMouseRButtonUp(wParam, lParam);
			}
			break;	

		case CWM_MOUSEMOVE:
			if (controller!=0) 			
			{
				lrReturn = controller->OnMouseMove(wParam, lParam);
			}
			break;	

		case CWM_CAPTURECHANGED:
			if (controller!=0) 			
			{
				lrReturn = controller->OnCaptureChanged((HWND)lParam);
			}
			break;

		default: 

			intConvertedMessage = wgMessageQueue(message);

			if (intConvertedMessage!=CWM_UNKNOWN_MESSAGE)
			{
				lrReturn = 1;
				::SendMessage(_hWnd, intConvertedMessage, wParam, lParam);
			}
			else if (controller!=0)
			{
				lrReturn = controller->OnOther(message, wParam, lParam);
			}
	};

	if (OldProcess!=0 && lrReturn!=0)
	{
		lrReturn = CallWindowProc((WNDPROC)(OldProcess), _hWnd, message, wParam, lParam);
	}

	return lrReturn;
}





void wgWinControl::_OnUser()
{
	RECT rect;
	POINT p;

	GetWindowRect(hWnd, &rect);
	p.x = rect.left;
	p.y = rect.top;
	ScreenToClient(hWnd, &p);

	if (SWin_Width==0) SWin_Width = rect.right-rect.left;	// if zero set default
	if (SWin_Height==0) SWin_Height = rect.bottom-rect.top;	// if zero set default
	if (SWin_Left==0) SWin_Left = rect.left;				// if zero set default
	if (SWin_Top==0) SWin_Top = rect.top;					// if zero set default

	SetWindowPos(hWnd, HWND_TOP, p.x, p.y, SWin_Width, SWin_Height, SWP_NOZORDER|SWP_NOMOVE);
	blnCreated = 1;
	OnInit();	// call empty function OR child overridden function
}



LRESULT wgWinControl::_OnNCDestroy()
{
	blnCreated = 0;
	return OnNCDestroy();
}

////////////////////// START OF wgWinControl Implementation //////////////////









////////////////////    END OF wgWinControl Implementation	///////////////////

