#include "winglib/wgWinMaker.h"
#include <process.h>

using namespace std;

#include <stdio.h>

// code to wgWinMaker


wgWinMaker::wgWinMaker(HINSTANCE inst, UINT size, LPCTSTR cname, WNDPROC proc, 
				   HBRUSH brush, HCURSOR curs, HICON icl, HICON ics, 
				   LPCTSTR menu, UINT style, int clsextra, int wndextra)
{
	wclassex.hInstance = inst;
	wclassex.cbSize = size;
	wclassex.lpszClassName = cname;
	wclassex.lpfnWndProc = proc;
	wclassex.hbrBackground = brush;
	wclassex.hCursor = curs;
	wclassex.hIcon=icl;
	wclassex.hIconSm=ics;
	wclassex.lpszMenuName=menu;
	wclassex.style=style;
	wclassex.cbClsExtra=clsextra;
	wclassex.cbWndExtra=wndextra;

	char WMSemName[25]={0};

	sprintf(WMSemName, "Sem_ProcID=%d", _getpid());

	WMSem = CreateSemaphore(NULL, 1, 1, WMSemName);
}


wgWinMaker::wgWinMaker(WNDCLASSEX aclass)
{
	char WMSemName[25]={0};

	sprintf(WMSemName, "Sem_ProcID=%d", _getpid());
	WMSem = CreateSemaphore(NULL, 1, 1, WMSemName);

	wclassex = aclass;
}




wgWinMaker::~wgWinMaker()
{
	if (wclassex.hInstance)
	{
		UnregisterClass(wclassex.lpszClassName,wclassex.hInstance); // Unregister the WNDCLASSEX
	}
}


ATOM wgWinMaker::register_class()
{
	return RegisterClassEx(&wclassex);
}





// this function creates a window, and also registers the creation in the system registry
HWND wgWinMaker::create_window(char* window_title, DWORD style, HMENU menu, void* param, 
								int x, int y, int nWidth, int nHeight, 
									DWORD _dwExStyle, HWND _Parent)
{
	LPARAMS* create_data = new LPARAMS;
	create_data->user_param=param;
	create_data->hwinmaker=this;

	HWND hwnd = CreateWindowEx(
		_dwExStyle,					// usual (extended style)
		wclassex.lpszClassName,	// specified (class name)
		window_title,			// arg (window title)
		style,					// arg (window style) ie, child, border etc
		x,			// usual (x position of window)
		y,			// usual (y position of window)
		nWidth,			// usual (window width)
		nHeight,			// usual (window height)
		_Parent,				// usual (parent window)
		menu,					// arg (handle to menu)
		wclassex.hInstance,		// specified
		create_data				// usual lpvoid in CREATESTRUCT
	);

	delete create_data;	// by this stage WM_CREATE has happened and 
					// user should not keep a hold on this object cause
					// as you can see it get's deleted
	return hwnd; // handle if worked, NULL if fail
}



BOOL wgWinMaker::show(HWND _hwnd, int smode)
{
	BOOL temp;

	int result=((ShowWindow(_hwnd, smode)==TRUE)?1:0);
							//	Never fails.. just If the window was previously visible,
							//	the return value is nonzero. 
							//	If the window was previously hidden,
							//	the return value is zero. 
	temp = UpdateWindow(_hwnd);
							// UpdateWindow returns true or false
							// on succeed or fail respectively
	result = ((temp==FALSE)?-1:result);

	return result;
}
// returns nonzero or zero if succeed (depending on ShowWindow
// OR -1 if fail by UpdateWindow



HWND wgWinMaker::get_running_window()
{
	HWND the_hwnd = FindWindow(wclassex.lpszClassName, 0);

	if (IsWindow(the_hwnd))
	{
		HWND hwnd_popup = GetLastActivePopup(the_hwnd);

		if (IsWindow(hwnd_popup))
		{
			the_hwnd = hwnd_popup;
		}
	}
	else 
	{
		the_hwnd = 0;
	}

	return the_hwnd;
}


// LOCK
void* wgWinMaker::get_param(HWND ahwnd)
{
	void* object = 0;

	WaitForSingleObject(WMSem, INFINITE); // LOCK
	
	UserParamMapType::iterator iter = UserParamMap.find(ahwnd);

	if (iter != UserParamMap.end())
	{
		object = iter->second;
	}

	ReleaseSemaphore(WMSem, 1, NULL); // RELEASE LOCK

	return object; 
}
// UNLOCK



// LOCK
void wgWinMaker::put_param(HWND ahwnd, void* param){

	WaitForSingleObject(WMSem, INFINITE); // LOCK

	UserParamMapType::iterator iter = UserParamMap.find(ahwnd);

	if (iter != UserParamMap.end())
	{	// erase if exists. User must clean up his own memory before.
		UserParamMap.erase(iter);
	}

	UserParamMap[ahwnd] = param;

	ReleaseSemaphore(WMSem, 1, NULL); // RELEASE LOCK

}
// UNLOCK


// delete from linkedlist, register the closing of window via registry count
// postquit and delete OpenWindowInfo key if count is zero
// destroy the window if created using this wgWinMaker
void wgWinMaker::close(HWND ahwnd)
{
	UserParamMapType::iterator iter = UserParamMap.find(ahwnd);
	 // first free memory used in STORING user defined values
	// this DOES NOT delete the actual memory that the user defined
	// object is stored at. The user is expected to do that.
	// ie, delete actual object list holds in WM_DESTROY
	// this only is related to storing items, not to recording 
	// info about number of windows, 

	if (iter != UserParamMap.end())
	{	// erase if exists. User must clean up his own memory before.
		UserParamMap.erase(iter);
	}

	unregister_window(ahwnd); // will only decrement if was registered
	
	DestroyWindow(ahwnd); // destroy window as requested regardless of origin
}


void wgWinMaker::unregister(HWND _hWnd){

	wgWinMakerBase::unregister_window(_hWnd);
}


void wgWinMaker::update_registry(HWND _hWnd)
{
	wgWinMakerBase::update_registry(_hWnd);
}


void wgWinMaker::unregister_wgWinMaker()
{
	wgWinMakerBase::unregister_wgWinMaker();
}



void wgWinMaker::register_wgWinMaker()
{
	wgWinMakerBase::register_wgWinMaker();
}
				


/////////////////////    end of code to wgWinMaker   /////////////