#ifndef _WIN32_WINNT 
	#define _WIN32_WINNT 0x0400
#endif//_WIN32_WINNT
// fix for mfc
#if defined(WIN32)
	#ifdef _AFXDLL
		#include <afx.h>
	#else
		#include<windows.h>
	#endif
#endif	

#include "winglib/wgWindow.h"
#include <winutils/wuRegistry.h>


//using namespace wu;
// global static functions here on top






//////////////////////////////////////////////////////////////////////////
/////////////////    SWINDOWS CLASS IMPLEMENTATION       /////////////////
//////////////////////////////////////////////////////////////////////////

const UINT wgWindow::CREATE_APP_SUBCLASSING(RegisterWindowMessage("CREATE_EXCEL_SUBCLASSING"));

wgWindow::wgWindow(HINSTANCE _hInstance, bool _blnUseDialogDefaultProc) throw (wuException)
{
	winmaker = 0;
	wgWinBase::hInstance = _hInstance;
	wgWinBase::hWnd = 0;
	created=0;
	blnUseDialogDefaultProc = _blnUseDialogDefaultProc;

	if (blnUseDialogDefaultProc)
	{
		hDefWinProc = DefDlgProc;
		hWinProc = _DlgWinProc;
	}
	else
	{
		hDefWinProc = DefWindowProc;
		hWinProc = _WinProc;
	}
}



wgWindow::wgWindow(HINSTANCE _hInstance, 
					 LPCTSTR ClassName, 
					 UINT style,
					 int MenuName, 
					 HICON LargeIcon, 
					 HICON SmallIcon, 
					 HBRUSH brush, 
					 HCURSOR cursor, 
					 int ClassExtra,
					 bool _blnUseDialogDefaultProc
					 ) throw (wuException){

	BOOL class_exists = FALSE;
	WNDCLASSEX existing_class;
	winmaker = 0;
	wgWinBase::hInstance = _hInstance;
	wgWinBase::hWnd = 0;
	created=0;// boolean for is window created
	ATOM atom;
	blnUseDialogDefaultProc = _blnUseDialogDefaultProc;

	if (blnUseDialogDefaultProc)
	{
		hDefWinProc = DefDlgProc;
		hWinProc = _DlgWinProc;
	}
	else
	{
		hDefWinProc = DefWindowProc;
		hWinProc = _WinProc;
	}

	class_exists = GetClassInfoEx(hInstance, ClassName, &existing_class);

	if (class_exists)	// don't register if already exists
	{
		try
		{
			winmaker = new wgWinMaker(existing_class);
		}
		catch(...)
		{
			throw wuException("memory allocation error");
		}
	}
	else
	{			

		try
		{
			winmaker = new wgWinMaker(	// all windows are the same type so we only need one 
				_hInstance,								
				sizeof(WNDCLASSEX),						// automatic
				ClassName,								
				(WNDPROC)hWinProc,								// automatic
				brush,	
				cursor,			
				LargeIcon,		
				SmallIcon,									
				(MenuName==NO_MENU)?NULL:MAKEINTRESOURCE(MenuName),								
				style,
				ClassExtra,
				4);	// always give 4 bytes for the window create extra for the pointer 
		}
		catch(...)
		{
			// cleanup bad new statement
			throw wuException("memory allocation error");
		}
										// if didn't already exist, now register it
		atom = winmaker->register_class();
		if (atom == 0) throw wuException("class register error, see ::GetLastError");
	} // end of else
}






wgWindow::~wgWindow(){

	if (created && hWnd!=0) ::SendMessage(hWnd, WM_CLOSE, 0, 0); // destroy window
				 // so that after class is destroyed, no more messages come forth
	delete winmaker;
}





HWND wgWindow::Create(char* window_title, DWORD style, int x, int y, int nWidth, int nHeight,
						HMENU menu, DWORD _dwExStyle, HWND _Parent)throw (wuException){

	char hbuff[100]={0};

	if (winmaker==0) throw wuException("Window Not Registered or wgWinMaker Bad");
	if (!created){

		wgWinBase::hWnd=winmaker->create_window(	window_title, style, 
													menu, (void*)this,
													x, y, nWidth, nHeight,
													_dwExStyle, _Parent
												);

		if (hWnd!=0) created=1; // later, reset created to NULL after destroyed
		return hWnd;
	}	
	else return 0;
}




BOOL wgWindow::Show(int nCmdShow){

	return ShowWindow(hWnd, nCmdShow);
}



unsigned int wgWindow::GetNumOpenWindows(){

	return winmaker->GetNumOpenWindows();
}



LRESULT CALLBACK wgWindow::CommonControlSwitch(HWND _hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{

	LRESULT rval=1; // application has not processed this message

	switch(message)
	{
		case WM_SIZE:
			rval = _OnSize(wParam, lParam);
			break;

		case WM_SIZING:
			rval = OnSizing(wParam, lParam);
			break;

		case WM_MOVE:
			rval = _OnMove(wParam, lParam);
			break;

		case WM_MOVING:
			rval = OnMoving(wParam, lParam);
			break;			

		case WM_COMMAND:
			rval = OnCommand(wParam, lParam);
			break;

		case WM_NOTIFY:
			rval = OnNotify(wParam, lParam);
			break;

		case WM_SHOWWINDOW:
			rval = _OnShowWindow((BOOL)wParam , (int)lParam);
			break;
	
		case WM_CHAR:
			rval = OnChar(wParam, lParam);
			break;

		case WM_KEYDOWN:
			rval = OnKeyDown(wParam, lParam);
			break;

		case WM_KEYUP:
			rval = OnKeyUp(wParam, lParam);
			break;

		case WM_SYSKEYDOWN:
			rval = OnSysKeyDown(wParam, lParam);
			break;

		case WM_SYSKEYUP:
			rval = OnSysKeyUp(wParam, lParam);
			break;

		case WM_PAINT:
			rval = OnPaint();
			break;

		case WM_CLOSE:
			rval = _OnClose();
			break;

		case WM_DESTROY:
			rval = _OnDestroy();
			break;

		case WM_NCDESTROY:
			rval = OnNCDestroy();
			break;

		// add more message handlers here
		default:
			rval = OnOther(message, wParam, lParam);
	}; // end of case

	return rval;	// return zero in a window message process function usually means 'processed message'
}




LRESULT CALLBACK wgWindow::_DlgWinProc(HWND _hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	wgWindow* control = (wgWindow*)GetWindowLong (_hWnd, GWL_USERDATA);	
	LRESULT lrRes = 1;

	switch(message)
	{
		case WM_CREATE:
			try{
				LPARAMS* params=((LPARAMS*)((CREATESTRUCT*)lParam)->lpCreateParams); // on create, get owner wgWindow passed through at create call
				SetWindowLong(_hWnd, GWL_USERDATA, (LONG)(params->user_param));  // store owner wgWindow in window info storage
				((wgWindow*)params->user_param)->_OnCreate(_hWnd);	// later on we go straight to owner wgWindow handlers
			}
			catch(...){
				::MessageBox(NULL, "error in WM_CREATE", "WARNING", MB_OK);
			}
			// no need to store the winmaker, cause we already have it back in the wgWindow class instance
			break;

		default:
			
			if (control!=0)
			{
					lrRes = control->CommonControlSwitch(_hWnd, message, wParam, lParam);
			}
			else return DefDlgProc(_hWnd, message, wParam, lParam);

	}; // end of case

	if (lrRes!=0)
	{
		return DefDlgProc(_hWnd, message, wParam, lParam);
	}
	else return 0;	// return zero in a window message process function usually means 'processed message'
}




LRESULT CALLBACK wgWindow::_WinProc(HWND _hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	wgWindow* control = (wgWindow*)GetWindowLong (_hWnd, GWL_USERDATA);	
	LRESULT lrRes = 1;
 
	switch(message)
	{
		case WM_CREATE:
			try{
				LPARAMS* params=((LPARAMS*)((CREATESTRUCT*)lParam)->lpCreateParams); // on create, get owner wgWindow passed through at create call
				SetWindowLong(_hWnd, GWL_USERDATA, (LONG)(params->user_param));  // store owner wgWindow in window info storage
				((wgWindow*)params->user_param)->_OnCreate(_hWnd);	// later on we go straight to owner wgWindow handlers
			}
			catch(...){
				::MessageBox(NULL, "error in WM_CREATE", "WARNING", MB_OK);
			}
			// no need to store the winmaker, cause we already have it back in the wgWindow class instance
			break;

		default:
			
			if (control!=0)
			{
					lrRes = control->CommonControlSwitch(_hWnd, message, wParam, lParam);
			}
			else return DefWindowProc(_hWnd, message, wParam, lParam);

	}; // end of case

	if (lrRes!=0)
	{
		return DefWindowProc(_hWnd, message, wParam, lParam);
	}
	else return 0;	// return zero in a window message process function usually means 'processed message'
}






LRESULT wgWindow::_OnCreate(HWND _hWnd){

	wgWinBase::hWnd = _hWnd;
	RECT MyRect;
	GetWindowRect(hWnd, &MyRect);

	SWin_Width = MyRect.right - MyRect.left;
	SWin_Height= MyRect.bottom - MyRect.top;
	SWin_Left= MyRect.left;
	SWin_Top= MyRect.top;

	winmaker->update_registry(hWnd);

	return OnCreate();
}




LRESULT wgWindow::_OnSize(WPARAM wParam, LPARAM lParam){

	// set size info for this window
	SWin_Width = LOWORD(lParam);
	SWin_Height = HIWORD(lParam);

	return OnSize(wParam, lParam);
}

LRESULT wgWindow::_OnMove(WPARAM wParam, LPARAM lParam){

	// set size info for this window
	SWin_Left = LOWORD(lParam);
	SWin_Top = HIWORD(lParam);

	return OnMove(wParam, lParam);
}



LRESULT wgWindow::_OnShowWindow(BOOL fShow, int fnStatus){

	RECT rect;
	GetClientRect(hWnd,  &rect);
	SWin_Width = rect.right - rect.left;
	SWin_Height = rect.bottom - rect.top;

	return OnShowWindow(fShow, fnStatus);
}



void wgWindow::Cleanup(){

	created = 0;
	winmaker->unregister(hWnd);
	// we should do this unregister here because of the chance of OnClose being
	// bypassed
}

  

LRESULT wgWindow::_OnClose()
{
	LRESULT lrRes = 1;

	lrRes = OnClose();
	winmaker->close(hWnd); // executes DestroyWindow

	return lrRes;
}

LRESULT wgWindow::OnDestroy(){

	if (winmaker->GetNumOpenWindows()<1) 
	{
		winmaker->unregister_wgWinMaker();
		PostQuitMessage(0);
	}	

	return 0;
}


LRESULT wgWindow::_OnDestroy()
{
	Cleanup();	
	return OnDestroy();
}




WindowProcessor wgWindow::GetWindowMessageHandler()
{
	return hWinProc;
}


