#include "winglib/wgWindowTimer.h"





wgWindowTimer::wgWindowTimer(){

	char mutex1[100]={0};
	char mutex2[100]={0};

	sprintf(mutex1, "wgwgWindowTimerMutex%d", (int)this);
	sprintf(mutex2, "wgwgWindowTimerEventMutex%d", (int)this);

	wgwgWindowTimerMutex = CreateMutex(NULL, FALSE, mutex1); // make multithread safe
	wgwgWindowTimerEventMutex = CreateMutex(NULL, FALSE, mutex2); // make multithread safe
	/// mutex is object in scope, through use of unique names
}

		

wgWindowTimer::~wgWindowTimer()
{
	Cleanup();

	CloseHandle(wgwgWindowTimerMutex);
	CloseHandle(wgwgWindowTimerEventMutex);
}


UINT wgWindowTimer::SetTimer(UINT idEvent, UINT ElapsePeriod)
{
	wgWindowTimerInfo* TempwgWindowTimerInfo = 0;
	TimerMapType::iterator iter;
	iter = mTimerMap.find(idEvent);

	if (iter != mTimerMap.end())
	{
		TempwgWindowTimerInfo = iter->second;
		TempwgWindowTimerInfo->NextElapsePeriod = ElapsePeriod;
		return 1;
	}
	else
	{
		TempwgWindowTimerInfo = new wgWindowTimerInfo();
		TempwgWindowTimerInfo->CurrentElapsePeriod = ElapsePeriod;
		TempwgWindowTimerInfo->NextElapsePeriod = ElapsePeriod;
		TempwgWindowTimerInfo->AssociatedObject = this;
		TempwgWindowTimerInfo->idEvent = idEvent;
		mTimerMap[idEvent]=TempwgWindowTimerInfo;//TimerList.Add(idEvent, TempwgWindowTimerInfo);
	}

	return ::SetTimer(hWnd, (UINT)TempwgWindowTimerInfo, TempwgWindowTimerInfo->CurrentElapsePeriod, TimerWindowProc);
}


BOOL wgWindowTimer::KillTimer(UINT idEvent)
{
	WaitForSingleObject(wgwgWindowTimerMutex, 120000); // ask for ownership

	wgWindowTimerInfo* TempwgWindowTimerInfo = 0;
	BOOL Result = FALSE;
	TimerMapType::iterator iter;

	iter = mTimerMap.find(idEvent);	

	if (iter != mTimerMap.end())
	{
		TempwgWindowTimerInfo = iter->second;
		Result = ::KillTimer(hWnd, (unsigned int)TempwgWindowTimerInfo);
		mTimerMap.erase(iter);
		delete TempwgWindowTimerInfo;
	}
	else
	{
		Result = FALSE;
	}

	ReleaseMutex(wgwgWindowTimerMutex);
	return Result;
}

// should have some mutex's between KillTimer, and TimerWindowProc

VOID CALLBACK wgWindowTimer::TimerWindowProc(HWND hwnd, UINT uMsg, UINT idEvent, DWORD dwTime){
 	
	if (idEvent!=0){

		wgWindowTimerInfo* TempwgWindowTimerInfo = 0;
		wgWindowTimer* MyTimer = 0;
		HANDLE wgwgWindowTimerMutex2 =0;
		HANDLE wgwgWindowTimerEventMutex2=0;

		TempwgWindowTimerInfo = (wgWindowTimerInfo*)idEvent;
		MyTimer = (wgWindowTimer*)TempwgWindowTimerInfo->AssociatedObject;
		
		// kill timer temporarily - experimentatal code
		::KillTimer(MyTimer->hWnd, idEvent);

		// Re-Create a handle to the mutex's
		char mutex1[100]={0};
		char mutex2[100]={0};

		sprintf(mutex1, "wgwgWindowTimerMutex%d", (int)MyTimer);
		sprintf(mutex2, "wgwgWindowTimerEventMutex%d", (int)MyTimer);

		wgwgWindowTimerMutex2 = CreateMutex(NULL, FALSE, mutex1); // make multithread safe
		wgwgWindowTimerEventMutex2 = CreateMutex(NULL, FALSE, mutex2); // make multithread safe
	
		// Wait on the Objects mutex if need be
		WaitForSingleObject(wgwgWindowTimerMutex2, 120000); // ask for ownership

		TempwgWindowTimerInfo->CurrentElapsePeriod = TempwgWindowTimerInfo->NextElapsePeriod;

		ReleaseMutex(wgwgWindowTimerMutex2);

		WaitForSingleObject(wgwgWindowTimerEventMutex2, 120000);

		// CALL THE ONWINDOWTIMER EVENT
		MyTimer->OnwgWindowTimer(TempwgWindowTimerInfo->idEvent); // while in the event, other process may change the Timer attributes without getting stuck
															// in mutex
		ReleaseMutex(wgwgWindowTimerEventMutex2);
											
		WaitForSingleObject(wgwgWindowTimerMutex2, 120000); // ask for ownership

		// now check if timer is still valid
		// if it isn't, the memory for that would already have been taken care of, 
		// so don't touch the original in any case. Because it may have already been re-instated at another address.
		// If re-instated we will need to re get the handle. If it wasn't the associated handle will not exist, and we will not re-call the Timer function
		TimerMapType::iterator iter = MyTimer->mTimerMap.find(TempwgWindowTimerInfo->idEvent);
		
		if (iter != MyTimer->mTimerMap.end())
		{
			TempwgWindowTimerInfo = iter->second;
			::SetTimer(MyTimer->hWnd, (UINT)TempwgWindowTimerInfo, TempwgWindowTimerInfo->CurrentElapsePeriod, TimerWindowProc);
		}

		ReleaseMutex(wgwgWindowTimerMutex2);
		CloseHandle(wgwgWindowTimerMutex2);
		CloseHandle(wgwgWindowTimerEventMutex2);
	}
}



int wgWindowTimer::Cleanup()
{
	wgWindowTimerInfo* TempwgWindowTimerInfo = 0;

	TimerMapType::iterator iter = mTimerMap.begin();

	try
	{
		while (iter != mTimerMap.end())
		{
			TempwgWindowTimerInfo = iter->second;

			if (TempwgWindowTimerInfo!=0)
			{
				KillTimer(TempwgWindowTimerInfo->idEvent);
				delete TempwgWindowTimerInfo;
				TempwgWindowTimerInfo = 0;
			}

			iter++;
		}
	}
	catch(exception& e){

	}

	mTimerMap.clear();

	return FALSE;
} 

