

#ifndef _WU_EXCEPTION
#define _WU_EXCEPTION

#include <stdexcept>
#include <string>

 


////////////////////////////////////////////////////////////////////////
///						WU_EXCEPTION								  
////////////////////////////////////////////////////////////////////////
/// wuException class is the base exception class used within all WU applications.
/// All other WU exception types inherit from this class, and this class, in turn, inherits from std::exception
/// Code includes a little known fix for windows exception handling. See web page
/// http://www.boost.org/more/error_handling.html 
/// which looks like this:
///
///	extern "C" void straight_to_debugger(unsigned int, EXCEPTION_POINTERS*)<br>
///	{<br>
///	    throw;<br>
///	}<br>
///	extern "C" void (*old_translator)(unsigned, EXCEPTION_POINTERS*) <br>
///	         = _set_se_translator(straight_to_debugger);<br>
///



class wuException : public std::exception{

	public:	
		
			/// wuException main constructor
			/// \param amessage The exception message that is passed to the exception handler
		wuException(const char* amessage) throw();	

			/// wuException copy constructor
		wuException(const wuException& other) throw();	

			/// wuException destructor
		virtual ~wuException() throw();							

	//	virtual const char* what() const throw();

	protected:

/* ///FOR GCC VERSION ONLY
		std::string _M_msg;
*/
	
};






#endif //  _WU_EXCEPTION




