

//Things to do:

//1. ftp class accepts only i.p. Make it accept host name

// NOTE "599 Application Error: SendFromStream Unsuccessfull" is application defined error code




#include "winutils/wuTCPSocket.h" 
#include "winutils/wuException.h"
#include "winutils/wuFtpState.h"


#ifndef _WU_FTP_SOCKET
#define _WU_FTP_SOCKET

// SOCKET DEFAULTS

	#define _FTP_SOCKET_TIMEOUT 5000
	#define _FTP_SOCKET_RETRIES 10

// END SOCKET DEFAULTS


#pragma warning( disable : 4290 ) 





// GENERAL DEFAULTS

	#define _MAX_LINE 1024

// END GENERAL DEFAULTS






///
///	wuFtpSocket is used to control Input & Output to ftp servers. 
/// If you want to build an ftp client. Use this. 
///
class wuFtpSocket{

	public:

	// public member functions

		wuFtpSocket();
		~wuFtpSocket();
		wuFtpSocket(const char* _cstrHost, unsigned int _uintHostPort) throw(wuSocketNoHost, wuSocketBadRead, wuSocketException, exception);
		void ApplicationName(const std::string& strAppName);

		void Connect(const char* _cstrHost, unsigned int _uintHostPort) throw(wuSocketNoHost, wuSocketBadRead, wuSocketException, exception);
		void Connect() throw(wuSocketNoHost, wuSocketBadRead, exception);

		int Close();

		void SetTimeout(unsigned int _uintTimeout) throw();
		void SetReconnectionRetries(unsigned int _uintRetries);

	// command specific functions
	// NOTE *** All reply messages may be retrieved through the function GetLastReply ***

		// ALL commands below return BOOL for Follow up information required (eg. first code num is 3, or sometimes 1)
		// STOR, and RETR set state to 1xx if all is well so far. And server sends further info after file is sent. State then will be 2xx

		void SendRequest(const char* pcharRequestString, unsigned int uintRequestStringLen) throw(wuSocketException, wuSocketInterrupt, exception);

		BOOL SendUserRequest(const char* _cstrUserName) throw (wuSocketException, std::exception);
		BOOL SendRNFRRequest(const char* _Path) throw (wuSocketException, std::exception);
		BOOL SendSTORRequest(const char* _Path) throw (wuSocketException, std::exception);
		BOOL SendRETRRequest(const char* _Path) throw (wuSocketException, std::exception);

	// these commands return TRUE on Success, FALSE on fail
		BOOL SendPasswordRequest(const char* _cstrPassword) throw (wuSocketException, std::exception);
		BOOL SendRNTORequest(const char* _Path) throw (wuSocketException, std::exception);
		BOOL SendQuitRequest() throw (wuSocketException, std::exception);
		BOOL SendNoOpRequest() throw (wuSocketException, std::exception);
		BOOL SendDummyRequest() throw (wuSocketException, std::exception);
		BOOL SendTypeRequest(const char _type) throw (wuSocketException, std::exception);// legal types include 'A' , 'E' ,'I' 
		BOOL SendCWDRequest(const char* _Path) throw (wuSocketException, std::exception);// change current working directory 
		BOOL SendCDUPRequest() throw (wuSocketException, std::exception);
		BOOL SendReInitializeRequest() throw (wuSocketException, std::exception);
		BOOL SendPWDRequest() throw (wuSocketException, std::exception);
		BOOL SendPASVRequest(std::string& strHostIP, unsigned int& _uintPort ) throw (wuSocketException, std::exception);


		// these commands have various return types and values
		// various higher level, user friendly commands
		BOOL FtpLogin(const char* LoginName, const char* Password) throw (wuSocketException, std::exception);
		BOOL Rename(const char* SourceName, const char* DestinationName) throw (wuSocketException, std::exception);
		BOOL Move(const char* SourceName, const char* DestinationName) throw (wuSocketException, std::exception);
		BOOL SendDataPasv(FILE* _SourceStream, const char* _UploadFileName, const char _Type='i') throw (wuSocketException, std::exception);
		BOOL RecieveDataPasv(FILE* _DestinationStream, const char* _DownloadFileName, const char _Type) throw (wuSocketException, std::exception);



		inline wuFtpState GetLastReply(){return LastReply;}
		BOOL IsConnected();
		BOOL IsLoggedOn();
 
	// public member variables

		static const std::string TELNET_EOF;

	private:

	// private member variables
	
		char* cstrHost;	 // ip address only. if in future, this supports hostname, add extra member variable as we need this ip for use in PASV mode IPv6
		unsigned int uintHostPort;
		wuTCPSocket fSocket;
		unsigned int uintTimeout;
		unsigned int uintRetries;
		wuFtpState LastReply;
	
	// private member functions
		

		void SetHost(const char* _cstrHost, unsigned int _uintHostPort) throw(wuSocketException);

		// this function recieves from the socket, the ftp server's reply, and stores it in a known 
		// variable of this class, LastReply.		
		void GetReply() throw(wuSocketBadRead, wuSocketWaitTimeout, exception);
		BOOL ExtractFtpHostNPort(const std::string& _strServerReplyMessage, std::string& _strHostIP, unsigned int& _uintPort);
		std::string StripEndsOfNonNumerals(const std::string _strLine);
		int SendFromStream(wuSocket* _Destination, FILE* _SourceStream ) throw(wuSocketException);
		int RecieveToStream(FILE* _DestinationStream, wuSocket* _Source) throw(wuSocketException);
};


#endif//_WU_FTP_SOCKET



/*
	INFORMATION *** FTP REPLY CODES ***  AS PER RFC 959
	http://www.freesoft.org/CIE/RFC/959/index.htm

There are five values for the first digit of the reply code: 


1yz Positive Preliminary reply
The requested action is being initiated; expect another reply before proceeding with a new command. (The user-process sending another command before the completion reply would be in violation of protocol; but server-FTP processes should queue any commands that arrive while a preceding command is in progress.) This type of reply can be used to indicate that the command was accepted and the user-process may now pay attention to the data connections, for implementations where simultaneous monitoring is difficult. The server-FTP process may send at most, one 1yz reply per command. 


2yz Positive Completion reply
The requested action has been successfully completed. A new request may be initiated. 


3yz Positive Intermediate reply
The command has been accepted, but the requested action is being held in abeyance, pending receipt of further information. The user should send another command specifying this information. This reply is used in command sequence groups. 


4yz Transient Negative Completion reply
The command was not accepted and the requested action did not take place, but the error condition is temporary and the action may be requested again. The user should return to the beginning of the command sequence, if any. It is difficult to assign a meaning to "transient", particularly when two distinct sites (Server- and User-processes) have to agree on the interpretation. Each reply in the 4yz category might have a slightly different time value, but the intent is that the user-process is encouraged to try again. A rule of thumb in determining if a reply fits into the 4yz or the 5yz (Permanent Negative) category is that replies are 4yz if the commands can be repeated without any change in command form or in properties of the User or Server (e.g., the command is spelled the same with the same arguments used; the user does not change his file access or user name; the server does not put up a new implementation.) 


5yz Permanent Negative Completion reply
The command was not accepted and the requested action did not take place. The User-process is discouraged from repeating the exact request (in the same sequence). Even some "permanent" error conditions can be corrected, so the human user may want to direct his User-process to reinitiate the command sequence by direct action at some point in the future (e.g., after the spelling has been changed, or the user has altered his directory status.) 

The following function groupings are encoded in the second digit: 


x0z Syntax 
These replies refer to syntax errors, syntactically correct commands that don't fit any functional category, unimplemented or superfluous commands. 

x1z Information 
These are replies to requests for information, such as status or help. 

x2z Connections 
Replies referring to the control and data connections. 

x3z Authentication and accounting 
Replies for the login process and accounting procedures. 

x4z 
Unspecified as yet. 

x5z File system 
These replies indicate the status of the Server file system vis-a-vis the requested transfer or other file system action. 
The third digit gives a finer gradation of meaning in each of the function categories, specified by the second digit. The list of replies below will illustrate this. Note that the text associated with each reply is recommended, rather than mandatory, and may even change according to the command with which it is associated. The reply codes, on the other hand, must strictly follow the specifications in the last section; that is, Server implementations should not invent new codes for situations that are only slightly different from the ones described here, but rather should adapt codes already defined. 

A command such as TYPE or ALLO whose successful execution does not offer the user-process any new information will cause a 200 reply to be returned. If the command is not implemented by a particular Server-FTP process because it has no relevance to that computer system, for example ALLO at a TOPS20 site, a Positive Completion reply is still desired so that the simple User-process knows it can proceed with its course of action. A 202 reply is used in this case with, for example, the reply text: "No storage allocation necessary." If, on the other hand, the command requests a non-site-specific action and is unimplemented, the response is 502. A refinement of that is the 504 reply for a command that is implemented, but that requests an unimplemented parameter. 


4.2.1 Reply Codes by Function Groups

         200 Command okay.
         500 Syntax error, command unrecognized.
             This may include errors such as command line too long.
         501 Syntax error in parameters or arguments.
         202 Command not implemented, superfluous at this site.
         502 Command not implemented.
         503 Bad sequence of commands.
         504 Command not implemented for that parameter.
          
         110 Restart marker reply.
             In this case, the text is exact and not left to the
             particular implementation; it must read:
                  MARK yyyy = mmmm
             Where yyyy is User-process data stream marker, and mmmm
             server's equivalent marker (note the spaces between markers
             and "=").
         211 System status, or system help reply.
         212 Directory status.
         213 File status.
         214 Help message.
             On how to use the server or the meaning of a particular
             non-standard command.  This reply is useful only to the
             human user.
         215 NAME system type.
             Where NAME is an official system name from the list in the
             Assigned Numbers document.
          
         120 Service ready in nnn minutes.
         220 Service ready for new user.
         221 Service closing control connection.
             Logged out if appropriate.
         421 Service not available, closing control connection.
             This may be a reply to any command if the service knows it
             must shut down.
         125 Data connection already open; transfer starting.
         225 Data connection open; no transfer in progress.
         425 Can't open data connection.
         226 Closing data connection.
             Requested file action successful (for example, file
             transfer or file abort).
         426 Connection closed; transfer aborted.
         227 Entering Passive Mode (h1,h2,h3,h4,p1,p2).
          
         230 User logged in, proceed.
         530 Not logged in.
         331 User name okay, need password.
         332 Need account for login.
         532 Need account for storing files.
          
         150 File status okay; about to open data connection.
         250 Requested file action okay, completed.
         257 "PATHNAME" created.
         350 Requested file action pending further information.
         450 Requested file action not taken.
             File unavailable (e.g., file busy).
         550 Requested action not taken.
             File unavailable (e.g., file not found, no access).
         451 Requested action aborted. Local error in processing.
         551 Requested action aborted. Page type unknown.
         452 Requested action not taken.
             Insufficient storage space in system.
         552 Requested file action aborted.
             Exceeded storage allocation (for current directory or
             dataset).
         553 Requested action not taken.
             File name not allowed.


4.2.2 Numeric Order List of Reply Codes

         110 Restart marker reply.
             In this case, the text is exact and not left to the
             particular implementation; it must read:
                  MARK yyyy = mmmm
             Where yyyy is User-process data stream marker, and mmmm
             server's equivalent marker (note the spaces between markers
             and "=").
         120 Service ready in nnn minutes.
         125 Data connection already open; transfer starting.
         150 File status okay; about to open data connection.
          
         200 Command okay.
         202 Command not implemented, superfluous at this site.
         211 System status, or system help reply.
         212 Directory status.
         213 File status.
         214 Help message.
             On how to use the server or the meaning of a particular
             non-standard command.  This reply is useful only to the
             human user.
         215 NAME system type.
             Where NAME is an official system name from the list in the
             Assigned Numbers document.
         220 Service ready for new user.
         221 Service closing control connection.
             Logged out if appropriate.
         225 Data connection open; no transfer in progress.
         226 Closing data connection.
             Requested file action successful (for example, file
             transfer or file abort).
         227 Entering Passive Mode (h1,h2,h3,h4,p1,p2).
         230 User logged in, proceed.
         250 Requested file action okay, completed.
         257 "PATHNAME" created.
          
         331 User name okay, need password.
         332 Need account for login.
         350 Requested file action pending further information.
          
         421 Service not available, closing control connection.
             This may be a reply to any command if the service knows it
             must shut down.
         425 Can't open data connection.
         426 Connection closed; transfer aborted.
         450 Requested file action not taken.
             File unavailable (e.g., file busy).
         451 Requested action aborted: local error in processing.
         452 Requested action not taken.
             Insufficient storage space in system.
          
         500 Syntax error, command unrecognized.
             This may include errors such as command line too long.
         501 Syntax error in parameters or arguments.
         502 Command not implemented.
         503 Bad sequence of commands.
         504 Command not implemented for that parameter.
         530 Not logged in.
         532 Need account for storing files.
         550 Requested action not taken.
             File unavailable (e.g., file not found, no access).
         551 Requested action aborted: page type unknown.
         552 Requested file action aborted.
             Exceeded storage allocation (for current directory or
             dataset).
         553 Requested action not taken.
             File name not allowed.
         




*/


