#ifndef wuSysLogFile_H
#define wuSysLogFile_H


//--------------------------------------------------------------


// INCLUDES
//

#include "winutils/wuFile.h"
#include <string>
#include "winutils/wuSysLogger.h"


//--------------------------------------------------------------

///
/// wuSysLogFile class provides the functionality to log syslog format logs to a text file. 
/// This logfile will write logs of a filename with an order index just prior to the suffix .log. 
/// This index has possible values from 001 to 999, and once 999 is reached, begins again. 
/// Using this logfile generator one can limit the number of files, and the size of each file, therefore
/// limiting the total size of logs. A logfile will roll once the max_size is reached, and also
/// when there is a new day. This class maintains the max log files limit regardless of rolling days. 
/// In otherwords, if limited to 10 log files, after 2, 3, 4 ... days, the number of log files present 
/// is guaranteed to be 10 or less.
///
class wuSysLogFile : public wuFile, public wuSysLogger
{ 
	public:

	// MAIN FUNCTIONALITY

			/// Default constructor
		wuSysLogFile(void);

			/// Constructor
		wuSysLogFile(	std::string			strAppName,								///< Application name
						std::string			strLogFileUniqueName,					///< Unique Log filename string
						std::string			strFilePath,							///< File path, absolute or relative
						SysLogSeverityCode	objSeverityCode,						///< Least Significant Severity Code to log to.
						unsigned long		lngMaxFileSize,							///< Max file size per file (bytes)
						unsigned int		intMaxNumFiles,							///< Maximum number of files at any one time.
						unsigned int		uintWinsockSignificantVersNum=2,		///< version of winsock (significant digit)
						unsigned int		uintWinsockInsignificantVersNum=2		///< version of winsock (insignificant digit)
					);

			/// Destructor
		virtual ~wuSysLogFile(void);

			/// initialise settings
			/// Note: max file size is inputed in bytes
		void InitialiseSettings(std::string			strAppName,								///< Application name
								std::string			strLogFileUniqueName,					///< Unique Log filename string
								std::string			strFilePath,							///< File path, absolute or relative
								SysLogSeverityCode	objSeverityCode,						///< Least Significant Severity Code to log to.
								unsigned long		lngMaxFileSize,							///< Max file size per file (bytes)
								unsigned int		intMaxNumFiles,							///< Maximum number of files at any one time.
								unsigned int		uintWinsockSignificantVersNum = 2,		///< version of winsock (significant digit)
								unsigned int		uintWinsockInsignificantVersNum = 2		///< version of winsock (insignificant digit)
							);	

			/// Open log file
			/// \param blnNewFile Create as a new file if true. Otherwise continue existing file.
		virtual bool Open(bool blnNewFile = false);

			/// Overriden function from wuSysLogger
			/// \param Facility indicates the Facility code as defined in RFC 3164.
			/// \param Severity indicates the Severity code as defined in RFC 3164.
			/// \param cpcharMessageContent is the user defined message to log to the file. This is
			/// used as the CONTENT of the MSG part of a syslog as defined in  RFC 3164.
		bool Log(	SysLogFacilityCode Facility, 
					SysLogSeverityCode Severity, 
					const char* cpcharMessageContent);

		SysLogSeverityCode GetLogSeverityLevel();

	// HELPER FUNCTIONS

			/// Roll to next file
		void RollToNextFile(void);

	protected:

		bool Log(const char* LogString);

		void SetFileName(void);

		void LimitNumLogFiles(void);

	private:

		std::string				mstrAppName;			///< application name
		std::string				mstrLogFileUniqueName;
		SYSTEMTIME				mSYSTIMELastWriteDate;		///< last write date
		SysLogSeverityCode		mobjSeverityCode;
		unsigned long			mlngMaxFileSize;		///< Size in bytes
		unsigned int			mintMaxNumFiles;
		unsigned int			mintFileCount;			///< counts number of files today
};


//--------------------------------------------------------------


#endif //wuSysLogFile

