
#include <yvals.h>              // warning numbers get enabled in yvals.h 
#pragma warning(disable: 4786)  // identifier was truncated to 'number' characters in 
                                // the debug information
	// BUG: C4786 Warning Is Not Disabled with #pragma Warning
	// STATUS: Microsoft has confirmed this to be a bug in the Microsoft product. 
	// This warning can be ignored. This occured only in the <map> container.
#include <hash_map>
#include "winutils/wuSysLogger.h"


#ifndef _WU_SYS_LOG_SET
#define _WU_SYS_LOG_SET




struct SysLoggerInfo
{
	SysLoggerInfo(wuSysLogger* SysLogger, bool blnCleanupSysLogger)
	{
		mSysLogger = SysLogger;
		mblnCleanupSysLogger = blnCleanupSysLogger;
	}

	wuSysLogger* mSysLogger;
	bool mblnCleanupSysLogger;
};




typedef std::hash_multimap<std::string, SysLoggerInfo> LoggerListType;



///
/// wuSysLogSet allows the control of multiple SysLoggers via one set object. 
///
class wuSysLogSet
{
	public:

		wuSysLogSet();

		~wuSysLogSet();


		void Add(wuSysLogger* Logger, bool blnCleanup);
		void Add(std::string strLoggerKey, wuSysLogger* Logger, bool blnCleanup);

		void ClearSet();

			/// initialise the Syslogger.
		void Initialise(const char* cpcharProcName,				///< The process name. This argument is to be used as the TAG field of the MSG part of the syslog as defined in RFC 3164.
						unsigned int uintWinsockSigVersNum,		///< version of winsock (significant digit)		
						unsigned int uintWinsockInsigVersNum	///< version of winsock (insignificant digit)		
					);

			/// the Log function organises the logging of this message to all SysLoggers in this set. 
			/// \param Facility indicates the Facility code as defined in RFC 3164.
			/// \param Severity indicates the Severity code as defined in RFC 3164.
			/// \param cpcharMessageContent is the user defined message to log to the file. This is
			/// used as the CONTENT of the MSG part of a syslog as defined in  RFC 3164.
		bool Log(	SysLogFacilityCode Facility, 
					SysLogSeverityCode Severity, 
					const char* cpcharMessageContent);

			/// the Log function organises the logging of this message to the Syslogger in this set
			/// which is associated with the key strLoggerKey. This message will also be logged with 
			/// any Syslog in this set which has an empty key string.
			/// \param strLoggerKey indicates the Syslog key associated with one or more Sysloggers in this Set.
			/// \param Facility indicates the Facility code as defined in RFC 3164.
			/// \param Severity indicates the Severity code as defined in RFC 3164.
			/// \param cpcharMessageContent is the user defined message to log to the file. This is
			/// used as the CONTENT of the MSG part of a syslog as defined in  RFC 3164.
		bool Log(	std::string strLoggerKey,
					SysLogFacilityCode Facility, 
					SysLogSeverityCode Severity, 
					const char* cpcharMessageContent);

	protected:


		bool Log(	wuSysLogger* TheLogger,
					SysLogFacilityCode Facility, 
					SysLogSeverityCode Severity, 
					const char* cpcharMessageContent);

		LoggerListType LoggerList;

	private:

};




#endif//_WU_SYS_LOG_SET

