#include <stdio.h>
#include "winutils/wuCSVLogFile.h"
#include "winutils/wuSYSTEMTIME.h"


using namespace std;


static const char* spcharEndLine="\n\0";


const char* CSVFileSuffix=".csv";



wuCSVLogFile::wuCSVLogFile(void) : wuFile ()
{
}



wuCSVLogFile::~wuCSVLogFile(void)
{
	// its ok. Close is performed by the parent class.
}


//--------------------------------------------------------------




wuCSVLogFile::wuCSVLogFile(	const std::string&	strAppName,
								const std::string&	strLogFileUniqueName,
								const std::string&	strFilePath,
								const std::string&	strDelim,
								unsigned long		lngMaxFileSize,
								unsigned int		intMaxNumFiles,
								bool				blnCacheLineIfWriteFail)
{
	InitialiseSettings(	strAppName,
						strLogFileUniqueName,
						strFilePath,
						strDelim,
						lngMaxFileSize,
						intMaxNumFiles,
						blnCacheLineIfWriteFail);
}





void wuCSVLogFile::InitialiseSettings(	const std::string&	strAppName,
										const std::string&	strLogFileUniqueName,
										const std::string&	strFilePath,
										const std::string&	strDelim,
										unsigned long		lngMaxFileSize,
										unsigned int		intMaxNumFiles,
										bool				blnCacheLineIfWriteFail)
{
	wuFile::Initialise(strFilePath, "", 1000, true, false);
	mstrAppName = strAppName;
	mstrLogFileUniqueName = strLogFileUniqueName;
	mlngMaxFileSize = lngMaxFileSize;
	mintMaxNumFiles = intMaxNumFiles;
	mintFileCount = 1;
	SetDelimeter(strDelim);
	mblnCacheLineIfWriteFail = blnCacheLineIfWriteFail;

	GetLocalTime(&mSYSTIMELastWriteDate);
}




void wuCSVLogFile::AddValue(const std::string& strValue)
{
	mstrLine = mstrLine + strValue + mstrDelim;
}


void wuCSVLogFile::SetDelimeter(const std::string& strDelim)
{
	mstrDelim = strDelim;
}





bool wuCSVLogFile::Open(bool blnNewFile)
{
	bool	blnSuccess = true;
	int		intFileCount = 0;
	char	achrDateBuffer[10] = {0};
	string	strSearch = "";
	string	strLatestFile = "";

	WIN32_FIND_DATA objFindFileData;
	HANDLE hdlFind;

	GetLocalTime(&mSYSTIMELastWriteDate);

		// if creating file but continuing form an old log file, search for the lastast log file
	if (!blnNewFile)
	{
			// get date time as string
		wuSYSTEMTIME::FormatDate(achrDateBuffer, 10, "YYYYMMDD", mSYSTIMELastWriteDate);

			// create search string
		if (mstrLogFileUniqueName == "")
		{
			strSearch = GetFilePath() + "\\" + achrDateBuffer + "_" + mstrAppName + "_*" +CSVFileSuffix;
		}
		else
		{
			strSearch = GetFilePath() + "\\" + achrDateBuffer + "_" + mstrAppName + "_" + mstrLogFileUniqueName + "_*"+CSVFileSuffix;
		}

		hdlFind = FindFirstFile(strSearch.c_str(), &objFindFileData);
		if (hdlFind != INVALID_HANDLE_VALUE)
		{
			while (blnSuccess)
			{
				strLatestFile = objFindFileData.cFileName;
				blnSuccess = FindNextFile(hdlFind, &objFindFileData);
				mintFileCount++;
			}
			mintFileCount--;
		}
		FindClose(hdlFind);
	}
	
		// reset the file name before opening
	SetFileName();
	blnSuccess = wuFile::Open();

	return (blnSuccess);
}





//--------------------------------------------------------------


bool wuCSVLogFile::Log(bool blnEndLine)
{
	int intLineLen = mstrLine.size();
	SYSTEMTIME lCurrentTime;
	GetLocalTime(&lCurrentTime);
	bool blnResult = false;

	// only if we have some characters in buffer bother with write calls
	if (intLineLen>0)
	{	// if file has reach max size or if new day, roll to the next file
		if (GetFileSize() > mlngMaxFileSize || lCurrentTime.wDay != mSYSTIMELastWriteDate.wDay)
		{
			RollToNextFile();
		}

		if (blnEndLine)
		{
			// replace the existing end comma with an endline
			mstrLine.replace(mstrLine.size()-1, 1, spcharEndLine);
		}

		blnResult = Write(mstrLine.c_str());

		if (blnResult)
		{
			mSYSTIMELastWriteDate = lCurrentTime;	
			mstrLine = "";
		}
		else if (!mblnCacheLineIfWriteFail)
		{
			mstrLine = "";
		}
	}

	return blnResult;
}



const std::string wuCSVLogFile::GetLogString()
{
	return mstrLine;
}



void wuCSVLogFile::SetLogString(const std::string& strLine)
{
	mstrLine = strLine;
}




void wuCSVLogFile::ClearLogString()
{
	mstrLine = "";
}



//--------------------------------------------------------------



void wuCSVLogFile::RollToNextFile(void)
{
	Close();
	LimitNumLogFiles();
	SYSTEMTIME lCurrentTime;
	GetLocalTime(&lCurrentTime);

		// reset the file counter if we have reached a new day or rearch the maximum 
		// number of files per day (not max number of files)
	if (lCurrentTime.wDay != mSYSTIMELastWriteDate.wDay || mintFileCount >= 999)
	{
		mintFileCount = 1;
	}
	else
	{
		mintFileCount++;
	}

		// open the new file without seaching for the latest
	Open(true);
}


//--------------------------------------------------------------



void wuCSVLogFile::SetFileName(void)
{
	char	achrDateBuffer[10] = {0};
	char	achrBuffer[1024] = {0};
	SYSTEMTIME lCurrentTime;

	GetLocalTime(&lCurrentTime);

		// get date time as string
	wuSYSTEMTIME::FormatDate(achrDateBuffer, 10, "YYYYMMDD", lCurrentTime);

		// construct log file name
	if (mstrLogFileUniqueName == "")
	{
		sprintf(achrBuffer, "%s_%s_%03u%s", achrDateBuffer, mstrAppName.c_str(), mintFileCount, CSVFileSuffix);
	}
	else
	{
		sprintf(achrBuffer, "%s_%s_%s_%03u%s", achrDateBuffer, mstrAppName.c_str(), mstrLogFileUniqueName.c_str(), mintFileCount, CSVFileSuffix);
	}
	wuFile::SetFileName(achrBuffer);
}


//--------------------------------------------------------------


void wuCSVLogFile::LimitNumLogFiles(void)
{
	bool	blnSuccess = true;
	int		intFileCount = 0;
	string	strSearch = "";
	string	strOldestFile = "";
	string	strOldestPathFile = "";
	WIN32_FIND_DATA objFindFileData;
	HANDLE hdlFind;
	FILETIME objFileOldestCreationTime;
	
	
		// create search string
	if (mstrLogFileUniqueName == "")
	{
		strSearch = GetFilePath() + "\\*_" + mstrAppName + "_*" + CSVFileSuffix;
	}
	else
	{
		strSearch = GetFilePath() + "\\*_" + mstrAppName + "_" + mstrLogFileUniqueName + "_*" + CSVFileSuffix;
	}

	hdlFind = FindFirstFile(strSearch.c_str(), &objFindFileData);
	objFileOldestCreationTime = objFindFileData.ftCreationTime;
	strOldestFile = objFindFileData.cFileName;
	if (hdlFind != INVALID_HANDLE_VALUE)
	{
		while (blnSuccess)
		{
			blnSuccess = FindNextFile(hdlFind, &objFindFileData);
			intFileCount++;
			//***
			int i = CompareFileTime(&objFileOldestCreationTime, &objFindFileData.ftCreationTime);
			int b = CompareFileTime( &objFindFileData.ftCreationTime, &objFileOldestCreationTime);
			if (CompareFileTime(&objFileOldestCreationTime, &objFindFileData.ftCreationTime) == 1)
			{
				strOldestFile = objFindFileData.cFileName;
				objFileOldestCreationTime = objFindFileData.ftCreationTime;
			}
		}
	}

	FindClose(hdlFind);

		// delete oldest file
	if (intFileCount >= mintMaxNumFiles)
	{
		strOldestPathFile = GetFilePath() + "\\" + strOldestFile;
		remove(strOldestPathFile.c_str());
	}
}


//--------------------------------------------------------------
