#include "winutils/wuCalendar.h"




static const char* WeekDays[13] = {"Sunday",
							"Monday",
							"Tuesday",
							"Wednesday",
							"Thursday",
							"Friday",
							"Saturday",
							""};


// number of days in the months (non-leap year)
static const unsigned int uintDaysInMonthReg[13] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31, 0};

static const unsigned int uintDaysSinceJan1Reg[13] = {0,31,59,90,120,151,181,212,243,273,304,334,365};



wuCalendar::wuCalendar()
{


}




wuCalendar::~wuCalendar()
{
	CleanCalendar();
}






void wuCalendar::CleanCalendar()
{
	CountryHMapType::iterator CountryFindIter;
	RegionHMapType* RegionalHMap = 0;

	CountryFindIter = mIntHolidaysMap.begin();

	while (CountryFindIter!=mIntHolidaysMap.end())
	{
		RegionalHMap = (RegionHMapType*)(CountryFindIter->second);

		if (RegionalHMap)
		{
			FreeNationalHolidays(RegionalHMap);
			delete RegionalHMap;
		}

		CountryFindIter++;
	}

	mIntHolidaysMap.clear();
}




void wuCalendar::FreeNationalHolidays(RegionHMapType* RegionalMap)
{
	RegionHMapType::iterator RHolsFindIter;
	HolidayMapType* HolidaysMap = 0;

	RHolsFindIter = RegionalMap->begin();

	while (RHolsFindIter!=RegionalMap->end())
	{
		HolidaysMap = RHolsFindIter->second;

		if (HolidaysMap)
		{
			delete HolidaysMap;
		}

		RHolsFindIter++;
	}

	RegionalMap->clear();
}





wuSYSTEMTIME wuCalendar::ConformDate(const wuSYSTEMTIME& _DateTime)
{
	wuSYSTEMTIME Temp = _DateTime;

	// next we make sure that this systemtime can not be used for anything more specific than a day
	// this does lose information, but we only use wuSYSTEMTIME for standardisation reasons
	Temp.wHour = 0;
	Temp.wMinute = 0;
	Temp.wSecond = 0;
	Temp.wMilliseconds = 0;

	// now we ensure that the week day in the wuSYSTEMTIME is correct. we don't trust the user to have
	// entered it in properly.
	Temp.wDayOfWeek = wuCalendar::GetDayOfWeek(Temp.wYear, Temp.wMonth, Temp.wDay);

	return Temp;
}







void wuCalendar::AddNonWorkDay(const wuSYSTEMTIME& Date, a3CountryCode Country, int _intRegionCode)
{
	RegionHMapType* RegionalHMap = 0;
	CountryHMapType::iterator CountryFindIter;

	CountryFindIter = mIntHolidaysMap.find(Country);

	if (CountryFindIter!=mIntHolidaysMap.end() && CountryFindIter->second!=0)
	{
		RegionalHMap = (RegionHMapType*)(CountryFindIter->second);
	}
	else  // we must here then add a National Holidays map for that CountryCode
	{
		if (CountryFindIter!=mIntHolidaysMap.end() && CountryFindIter->second==0)
		{	  // we found the country, but no valid holidays map. i.e. country had no holidays
			  // first we delete the existing entry, then add a new one.
			mIntHolidaysMap.erase(CountryFindIter);
		}

		RegionalHMap = new RegionHMapType();
		mIntHolidaysMap.insert(CountryHMapType::value_type((int)Country, RegionalHMap));
	}

	AddNonWorkDay(Date, _intRegionCode, RegionalHMap);
}



// private function
void wuCalendar::AddNonWorkDay(const wuSYSTEMTIME& TimeDate, int _intRegionCode, RegionHMapType* RegionalHMap)
{
	RegionHMapType::iterator RegionIter;
	HolidayMapType* HolidayMap;
	wuSYSTEMTIME Date = ConformDate(TimeDate); // remove the time factor

	RegionIter = RegionalHMap->find(_intRegionCode);

	if (RegionIter != RegionalHMap->end())
	{
		HolidayMap = RegionIter->second;
	}
	else
	{
		HolidayMap = new HolidayMapType();
		RegionalHMap->insert(RegionHMapType::value_type(_intRegionCode, HolidayMap));
	}

	HolidayMap->insert(HolidayMapType::value_type(Date, 0));
}



void wuCalendar::AddNonWorkDay(const wuSYSTEMTIME& Date, a2CountryCode Country, int _intRegionCode)
{
	AddNonWorkDay(Date, (a3CountryCode)Country, _intRegionCode);
}



void wuCalendar::AddNonWorkDay(const wuSYSTEMTIME& Date, ccyCurrencyCode Country, int _intRegionCode)
{
	AddNonWorkDay(Date, (a3CountryCode)Country, _intRegionCode);
}




void wuCalendar::AddRegion(a3CountryCode Country, int _intRegionCode)
{
	RegionHMapType* RegionalHMap = 0;
	CountryHMapType::iterator CountryFindIter;
	HolidayMapType* HolidayMap;

	CountryFindIter = mIntHolidaysMap.find(Country);

	if (CountryFindIter!=mIntHolidaysMap.end() && CountryFindIter->second!=0)
	{
		RegionalHMap = (RegionHMapType*)(CountryFindIter->second);
	}
	else  // we must here then add a National Holidays map for that CountryCode
	{
		if (CountryFindIter!=mIntHolidaysMap.end() && CountryFindIter->second==0)
		{	  // we found the country, but no valid holidays map. i.e. country had no holidays
			  // first we delete the existing entry, then add a new one.
			mIntHolidaysMap.erase(CountryFindIter);
		}

		RegionalHMap = new RegionHMapType();
		mIntHolidaysMap.insert(CountryHMapType::value_type((int)Country, RegionalHMap));
	}

	HolidayMap = new HolidayMapType();
	RegionalHMap->insert(RegionHMapType::value_type(_intRegionCode, HolidayMap));
}



void wuCalendar::AddRegion(a2CountryCode Country, int _intRegionCode)
{
	AddRegion((a3CountryCode)Country, _intRegionCode);
}


void wuCalendar::AddRegion(ccyCurrencyCode Country, int _intRegionCode)
{
	AddRegion((a3CountryCode)Country, _intRegionCode);
}




bool wuCalendar::IsWorkingDay(const wuSYSTEMTIME& wDate, a2CountryCode Country, int _intRegionCode) throw (exception)
{
	return IsWorkingDay(wDate, (a3CountryCode)Country, _intRegionCode);
}



bool wuCalendar::IsWorkingDay(const wuSYSTEMTIME& wDate, ccyCurrencyCode Country, int _intRegionCode) throw (exception)
{
	return IsWorkingDay(wDate, (a3CountryCode)Country, _intRegionCode);
}



HolidayMapType* wuCalendar::GetHolidayMap(a3CountryCode Country, int _intRegionCode)
{
	// country map
	CountryHMapType::const_iterator CountryIter;

	// region map
	RegionHMapType* RegionHMap = 0;
	RegionHMapType::const_iterator RegionIter;

	// date map
	HolidayMapType* HolidayMap = 0;
	CountryIter = mIntHolidaysMap.find((int)Country);

	if (CountryIter!=mIntHolidaysMap.end())
	{
		RegionHMap = (RegionHMapType*)(CountryIter->second);
		
		if (RegionHMap)
		{
			RegionIter = RegionHMap->find(_intRegionCode);

			if (RegionIter!=RegionHMap->end())
			{
				HolidayMap = RegionIter->second;
			}
		}
	}

	return HolidayMap;
}




bool wuCalendar::IsWorkingDay(const wuSYSTEMTIME& TimeDate, a3CountryCode Country, int _intRegionCode) throw (exception)
{
	bool blnRes = false;
	wuSYSTEMTIME wDate = ConformDate(TimeDate); // remove the time factor

	// date map
	HolidayMapType* HolidayMap;
	HolidayMapType::const_iterator DateIter;

	if (!IsWeekend(	wDate.wYear, 
					wDate.wMonth, 
					wDate.wDay))
	{
		HolidayMap = GetHolidayMap(Country, _intRegionCode);
		
		if (HolidayMap)
		{
			DateIter = HolidayMap->find(wDate);

			if (DateIter == HolidayMap->end())
			{
				  // didn't find that date. Therefore, this is a work date!!
				blnRes = true;
			}
		}
		else
		{
			   // if don't find the country and/or region, we are doing a search with wrong arguments.
			throw exception("Unknown Country Exception");
		}

	} // if is a weekend, obviously not a work day

	return blnRes;
}



  // sourced from http://www.tondering.dk/claus/cal/node3.html#SECTION00360000000000000000
  // The Calendar FAQ
unsigned char wuCalendar::GetDayOfWeek(unsigned int uintYear, unsigned int uintMonth, unsigned int uintDayInMonth)
{
	unsigned int a = (14-uintMonth)/12;
	unsigned int y = uintYear - a;
	unsigned int m = uintMonth + (12*a) - 2;
	// for gregorian calendar
	unsigned int d = (uintDayInMonth + y + (y/4) - (y/100) + (y/400) + ((31*m)/12));
	d = d % 7;

	return (unsigned char)d;
}





unsigned int wuCalendar::DaysInMonth(wuSYSTEMTIME objDate)
{
	unsigned int uintDays = 0;

	if (objDate.wMonth<13 && objDate.wMonth>0)
	{
		if (objDate.wMonth==2 && IsLeapYear(objDate.wYear))
		{
			uintDays = 29;
		}
		else
		{
			uintDays = uintDaysInMonthReg[objDate.wMonth];
		}
	}

	return uintDays;
}




bool wuCalendar::IsLeapYear(unsigned int uintYear)
{
	bool blnLeapYear = false;

	if (uintYear%4==0)
	{
		if (uintYear%100==0)
		{
			if (uintYear%400==0)
			{
				blnLeapYear = true;
			}
		}
		else
		{
			blnLeapYear = true;
		}
	}

	return blnLeapYear;
}



bool wuCalendar::IsWeekend(unsigned int uintYear, unsigned int uintMonth, unsigned int uintDayInMonth)
{
	unsigned char ucharDay = GetDayOfWeek(uintYear, uintMonth, uintDayInMonth);

	if (ucharDay==0 ||
		ucharDay==6)
	{
		return true;
	}
	else
	{
		return false;
	}
}





const ULARGE_INTEGER wuCalendar::GetTimeSpanIn100Nano(wuTimeSpan SpanConstant)
{
	ULARGE_INTEGER ularReturn;

	if (SpanConstant==tsSec)
	{
		ularReturn.QuadPart = (__int64)10000000;
	}
	else if (SpanConstant==tsMin)
	{
		ularReturn.QuadPart = (__int64)10000000*60;
	}
	else if (SpanConstant==tsHour)
	{
		ularReturn.QuadPart = (__int64)10000000*60*60;
	}
	else if (SpanConstant==tsDay)
	{
		ularReturn.QuadPart = (__int64)10000000*60*60*24;
	}	
	else if (SpanConstant==tsWeek)
	{
		ularReturn.QuadPart = (__int64)10000000*60*60*24*7;
	}

	return ularReturn;
}





wuSYSTEMTIME wuCalendar::SYSAddTimeDiff(	const wuSYSTEMTIME& pst,		///< valid datetime which we want change
										wuTimeSpan SpanConstant,	///< TimeSpan constant enum
										const __int64 num			///< Multiplier for the SpanConstant.
							)
{
	FILETIME ft;
	wuSYSTEMTIME st;
	ULARGE_INTEGER temp;
	
	SystemTimeToFileTime (&pst,&ft); 

	temp.HighPart = ft.dwHighDateTime;
	temp.LowPart = ft.dwLowDateTime;

	temp.QuadPart = temp.QuadPart + num*GetTimeSpanIn100Nano(SpanConstant).QuadPart; 

	ft.dwHighDateTime = temp.HighPart;
	ft.dwLowDateTime = temp.LowPart;

	FileTimeToSystemTime (&ft,&st); 

	return st;
} 






ULARGE_INTEGER wuCalendar::GetAbsDiff(const wuSYSTEMTIME& Left, const wuSYSTEMTIME& Right, wuTimeSpan SpanConstant)
{
	FILETIME ftLarge;
	FILETIME ftSmall;
	ULARGE_INTEGER ularLarge;
	ULARGE_INTEGER ularSmall;
	ULARGE_INTEGER ularRes;
	wuSYSTEMTIME Small;
	wuSYSTEMTIME Large;

	// first make sure we know the least time using our working comparison functions
	// we say Left is largest
	if (Left<Right)
	{
		Small = Left;
		Large = Right;
	}
	else
	{
		Small = Right;
		Large = Left;
	}

	// now we do the subtraction of lowest from highest

	SystemTimeToFileTime(&Large, &ftLarge);
	SystemTimeToFileTime(&Small, &ftSmall);	

	ularLarge.LowPart        =   ftLarge.dwLowDateTime; 
	ularLarge.HighPart       =   ftLarge.dwHighDateTime; 
	ularSmall.LowPart       =   ftSmall.dwLowDateTime; 
	ularSmall.HighPart      =   ftSmall.dwHighDateTime; 

	// Subtracts the lower and higher parts of the ULARGE_INTEGER structure 
	ularRes.HighPart  =   ularLarge.HighPart - ularSmall.HighPart; 
	ularRes.LowPart   =   ularLarge.LowPart  - ularSmall.LowPart; 

	// now we work out how many of type TimeSpan we have by dividing the answer into that
	// we use GetTimeSpanIn100Nano(wuTimeSpan SpanConstant)
	ularRes.QuadPart = ularRes.QuadPart/GetTimeSpanIn100Nano(SpanConstant).QuadPart;

	return ularRes;
}







wuSYSTEMTIME wuCalendar::SYSAddMonths(	const wuSYSTEMTIME& pst,		///< valid datetime which we want change
										int intAddMonths)
{
	wuSYSTEMTIME NewDate = pst;
	int intYearOffset = 0; 
	int intMonthOffset = 0;

	// first we work out the year offset
	int intDivTemp = (pst.wMonth + intAddMonths -1) /12;
	int intModTemp = ((pst.wMonth + intAddMonths -1) %12) +1 ;

	  // set the new month
	  // if we are going back a year
	if ((intDivTemp<=0 && intModTemp ==0) || intModTemp<0)
	{
		intYearOffset = intDivTemp - 1;
		NewDate.wMonth = 12 +intModTemp; 
	} 
	else  
	{ // else we are going forward a year or within the same year
		intYearOffset = intDivTemp;
		NewDate.wMonth = intModTemp;
	}

	// set the new year
	NewDate.wYear += intYearOffset;

	// check day in month is valid
	int intDaysInNewMonth = uintDaysInMonthReg[NewDate.wMonth-1];

	  // if its a leap year, and its feburuary
	if (NewDate.wMonth==2 && IsLeapYear(NewDate.wYear))
	{
		intDaysInNewMonth +=1;
	}

	intMonthOffset = ((pst.wDay / intDaysInNewMonth)>0 && (pst.wDay % intDaysInNewMonth)!=0) ;

	  // if it is now not a valid day
	if (intMonthOffset && intAddMonths>=0)
	{	  // if went forward in time and day is not valid, 
		  // make sure we offset by number of days missing from new month. 
		NewDate.wDay = pst.wDay % intDaysInNewMonth;
		NewDate.wMonth++; // we must then add a month
	}
	else if (intMonthOffset && intAddMonths<0)
	{	  // if we went backwards in time, and day is not valid, 
		  // make it the maximum day available
		  // we don't add a month, as we are going backwards.
		NewDate.wDay = intDaysInNewMonth;
	}

	  // lastly we validate the week day
	NewDate.wDayOfWeek = GetDayOfWeek(NewDate.wYear, NewDate.wMonth, NewDate.wDay);

	return NewDate;
}



wuSYSTEMTIME wuCalendar::SYSAddYears(const wuSYSTEMTIME& pst, int intAddYears)
{
	wuSYSTEMTIME NewTime = pst;

	NewTime.wYear = NewTime.wYear + intAddYears;

	return NewTime;
}








