//LOCAL INCLUDES
//
#include "winutils/wuFile.h"
#include <stdio.h>

using namespace std;



const char* cpcharEOL = "\n\0";


//--------------------------------------------------------------

// This function wraps GetModuleFileName and a tokenizer to get only the path NOT the filename
DWORD GetHandleFilePath(HMODULE hModule, LPTSTR lpFilename,  DWORD nSize)
{
	DWORD intNumGotChars = GetModuleFileName(hModule, lpFilename, nSize);
	char* Pos = strrchr(lpFilename, '\\');
	(char)(*Pos) = 0; // just put a null at the end of the path
	return (int)(Pos-lpFilename);
}






wuFile::wuFile(void)
{
	Initialise("", "", 0, false, false);
}


//--------------------------------------------------------------


wuFile::wuFile(	string			strFilePath,
					string			strFileName,
					unsigned int	intTimeout, 
					bool			blnFileExist,
					bool			blnReadOnly)
{
	Initialise(strFilePath, strFileName, intTimeout, blnFileExist, blnReadOnly);
}

				
//--------------------------------------------------------------


wuFile::~wuFile(void)
{
	Close();
	CloseHandle(mhdlMutex);
}


//--------------------------------------------------------------


bool wuFile::Initialise(	string			strFilePath,
							string			strFileName,
							unsigned int	intTimeout,
							bool			blnFileExist,
							bool			blnReadOnly)
{
	bool	blnReturn = false;
	char	achrMutexName[256] = {0};

		// create mutex with unique string
	sprintf(achrMutexName, "File%u", (unsigned int)this);
	mhdlMutex = CreateMutex(NULL, FALSE, achrMutexName);
	if (mhdlMutex != NULL)
	{
		mobjFile = NULL;
		SetSettings(strFilePath, strFileName, intTimeout, blnFileExist, blnReadOnly);
	}

	return (blnReturn);
}


//--------------------------------------------------------------


void wuFile::SetSettings(	string			strFilePath,
							string			strFileName,
							unsigned int	intTimeout,
							bool			blnFileExist,
							bool			blnReadOnly)
{
	mstrFilePath = strFilePath;
	mstrFileName = strFileName;
	mintTimeout = intTimeout;

	mblnFileExist = blnFileExist;
	mblnReadOnly = blnReadOnly;
}


//--------------------------------------------------------------


bool wuFile::ValidSettings(void)
{
	bool	blnReturn = false;


	if (mstrFilePath != "" && mstrFileName != "" && mintTimeout < 10000)
	{
		blnReturn = true;
	}

	return (blnReturn);
}


//--------------------------------------------------------------


DWORD wuFile::WaitForFileMutex(void)
{
	return WaitForSingleObject(mhdlMutex, mintTimeout);
}


//--------------------------------------------------------------


bool wuFile::ReleaseFileMutex(void)
{
	bool	blnReturn = false;

	
	if (ReleaseMutex(mhdlMutex) > 0)
	{
		blnReturn = true;
	}

	return (blnReturn);
}



std::string wuFile::GetFilePath(void) 
{
	char pcharBuffer[124]={0};

	if (mstrFilePath=="")
	{
		GetHandleFilePath(NULL, pcharBuffer, 124);
		mstrFilePath = pcharBuffer;
	}

	return(mstrFilePath);
}




//--------------------------------------------------------------

		
bool wuFile::Open(void)
{
	bool	blnReturn = true;
	string	strFullFilePath = "";	// full file path, include path and file name
	DWORD	dwAccess = 0;
	DWORD	dwCreateDisposition = 0;

	DWORD MutexRes = WaitForFileMutex();

	if (MutexRes == WAIT_ABANDONED || MutexRes == WAIT_OBJECT_0)
	{
			// set the open file option
		if (mblnFileExist)
		{
			if (mblnReadOnly)
			{
				dwAccess = GENERIC_READ;
				dwCreateDisposition = OPEN_EXISTING;
			}
			else
			{
				dwCreateDisposition = OPEN_EXISTING;
				dwAccess = GENERIC_WRITE;
			}
		}
		else
		{
			dwCreateDisposition = CREATE_ALWAYS;
			dwAccess = GENERIC_WRITE;
		}

		strFullFilePath = mstrFilePath + "\\" + mstrFileName;
		mobjFile = CreateFile(strFullFilePath.c_str(), dwAccess, FILE_SHARE_WRITE|FILE_SHARE_READ,
								NULL, dwCreateDisposition, FILE_ATTRIBUTE_NORMAL, NULL);

		if (mobjFile==INVALID_HANDLE_VALUE && mblnFileExist)
		{
			mobjFile = CreateFile(strFullFilePath.c_str(), dwAccess, FILE_SHARE_WRITE|FILE_SHARE_READ,
									NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
		}

		if (mobjFile==INVALID_HANDLE_VALUE)
		{
			mobjFile = NULL;
			blnReturn = false;
		}

		SetFilePointer(mobjFile, 0, 0, FILE_END);

		ReleaseFileMutex();
	}

	return (blnReturn);
}


//--------------------------------------------------------------


void wuFile::Close(void)
{
	DWORD MutexRes = WaitForFileMutex();

	if (MutexRes == WAIT_ABANDONED || MutexRes == WAIT_OBJECT_0)
	{
		if (mobjFile!= NULL)
		{
			FlushFileBuffers(mobjFile);
			CloseHandle(mobjFile);
			mobjFile = NULL;
		}

		ReleaseFileMutex();
	}
}


//--------------------------------------------------------------





bool wuFile::Write(const char* cpcharMsg)
{
	BOOL	blnReturn = false;
	DWORD	dwBytesWritten = 0;

	DWORD MutexRes = WaitForFileMutex();

	if (MutexRes == WAIT_ABANDONED || MutexRes == WAIT_OBJECT_0)
	{
		if (mobjFile != NULL)
		{
			blnReturn = WriteFile(mobjFile, cpcharMsg, strlen(cpcharMsg), &dwBytesWritten, NULL);

			if (blnReturn!=NULL)
			{
				Flush();
				blnReturn = true;
			}
		}

		ReleaseFileMutex();
	}

	return (blnReturn==TRUE);
}


//--------------------------------------------------------------


bool wuFile::WriteLine(const char* cpcharMsg)
{
	DWORD	dwBytesWritten = 0;
	DWORD	dwBytesWritten2 = 0;
	BOOL	blnReturn = FALSE;

	DWORD MutexRes = WaitForFileMutex();

	if (MutexRes == WAIT_ABANDONED || MutexRes == WAIT_OBJECT_0)
	{
		if (mobjFile != NULL)
		{
			blnReturn = WriteFile(mobjFile, cpcharMsg, strlen(cpcharMsg), &dwBytesWritten, NULL);
			blnReturn = WriteFile(mobjFile, "\n", 1, &dwBytesWritten2, NULL);
			Flush();
			blnReturn = true;
		}

		ReleaseFileMutex();
	}

	return (blnReturn==TRUE);
}


//--------------------------------------------------------------


void wuFile::Flush(void)
  // flush the log file
{
	DWORD MutexRes = WaitForFileMutex();

	if (MutexRes == WAIT_ABANDONED || MutexRes == WAIT_OBJECT_0)
	{
		if (mobjFile != NULL)
		{
			FlushFileBuffers(mobjFile);
		}

		ReleaseFileMutex();
	}
}
 

//--------------------------------------------------------------


unsigned long wuFile::GetFileSize(void)
{
	unsigned long	lngEnd = 0;

	DWORD MutexRes = WaitForFileMutex();

	if (MutexRes == WAIT_ABANDONED || MutexRes == WAIT_OBJECT_0)
	{
		if (mobjFile != NULL)
		{
			// store current position
			LARGE_INTEGER ularFileLen;
			ularFileLen.LowPart = ::GetFileSize(mobjFile, (unsigned long *)&(ularFileLen.HighPart));
			lngEnd = (unsigned int) ularFileLen.QuadPart;
		}

		ReleaseFileMutex();
	}

	return (lngEnd);
}


//--------------------------------------------------------------
