#include "winutils/wuFtpSocket.h"


using namespace std;


const std::string wuFtpSocket::TELNET_EOF="\r\n";


wuFtpSocket::wuFtpSocket()
{

	cstrHost=0;
	uintHostPort=0;
}



wuFtpSocket::wuFtpSocket(const char* _cstrHost, unsigned int _uintHostPort) throw(wuSocketNoHost, wuSocketBadRead, wuSocketException, exception)
{
	try
	{
		cstrHost=0;
		SetHost(_cstrHost, _uintHostPort);
		Connect();
	}
	catch(...)
	{
		throw;
	}

}



wuFtpSocket::~wuFtpSocket()
{

	if (fSocket.IsConnected())
		fSocket.Close(); // ensure socket is closed
	
	if (cstrHost) delete[] cstrHost;
}






BOOL wuFtpSocket::IsConnected()
{
	int ReturnVal = fSocket.IsConnected();

	return ReturnVal;	// else there was a basic socket error
}


BOOL wuFtpSocket::IsLoggedOn()
{
	int ReturnVal = fSocket.IsConnected();

	if (ReturnVal)
	{
		try
		{
			ReturnVal = SendDummyRequest();
		}
		catch(...)
		{
			return FALSE;
		}
	}

	return ReturnVal;	// else there was a basic socket error
}



int wuFtpSocket::Close()
{
	return fSocket.Close();
}





void wuFtpSocket::SetTimeout(unsigned int _uintTimeout) throw()
{
	try
	{	// only do this if the socket has already been connected (if this is allowed)
		uintTimeout=_uintTimeout;
		fSocket.SetReceiveTimeout(uintTimeout);
		fSocket.SetSendTimeout(uintTimeout);
	}
	catch(...)
	{
		throw;
	}

}




void wuFtpSocket::SetReconnectionRetries(unsigned int _uintRetries){

	uintRetries=_uintRetries;// do work to set value, plus 
	// call socket calls to change actual 
	// but only do this if the socket has already been connected (if this is allowed)
}




void wuFtpSocket::SetHost(const char* _cstrHost, unsigned int _uintHostPort) throw(wuSocketException){

	int NameLen = strlen(_cstrHost);
			
	if (NameLen>0){
	
		if (cstrHost!=0) delete[] cstrHost;
		cstrHost=new char[NameLen+1];
		strcpy(cstrHost, _cstrHost);
		uintHostPort=_uintHostPort;
		
	}
	else{
		cstrHost=0;
		throw wuSocketException(wuSocketException::NOHOST);
	}
}



void wuFtpSocket::Connect(const char* _cstrHost, unsigned int _uintHostPort)  throw(wuSocketNoHost, wuSocketBadRead, wuSocketException, exception){

	try{
		SetHost(_cstrHost, _uintHostPort);
		Connect();
		SetTimeout(_FTP_SOCKET_TIMEOUT);
		SetReconnectionRetries(_FTP_SOCKET_RETRIES);
	}
	catch(...)
	{
		throw;
	}
}


// this command opens the connection,and performs any ftp handshaking required by a client
void wuFtpSocket::Connect() throw(wuSocketNoHost, wuSocketBadRead, wuSocketException, exception){

	try
	{
		fSocket.Connect(cstrHost, uintHostPort);
		GetReply();
	}
	catch(...)
	{
		throw;
	}
}
//TranslateWinsockStatus(int intCode)



void wuFtpSocket::SendRequest(const char* pcharRequestString, unsigned int uintRequestStringLen) 
											throw(wuSocketException, wuSocketInterrupt, exception)
{
	if ((fSocket.Send(pcharRequestString, uintRequestStringLen))==-1)
	{
		throw wuSocketException(fSocket.TranslateWinsockStatus(WSAGetLastError()));
	}
	
	try
	{
		GetReply();
	}
	catch(...)
	{
		throw;
	}
}






BOOL wuFtpSocket::SendUserRequest(const char* _cstrUserName) throw(wuSocketException, wuSocketInterrupt, exception)
{
	char line[_MAX_LINE]={0};
	int intLineLength = 0;
	BOOL bResult = FALSE;

	sprintf(line, "user %s\r\n",_cstrUserName);
	intLineLength = strlen(line);

	try
	{
		SendRequest(line, intLineLength);
	}
	catch(...)
	{
		throw;
	}

	return ((BOOL)(LastReply.GetStateCode().intStateCode1==3));
}






BOOL wuFtpSocket::SendPasswordRequest(const char* _cstrPassword) throw(wuSocketException, exception){

	char line[_MAX_LINE]={0};
	int intLineLength = 0;
	BOOL bResult = FALSE;

	sprintf(line, "pass %s\r\n",_cstrPassword);
	intLineLength = strlen(line);

	try
	{
		SendRequest(line, intLineLength);
	}
	catch(...)
	{
		throw;
	}

	return ((BOOL)(LastReply.GetStateCode().intStateCode1==2));
}


BOOL wuFtpSocket::SendNoOpRequest() throw (wuSocketException, std::exception){

	char line[]="NOOP\r\n";
	int intLineLength =strlen(line);
	BOOL bResult = FALSE;

	try
	{
		SendRequest(line, intLineLength);
	}
	catch(...)
	{
		throw;
	}

	return ((BOOL)(LastReply.GetStateCode().intStateCode1==2));
}




BOOL wuFtpSocket::SendDummyRequest() throw (wuSocketException, std::exception){

	char line[]="CWD .\r\n";
	int intLineLength =strlen(line);
	BOOL bResult = FALSE;

	try
	{
		SendRequest(line, intLineLength);
	}
	catch(...)
	{
		throw;
	}

	return ((BOOL)(LastReply.GetStateCode().intStateCode1==2));
}



BOOL wuFtpSocket::SendPWDRequest() throw (wuSocketException, std::exception){

	char line[]="PWD\r\n";
	int intLineLength =strlen(line);
	BOOL bResult = FALSE;

	try
	{
		SendRequest(line, intLineLength);
	}
	catch(...)
	{
		throw;
	}

	return ((BOOL)(LastReply.GetStateCode().intStateCode1==2));
}




// legal types include 'A' , 'E' ,'I' 
BOOL wuFtpSocket::SendTypeRequest(const char type) throw (wuSocketException, std::exception)
{
	BOOL bResult = FALSE;
	char line[]="TYPE  \r\n";
	line[5]=type;
	int intLineLength =strlen(line);
	
	try
	{
		SendRequest(line, intLineLength);
	}
	catch(...)
	{
		throw;
	}

	return ((BOOL)(LastReply.GetStateCode().intStateCode1==2));
}


// change remote current working directory
BOOL wuFtpSocket::SendCWDRequest(const char* _Path) throw (wuSocketException, std::exception)
{
	BOOL bResult = FALSE;
	char line[_MAX_LINE]={0};
	sprintf(line, "CWD %s\r\n", _Path);
	int intLineLength =strlen(line);
	
	try
	{
		SendRequest(line, intLineLength);
	}
	catch(...)
	{
		throw;
	}

	return ((BOOL)(LastReply.GetStateCode().intStateCode1==2));
}




// change UP remote current working directory
BOOL wuFtpSocket::SendCDUPRequest() throw (wuSocketException, std::exception)
{
	BOOL bResult = FALSE;
	char line[]="CDUP\r\n";
	int intLineLength =strlen(line);
	
	try
	{
		SendRequest(line, intLineLength);
	}
	catch(...)
	{
		throw;
	}

	return ((BOOL)(LastReply.GetStateCode().intStateCode1==2));
}



// REIN command. A USER command may be expected to follow. 
BOOL wuFtpSocket::SendReInitializeRequest() throw (wuSocketException, std::exception)
{
	BOOL bResult = FALSE;
	char line[]="REIN\r\n";
	int intLineLength =strlen(line);
	
	try
	{
		SendRequest(line, intLineLength);
	}
	catch(...)
	{
		throw;
	}

	return ((BOOL)(LastReply.GetStateCode().intStateCode1==2));
}



// RENAME FROM command. A RENAME TO command should be expected to follow. 
BOOL wuFtpSocket::SendRNFRRequest(const char* _Path) throw (wuSocketException, std::exception)
{
	BOOL bResult = FALSE;
	char line[_MAX_LINE]={0};
	sprintf(line, "RNFR %s\r\n", _Path);
	int intLineLength =strlen(line);
	
	try
	{
		SendRequest(line, intLineLength);
	}
	catch(...)
	{
		throw;
	}

	return ((BOOL)(LastReply.GetStateCode().intStateCode1==3));
}



// RENAME FROM command. A RENAME TO command should be expected to follow. 
BOOL wuFtpSocket::SendRNTORequest(const char* _Path) throw (wuSocketException, std::exception)
{
	BOOL bResult = FALSE;
	char line[_MAX_LINE]={0};
	sprintf(line, "RNTO %s\r\n", _Path);
	int intLineLength =strlen(line);
	
	try
	{
		SendRequest(line, intLineLength);
	}
	catch(...)
	{
		throw;
	}

	return ((BOOL)(LastReply.GetStateCode().intStateCode1==2));
}




BOOL wuFtpSocket::Rename(const char* SourceName, const char* DestinationName) throw (wuSocketException, std::exception)
{
	try
	{

		BOOL More = SendRNFRRequest(SourceName);

		if (More) 
		{
			Sleep(100);
			SendRNTORequest(DestinationName);
		}
	}
	catch(...)
	{
		throw;
	}

	if (LastReply.GetStateCode().intStateCode1==2) return TRUE;
	else return FALSE;
}






BOOL wuFtpSocket::FtpLogin(const char* _LoginName, const char* _Password) throw (wuSocketException, std::exception){

	try
	{
		BOOL More = SendUserRequest(_LoginName);
		if (More) SendPasswordRequest(_Password);
	}
	catch(...)
	{
		throw;
	}

	if (LastReply.GetStateCode().intStateCode1==2) return TRUE;
	else return FALSE;
}





BOOL wuFtpSocket::Move(const char* SourceName, const char* DestinationName) throw (wuSocketException, std::exception)
{
	return Rename(SourceName, DestinationName);
}



// change remote current working directory
BOOL wuFtpSocket::SendSTORRequest(const char* _Path) throw (wuSocketException, std::exception)
{
	BOOL bResult = FALSE;
	char line[_MAX_LINE]={0};
	sprintf(line, "STOR %s\r\n", _Path);
	int intLineLength = strlen(line);
	
	try
	{
		SendRequest(line, intLineLength);
	}
	catch(...)
	{
		throw;
	}

	return ((BOOL)(LastReply.GetStateCode().intStateCode1==1));
}





// change remote current working directory
BOOL wuFtpSocket::SendRETRRequest(const char* _Path) throw (wuSocketException, std::exception)
{
	BOOL bResult = FALSE;
	char line[_MAX_LINE]={0};
	sprintf(line, "RETR %s\r\n", _Path);
	int intLineLength = strlen(line);
	
	try
	{
		SendRequest(line, intLineLength);
	}
	catch(...)
	{
		throw;
	}

	return ((BOOL)(LastReply.GetStateCode().intStateCode1==1));
}





BOOL wuFtpSocket::SendPASVRequest(string& _strHostIP, unsigned int& _uintPort ) throw (wuSocketException, std::exception)
{
	BOOL bResult=FALSE;
	char line[]="PASV\r\n";
	int intLineLength =strlen(line);
	BOOL ExtractionGood = FALSE;

	try
	{
		SendRequest(line, intLineLength);
	}
	catch(...)
	{
		throw;
	}

	// now need to extract important information from pasv reply
	if (LastReply.GetStateCode().intStateCode1==2)
	{// do the extraction if it was successful
		ExtractionGood = ExtractFtpHostNPort(LastReply.GetReplyMessage(),_strHostIP, _uintPort);
	} 

	return ExtractionGood;
}





BOOL wuFtpSocket::SendDataPasv(FILE* _SourceStream, const char* _UploadFileName, const char _Type) throw (wuSocketException, std::exception){

	string strHostIP="";
	unsigned int uintPort=0;
	wuTCPSocket* SendSocket=0;
	BOOL Success = FALSE;

	try
	{
		// first set up the send mode (binary, or ascii most likely)
		this->SendTypeRequest(_Type);
		if (LastReply.GetStateCode().intStateCode1!=2) return FALSE; // no point in going further if failed at this point

		// now set up Pasv Connection
		this->SendPASVRequest(strHostIP, uintPort );
		if (LastReply.GetStateCode().intStateCode1!=2) return FALSE; // no point in going further if failed at this point

		SendSocket = new wuTCPSocket();

		try
		{
			SendSocket->Connect(strHostIP.c_str(), uintPort);
		}
		catch(...)
		{
			delete SendSocket;
			return FALSE;	// todo - future - implement a SetStateCode, so we can say at this point - send socket failure for whatever reason if it does so
		}

		this->SendSTORRequest(_UploadFileName);
		if (LastReply.GetStateCode().intStateCode1!=1) return FALSE; // no point in going further if failed at this point

		Success = this->SendFromStream(SendSocket, _SourceStream);

		SendSocket->Close();
		delete SendSocket;

		GetReply();	// get servers opinion if transfer worked.
		// GetReply will set LastReply. Should overwrite this if Success is false, 
		// because file stream failed before the actual transfer.. and will give more information

		if (!Success){	
			LastReply.SetState("599 Application Error: SendFromStream Unsuccessfull");
			return FALSE;
		}
	}
	catch(...)
	{
		throw;
	}

	return TRUE;
}





BOOL wuFtpSocket::RecieveDataPasv(FILE* _DestinationStream, const char* _DownloadFileName, const char _Type) throw (wuSocketException, std::exception){

	string strHostIP="";
	unsigned int uintPort=0;
	wuSocket* RecieveSocket=0;
	BOOL Success = FALSE;

	try{
		// first set up the send mode (binary, or ascii most likely)
		this->SendTypeRequest(_Type);
		if (LastReply.GetStateCode().intStateCode1!=2) return FALSE; // no point in going further if failed at this point

		// now set up Pasv Connection
		this->SendPASVRequest(strHostIP, uintPort );
		if (LastReply.GetStateCode().intStateCode1!=2) return FALSE; // no point in going further if failed at this point

		RecieveSocket = new wuTCPSocket();

		try
		{
			RecieveSocket->Connect(strHostIP.c_str(), uintPort);
		}
		catch(...)
		{
			delete RecieveSocket;
			return FALSE;	// todo - future - implement a SetStateCode, so we can say at this point - send socket failure for whatever reason if it does so
		}

		this->SendRETRRequest(_DownloadFileName);
		if (LastReply.GetStateCode().intStateCode1!=1) return FALSE; // no point in going further if failed at this point

		Success = this->RecieveToStream(_DestinationStream, RecieveSocket);
	
		RecieveSocket->Close();
		delete RecieveSocket;

		GetReply();	// get servers opinion if transfer worked.
		// GetReply will set LastReply. Should overwrite this if Success is false, 
		// because file stream failed before the actual transfer.. and will give more information

		if (!Success){	

			LastReply.SetState("599 Application Error: Receive to stream Unsuccessfull");
			return FALSE;
		}
	}
	catch(...)
	{
		throw;
	}

	return TRUE;
}





// in future, could add some sort of 'Write to infoupdate' which gets called regularly through
// transfer. This way, another process would be able to read this data, and publish it somehow
// however, some synchronization mechanism might be required, though, only this would be writing to it
// returns number of bytes read and sent - future, add same set error code thing if file stream transfer fail
int wuFtpSocket::SendFromStream(wuSocket* Destination, FILE* _SourceStream) throw(wuSocketException){

	char line[_MAX_LINE]={0};
	int count = 0;
	int TotalCount = 0;
	int error_check = 0;

	count = fread((void*)line, 1, _MAX_LINE-1, _SourceStream);

	while(count)
	{
		TotalCount+=count;
		error_check = Destination->Send(line, count);

		if (error_check==SOCKET_ERROR) throw wuSocketException(wuSocketException::BADWRITE);
	
		memset((void*)line, 0, _MAX_LINE);
		count = fread((void*)line, 1, _MAX_LINE-1, _SourceStream);
	}

	return TotalCount;
}

// 27-06-03 SV





int wuFtpSocket::RecieveToStream(FILE* _DestinationStream, wuSocket* _Source) throw(wuSocketException){

	char line[_MAX_LINE]={0};
	int count = 0;
	int TotalCount = 0;

	do{
		count = _Source->Receive(line, _MAX_LINE-1); 

		if (count<0) throw wuSocketException(wuSocketException::BADREAD);

		TotalCount+=count;
		fwrite((void*)line, 1, count, _DestinationStream);
		memset((void*)line, 0, _MAX_LINE);

	}while(count>0);

	return TotalCount;
}








// CAUTION!! QUIT COMMAND AUTOMATICALLY ENSURES CLOSE OF SOCKET
BOOL wuFtpSocket::SendQuitRequest() throw(wuSocketException, exception)
{
	BOOL bResult = FALSE;
	char line[]="quit\r\n";
	int intLineLength =strlen(line);

	try
	{
		SendRequest(line, intLineLength);
	}
	catch(...)
	{
		throw;
	}

	fSocket.Close(); // ensure socket is closed

	return ((BOOL)(LastReply.GetStateCode().intStateCode1==2));
}




// compatible with D. Bernstein's idea for ftp and IPv6
BOOL wuFtpSocket::ExtractFtpHostNPort(const string& _strServerReplyMessage, string& _strHostIP, unsigned int& _uintPort){

	wuStokenizer* tok = new wuStokenizer(_strServerReplyMessage, " ");
	wuStokenizer* comtok = 0;
	char line[_MAX_LINE]={0};
	BOOL FoundHostInfo = FALSE;
	string CurrentToken = "";
	int index=0;
	unsigned int RetAry[6]={0};

	_strHostIP="";
	_uintPort=0;

	while(tok->more() && index<6){

		index = 0;
		CurrentToken = StripEndsOfNonNumerals(tok->next());
		comtok= new wuStokenizer(CurrentToken, ",");

		try{

			while(comtok->more() && index<6){

				RetAry[index] = wu::Ascii2Integer(comtok->next().c_str());
				index++;
			}
		}
		catch(exception& e)
		{	// nothing do do except catch it from Ascii2Integer 
		}
	
		delete comtok;
	}

	delete tok;


	// if index=6, we have an ip, and port, of sorts, is it good now ? test this, and get port num
	if (index==6){
	
		int count=0;

		while(count<4){
			if (RetAry[count]>255) break; // we have an invalid hostaddress - probably an IPv6 ... should use pre decided IP
			count++;
		}
		
		if (count==4){
			sprintf(line, "%d.%d.%d.%d",RetAry[0], RetAry[1], RetAry[2], RetAry[3]); 
			_strHostIP=line;
		}
		else _strHostIP=cstrHost; // original host address if is ipv6
		
		_uintPort=RetAry[4]*256+RetAry[5];//p1*256+p2 (GET PORT NUMBER)
		FoundHostInfo = TRUE;
	}
	else FoundHostInfo = FALSE;

	return FoundHostInfo;
}





string wuFtpSocket::StripEndsOfNonNumerals(const string _strLine){

	string numerals = "0123456789";

	int start = _strLine.find_first_of(numerals);
	int end   = _strLine.find_last_of(numerals);
	int len =0;
	if (start>=0) len = end-start+1;
	else len=-1;

	if (len>0) return _strLine.substr(start, len);
	else return "";
}




// this function recieves from the socket, the ftp server's reply, and stores it in a known 
// variable of this class, LastReply. watch for TELNET_EOF "\r\n"
void wuFtpSocket::GetReply() throw(wuSocketBadRead, wuSocketWaitTimeout, exception){

	char line[_MAX_LINE]={0};
	int intLineLength = 0;
	unsigned int ReplyState = 0;
	int Recieved =0, StartOfSubCommand = 0, EndOfSubCommand = 0, ReachedTelnetEOFTest=0;
	string TotalCommand="", SubCommand = "";
	// contains command or string, up to the first endline, not necessarily
	// the TELNET_EOF, If this is not a telnet oef, and have more characters to get ,will be required to iterate again

	LastReply.Reset();

	do
	{
		// This block is reponsible for getting the next block of response from the other party
		StartOfSubCommand = 0; // need to reset this every time we want to read a new line from socket
		memset((void*)line, 0, _MAX_LINE); // need to flush the line!!! 
		Recieved = fSocket.Receive(line, _MAX_LINE-1);
		if (Recieved==0)
		{
			throw wuSocketBadRead("peer closed connection"); // will we really want to do this ?
		}

		if (Recieved<0)
		{
			int intError = WSAGetLastError();
			if (intError == WSAETIMEDOUT)
			{
				throw wuSocketWaitTimeout(wuSocket::TranslateWinsockStatus(intError));
			}
			else	
			{
				throw wuSocketBadRead(wuSocket::TranslateWinsockStatus(intError)); // will we really want to do this ?
			}
		}

		TotalCommand = line;
		intLineLength = TotalCommand.size();

		do
		{
			EndOfSubCommand = TotalCommand.find_first_of(TELNET_EOF, StartOfSubCommand); 

			if	(	(EndOfSubCommand<intLineLength-1) && 
					(TotalCommand[EndOfSubCommand+1]=='\r' || 
					TotalCommand[EndOfSubCommand+1]=='\n'))
			{
						EndOfSubCommand++;		
			}
	
			SubCommand = TotalCommand.substr(StartOfSubCommand, (EndOfSubCommand - StartOfSubCommand)+1);
			StartOfSubCommand = EndOfSubCommand+1; // move pointer off the end line
			ReachedTelnetEOFTest = SubCommand.find(TELNET_EOF);

			try
			{
				ReplyState = LastReply.SetState(SubCommand);
			}
			catch(...)
			{
				throw;
			}

		}
		while(StartOfSubCommand<intLineLength-1);	

	}
	while(ReachedTelnetEOFTest<0 || ReplyState==wuFtpState::_FTP_WAITING_STATE);
}


