#include "winutils/wuFtpState.h"

#pragma warning( disable : 4290 ) 



// member variable DEFINITIONS

const unsigned int wuFtpState::_FTP_READY_STATE(1); 
const unsigned int wuFtpState::_FTP_WAITING_STATE(2);





wuFtpState::wuFtpState(){
			
	strMessage = "";
	fscLastReplyCode=0;
	intFtpReplyState = _FTP_READY_STATE;
}


wuFtpState::wuFtpState(std::string _strServerReply) throw (exception){

	strMessage = "";
	fscLastReplyCode=0;
	intFtpReplyState = _FTP_READY_STATE;

	try{

		SetState(_strServerReply);
	}
	catch(exception& e){
		throw exception(e.what());
	}
}



wuFtpState::wuFtpState(const wuFtpState& other){

	fscLastReplyCode=other.fscLastReplyCode;
	intFtpReplyState = other.intFtpReplyState;

	strMessage = other.strMessage;
}


wuFtpState& wuFtpState::operator=(const wuFtpState& other){

	if (this!=&other){

		fscLastReplyCode=other.fscLastReplyCode;
		intFtpReplyState = other.intFtpReplyState;		

		strMessage = other.strMessage;	
	}

	return *this;
}


int wuFtpState::SetState(std::string _strServerReply) throw (exception){

	std::string code="", message="", FirstChar="";

	// test for a white space at start of this line
	if (_strServerReply.size()<1){ 
		throw exception("test exception 1");
		//	return _FTP_WAITING_STATE;
	}
	else if (isspace(_strServerReply[0])) FirstChar=" ";

	wuStokenizer* tok = new wuStokenizer(_strServerReply, " ");

	if (tok->more()) code = tok->next();
	if (tok->more()) message = tok->UnTokenizedString();

	try{
		SetState(FirstChar + code, message); // intFtpReplyState will be set anew from here on
	}
	catch(exception& e){
		delete tok;
		throw exception(e.what());
	}

	delete tok;

	return intFtpReplyState; // let the user know where in the reply we are
}





// intFtpReplyState will be set anew from here on
int wuFtpState::SetState(std::string _strState, std::string _strMessage) throw (exception){

	unsigned int code=0;
	int HasLeadingWhiteSpace = false;
	bool HasHyphen = false, NumberValid = true;
	wuFtpStateCode NextCode= 0;

	HasLeadingWhiteSpace = isspace(_strState[0]);

	try{
		HasHyphen = GetCode(_strState.c_str(), code);
		if (code!=0) NextCode = code;
	}
	catch(exception& e){
		NumberValid = false; // need to know if we are in waiting state
	}

	if (intFtpReplyState==_FTP_WAITING_STATE){

		if (NumberValid && fscLastReplyCode==NextCode) intFtpReplyState = _FTP_READY_STATE;
	}
	else{ // if get here, we know number is valid, AND is in ready state (see try catch) 

		if (HasHyphen) intFtpReplyState = _FTP_WAITING_STATE;
		fscLastReplyCode = NextCode;
	}

	strMessage += _strState; // always do this
	if (_strMessage.size()>0) strMessage += " " + _strMessage;

	return intFtpReplyState;
}



bool wuFtpState::GetCode(std::string _strCode, unsigned int& uintCodeNum) throw (exception){

	_strCode = wuStokenizer::trim(_strCode);
	unsigned int code =0;
	uintCodeNum = code; // initially set to 0 to indicate nothing
	std::string ThreeChars = "";

	int GrabLen = _strCode.size();

	if (GrabLen<1) throw exception("Invalid Ftp Reply Code");
	else if (GrabLen>=3) GrabLen=3;

	ThreeChars = _strCode.substr(0,GrabLen);

	try
	{
		code = wu::Ascii2Integer(ThreeChars.c_str());
	}
	catch(exception& e){
		throw exception("Invalid Ftp Reply Code");
	}

	uintCodeNum = code;	// set again, cause by this stage, atoi has been successful
	
	// now we have to check for hyphen, for multiline purposes (see ftp protocol below)
	return ((int)_strCode.size()>GrabLen && _strCode[GrabLen]=='-');
}



void wuFtpState::Reset(){

	strMessage = "";
	fscLastReplyCode=0;
	intFtpReplyState = _FTP_READY_STATE;

}


