#include "winutils/wuManagedTCPSocket.h"




 
wuManagedTCPSocket::wuManagedTCPSocket(unsigned short	usintLocalPortRangeStart,
					unsigned short	usintLocalPortRangeEnd,
					ManagedPortFlagType _ManagementFlag) : wuTCPSocket()
{
	Settings(usintLocalPortRangeStart, usintLocalPortRangeEnd, _ManagementFlag);
}



wuManagedTCPSocket::wuManagedTCPSocket(	unsigned short	usintLocalPortRangeStart, unsigned short usintLocalPortRangeEnd, 
					unsigned short  usintRetries, ManagedPortFlagType _ManagementFlag) : wuTCPSocket()
{
	Settings(usintLocalPortRangeStart, usintLocalPortRangeEnd, usintRetries, _ManagementFlag);
}




void wuManagedTCPSocket::Settings(	unsigned short	usintLocalPortRangeStart, 
									unsigned short usintLocalPortRangeEnd, 
									ManagedPortFlagType _ManagementFlag)
{
	SetPortRange(usintLocalPortRangeStart, usintLocalPortRangeEnd);
	mintManagementFlag = (int)_ManagementFlag;
	musintRetries = musintLocalPortRangeEnd - musintLocalPortRangeStart +1;
}



void wuManagedTCPSocket::Settings(	unsigned short	usintLocalPortRangeStart, 
									unsigned short usintLocalPortRangeEnd, 
									unsigned short  usintRetries, 
									ManagedPortFlagType _ManagementFlag)
{
	SetPortRange(usintLocalPortRangeStart, usintLocalPortRangeEnd);
	mintManagementFlag = (int)_ManagementFlag;
	musintRetries = usintRetries;
}


void wuManagedTCPSocket::SetPortRange(	unsigned short	usintLocalPortRangeStart, 
										unsigned short usintLocalPortRangeEnd)
{
	musintLocalPortRangeStart = usintLocalPortRangeStart;
	musintLocalPortRangeEnd = usintLocalPortRangeEnd;

	// make sure that the smallest number is at 'Start
	if (musintLocalPortRangeStart > musintLocalPortRangeEnd)
	{
		unsigned short TempShort = musintLocalPortRangeStart;
		musintLocalPortRangeStart = musintLocalPortRangeEnd;
		musintLocalPortRangeEnd = TempShort;
	}
	
	muintLocalPort = musintLocalPortRangeStart;
}



void wuManagedTCPSocket::ManagedConnect(	const char* ahost, 
											unsigned short	usintLocalPortRangeStart, 
											unsigned short usintLocalPortRangeEnd,
											unsigned int intRemotePort, 
											unsigned int uintReceiveTimeout
											) throw (wuSocketException)
{
	Settings(usintLocalPortRangeStart, usintLocalPortRangeEnd, (ManagedPortFlagType)mintManagementFlag);

	ManagedConnect(ahost, intRemotePort, uintReceiveTimeout);
}




void wuManagedTCPSocket::IncrementCycleStage()
{
	muintLocalPort++;

	if (    (mintManagementFlag|(int)ManagedCycle) == mintManagementFlag && 
			(mintManagementFlag|(int)ManagedAnyPort) == mintManagementFlag) // if is a managed cycle with use of any also
	{
		if (muintLocalPort>musintLocalPortRangeEnd)
		{
			muintLocalPort = 0;
		}
		else if (muintLocalPort==0)
		{
			muintLocalPort = musintLocalPortRangeStart;
		}
	}
	else if ((mintManagementFlag|(int)ManagedCycle) == mintManagementFlag)// just wants named range only
	{
		if (muintLocalPort>musintLocalPortRangeEnd)
		{
			muintLocalPort = musintLocalPortRangeStart;
		}
	}
	else // any does not increment
	{
		muintLocalPort = 0;
	}
}




	/// ManagedConnect should be used as a client connect to a server or listening port.
	/// This function accepts I.P address OR Host-Name.
	/// This function will connect using the next specified local port available, dependant on 
	/// the control flags set in the constructor.
	/// This function will close the existing Socket before opening another.
void wuManagedTCPSocket::ManagedConnect(const char* ahost, unsigned int intRemotePort, unsigned int uintReceiveTimeout) throw (wuSocketException)
{
	bool blnSuccess = false;

	try
	{
		wuTCPSocket::Connect(ahost, muintLocalPort, intRemotePort, uintReceiveTimeout);
		blnSuccess = true;	// connect was successful
	}
	catch(...)
	{
		blnSuccess = false;	// connect was NOT successful
	}

	try
	{
		if (!blnSuccess)	// if first connect attempt didn't work
		{
			blnSuccess = CycleManagedReConnect(musintRetries);
			
			if (blnSuccess)
			{
				SetReceiveTimeout(uintReceiveTimeout);
			}
		}
	}
	catch(wuSocketException SockExcept)
	{
		throw SockExcept;
	}
	catch(...)
	{
		throw wuSocketException("Unknown error");
	}

	if (!blnSuccess)
	{
		throw wuSocketException("CycleManagedReConnect Failure");
	}
}



bool wuManagedTCPSocket::CycleManagedReConnect(int intSendReTry)
{
	bool blnSuccess = false;

	do
	{
		blnSuccess = TryReconnect();	
		intSendReTry--;
	}
	while (!blnSuccess && intSendReTry>0);
	
	return blnSuccess;
}


int wuManagedTCPSocket::Send(const char FAR* buf, int len)
{
	return wuManagedTCPSocket::Send(buf, len, 0);
}




int wuManagedTCPSocket::Send(const char FAR* buf, int len, int flags)
{
	WSASetLastError(0); // reset the error flag so that we know if something went wrong
	int intSentBytes = 0;
	int intCount = 0;
	bool Success = false;
	int intError = 0;

	intSentBytes = wuSocket::Send(buf, len, flags);

	if (intSentBytes<1 && intSentBytes<len)
	{
		intSentBytes = wuManagedTCPSocket::ReConnectAndSend(buf, len, flags, musintRetries);
	}

	return intSentBytes;
}



int wuManagedTCPSocket::ReConnectAndSend(const char FAR* buf, int len, int flags, int intRecursiveSendTry)
{
	bool Success = false;
	int intSentBytes = SOCKET_ERROR;

	Success = TryReconnect();

	if (Success)
	{
		intSentBytes = wuSocket::Send(buf, len, flags);
	}
	else if (intRecursiveSendTry>0)
	{ 
		intSentBytes = wuManagedTCPSocket::ReConnectAndSend(buf, len, flags, intRecursiveSendTry-1);
	}
	else
	{
		intSentBytes = SOCKET_ERROR;
	}

	return intSentBytes;
}



 

bool wuManagedTCPSocket::TryReconnect()
{
	bool blnSuccess = false;

	try
	{
		ForceReconnect();
		blnSuccess = true;
	}
	catch(wuSocketAddressInUse InUseException)
	{
		IncrementCycleStage(); // for next connect attempt (or send attempt) cycle the local port range
	}
	catch(...)
	{
	}

	return blnSuccess;
}

