#include "winutils/wuRegistry.h"
#include "winutils/wuStokenizer.h"
 

using namespace std;


// so far this function automatically creates the whole path. It SHOULD however provide the
// functionality to :
// 1. only create the leaf
// need to get the number of tokens, 
// if end up with a fail throw error
// if specified to only create node, then open is used for all but last key
// if in that case the open fails, throw error as normal.
// Should have, for convenience, another function which only opens, never creates.

long RegCreateKeyPath(	const char* Path, 
						HKEY StartKey,
						DWORD dwOptions, 
						REGSAM samDesired, 
						LPSECURITY_ATTRIBUTES lpSecurityAttributes,
						PHKEY phkResult,
						LPDWORD spdwDisposition) throw(){


	wuStokenizer tok(Path, "\\");
	string token="";
	long ReturnVal = ERROR_SUCCESS;
	HKEY LastKey=0;
	int Count =0;

	if (StartKey==NULL){

		if (tok.more()) token = tok.next(); // get the start hkey which is already known

		token = wuStokenizer::toUpperCase(token); // we know the hkey at the start is all upper case

		#if(WINVER >= 0x0400)

			if (token=="HKEY_CURRENT_CONFIG") LastKey =HKEY_CURRENT_CONFIG;
			else if (token=="HKEY_DYN_DATA") LastKey = HKEY_DYN_DATA;
			
		#endif /// WINVER

		if (token=="HKEY_CLASSES_ROOT")			LastKey=HKEY_CLASSES_ROOT;
		else if (token=="HKEY_CURRENT_USER")	LastKey=HKEY_CURRENT_USER;
		else if (token=="HKEY_LOCAL_MACHINE")	LastKey=HKEY_LOCAL_MACHINE;
		else if (token=="HKEY_USERS")			LastKey=HKEY_USERS;
		else if (token=="HKEY_PERFORMANCE_DATA") LastKey=HKEY_PERFORMANCE_DATA;
	}
	else LastKey = StartKey; // if specified, have it start here instead of 
								// spot indicated by first token

	while (tok.more() && ReturnVal==ERROR_SUCCESS){

		token = tok.next();

		ReturnVal = RegCreateKeyEx(LastKey, (char*)token.c_str(), 0, (char*)token.c_str(), 
						dwOptions, samDesired, lpSecurityAttributes,
						phkResult, spdwDisposition);

		if (Count++) RegCloseKey(LastKey);   // handle to key to close
		LastKey = *phkResult;
	}

	return ReturnVal;
}



long RegOpenKeyPath(	const char* Path, 
						HKEY StartKey,
						REGSAM samDesired, 
						PHKEY phkResult) throw(){

	wuStokenizer tok(Path, "\\");
	string token="";
	long ReturnVal = ERROR_SUCCESS;
	HKEY LastKey=0;
	int Count =0;

	if (StartKey==NULL){

		if (tok.more()) token = tok.next(); // get the start hkey which is already known

		token = wuStokenizer::toUpperCase(token); // we know the hkey at the start is all upper case

		#if(WINVER >= 0x0400)

			if (token=="HKEY_CURRENT_CONFIG") *phkResult =HKEY_CURRENT_CONFIG;
			else if (token=="HKEY_DYN_DATA") *phkResult = HKEY_DYN_DATA;
			
		#endif /// WINVER

		if (token=="HKEY_CLASSES_ROOT")			LastKey=HKEY_CLASSES_ROOT;
		else if (token=="HKEY_CURRENT_USER")	LastKey=HKEY_CURRENT_USER;
		else if (token=="HKEY_LOCAL_MACHINE")	LastKey=HKEY_LOCAL_MACHINE;
		else if (token=="HKEY_USERS")			LastKey=HKEY_USERS;
		else if (token=="HKEY_PERFORMANCE_DATA") LastKey=HKEY_PERFORMANCE_DATA;
	}
	else LastKey = StartKey; // if specified, have it start here instead of 
								// spot indicated by first token

	while (tok.more() && ReturnVal==ERROR_SUCCESS){

		token = tok.next();

		ReturnVal = RegOpenKeyEx(LastKey, (char*)token.c_str(), 0,
						samDesired, phkResult);

		if (Count++) RegCloseKey(LastKey);   // handle to key to close
		LastKey = *phkResult;
	}

	return ReturnVal;
}

