
#include "winutils/wuResourceUtils.h"
#include <tchar.h>
#include "winutils/wuStokenizer.h"
#include "winutils/wuStdUtils.h"

using namespace std;



// VS_FIXEDFILEINFO 
string wu::GetVersionInfo(HMODULE hModule)
{
	string Result = "Unknown Version";
	VS_FIXEDFILEINFO* TempStruct;

	if ( HRSRC hVersion = FindResource(hModule, MAKEINTRESOURCE(VS_VERSION_INFO), RT_VERSION ) )
	{
		if ( HGLOBAL hGlobal = LoadResource(hModule, hVersion ) )
		{
			WORD* pVersionInfo = (WORD*)LockResource( hGlobal );
			DWORD* p = (DWORD*)pVersionInfo;

			// Looks for the Value signature: 0xFEEF04BD
			for ( int i = 0; i < (*pVersionInfo) / 2; i++, p++ )
			{
				if ( *p  == 0xFEEF04BD )
				{
					
					TempStruct = (VS_FIXEDFILEINFO*)p; // for future functionality in getting more info easier
					// Jumps the signature and the structure version fields.
					p += 2;
					DWORD dwMS = *p++, dwLS = *p;
					TCHAR szVersion[1024] = _T("");
					wsprintf( szVersion, "%i.%i.%i.%i", HIWORD( dwMS ), LOWORD(dwMS), HIWORD( dwLS ), LOWORD( dwLS ) );
					Result = szVersion;
					break;
				}
			}

			UnlockResource( hGlobal );
			FreeResource( hGlobal );
		}
	}

	return Result;
}





bool wu::CheckVersionInfo(	HMODULE hModule, char* pcharModuleVersion,
						unsigned int uintVersionStringBufLen, const char* cpcharMinimumVersion)
{
	VS_FIXEDFILEINFO* TempStruct;
	unsigned int uintaMinVersionNumbers[4];
	wuStokenizer tok(cpcharMinimumVersion, "., ");
	bool blnMinVerNumGood = false;
	bool blnGoodVersion = false;
	unsigned int uintCount = 0;

	wu::strlcpy(pcharModuleVersion, "Unknown Version", uintVersionStringBufLen);

	// first get the four numbers from the min version string
	try
	{
		while (tok.more() && uintCount<4)
		{
			uintaMinVersionNumbers[uintCount] = wu::Ascii2Integer(tok.next().c_str());
			uintCount++;
		}

		blnMinVerNumGood = true;
	}
	catch(...)
	{// no good version string. Don't bother with checking, return false, and the 
	}


	if ( HRSRC hVersion = FindResource(hModule, MAKEINTRESOURCE(VS_VERSION_INFO), RT_VERSION ) )
	{
		if ( HGLOBAL hGlobal = LoadResource(hModule, hVersion ) )
		{
			WORD* pVersionInfo = (WORD*)LockResource( hGlobal );
			DWORD* p = (DWORD*)pVersionInfo;

			// Looks for the Value signature: 0xFEEF04BD
			for ( int i = 0; i < (*pVersionInfo) / 2; i++, p++ )
			{
				if ( *p  == 0xFEEF04BD )
				{
					
					TempStruct = (VS_FIXEDFILEINFO*)p; // for future functionality in getting more info easier
					// Jumps the signature and the structure version fields.
					p += 2;
					DWORD dwMS = *p++, dwLS = *p;
					TCHAR szVersion[1024] = _T("");
					wsprintf( szVersion, "%i.%i.%i.%i", HIWORD( dwMS ), LOWORD(dwMS), HIWORD( dwLS ), LOWORD( dwLS ) );
					wu::strlcpy(pcharModuleVersion, szVersion, uintVersionStringBufLen);

					blnGoodVersion = (uintaMinVersionNumbers[0]<=HIWORD( dwMS ));

					if (blnGoodVersion)
					{
						blnGoodVersion = (uintaMinVersionNumbers[1]<=LOWORD(dwMS));

						if (blnGoodVersion)
						{
							blnGoodVersion = (uintaMinVersionNumbers[2]<=HIWORD( dwLS ));
					
							if (blnGoodVersion)
							{
								blnGoodVersion = (uintaMinVersionNumbers[3]<=LOWORD( dwLS ));
							}
						}
					}

					break;
				}
			}

			UnlockResource( hGlobal );
			FreeResource( hGlobal );
		}
	}

	return (blnGoodVersion && blnMinVerNumGood);
}