#include "winutils/wuSYSTEMTIME.h"
#include "winutils/wuCalendar.h"
#include <stdio.h>
#include "winutils/wuStdUtils.h"

wuSYSTEMTIME::wuSYSTEMTIME()
{

}

wuSYSTEMTIME::~wuSYSTEMTIME()
{

}




wuSYSTEMTIME::wuSYSTEMTIME(const SYSTEMTIME& DateTime)
{
	SYSTEMTIME::wYear			= DateTime.wYear;
	SYSTEMTIME::wMonth			= DateTime.wMonth;
	SYSTEMTIME::wDayOfWeek		= DateTime.wDayOfWeek;
	SYSTEMTIME::wDay			= DateTime.wDay;
	SYSTEMTIME::wHour			= DateTime.wHour;
	SYSTEMTIME::wMinute			= DateTime.wMinute;
	SYSTEMTIME::wSecond			= DateTime.wSecond;
	SYSTEMTIME::wMilliseconds	= DateTime.wMilliseconds;
}



wuSYSTEMTIME& wuSYSTEMTIME::operator=(const SYSTEMTIME& DateTime)
{
	if (this!=&DateTime)
	{
		SYSTEMTIME::wYear			= DateTime.wYear;
		SYSTEMTIME::wMonth			= DateTime.wMonth;
		SYSTEMTIME::wDayOfWeek		= DateTime.wDayOfWeek;
		SYSTEMTIME::wDay			= DateTime.wDay;
		SYSTEMTIME::wHour			= DateTime.wHour;
		SYSTEMTIME::wMinute			= DateTime.wMinute;
		SYSTEMTIME::wSecond			= DateTime.wSecond;
		SYSTEMTIME::wMilliseconds	= DateTime.wMilliseconds;
	}

	return *this;
}




wuSYSTEMTIME::wuSYSTEMTIME(const double DateTime)
{
	ExtractDateTime(*this, DateTime);
}



wuSYSTEMTIME& wuSYSTEMTIME::operator=(const double DateTime)
{
	ExtractDateTime(*this, DateTime);

	return *this;
}



double wuSYSTEMTIME::GetAsDouble()
{
	return CreateDateAndTime(*this);
}



double wuSYSTEMTIME::CreateDateAndTime(const SYSTEMTIME& TimeAndDate)
{
	double	dblDate = 0.0;

    // Excel/Lotus 123 have a bug with 29-02-1900. 1900 is not a
    // leap year, but Excel/Lotus 123 think it is...
    if (TimeAndDate.wDay == 29 && TimeAndDate.wMonth == 02 && TimeAndDate.wYear==1900)
	{
		dblDate = 60.0000000;
	}
	else
	{
		// DMY to Modified Julian calculatie with an extra substraction of 2415019.
		dblDate =	int(( 1461 * ( TimeAndDate.wYear + 4800 + int(( TimeAndDate.wMonth - 14 ) / 12) ) ) / 4) +
					int(( 367 * ( TimeAndDate.wMonth - 2 - 12 * ( ( TimeAndDate.wMonth - 14 ) / 12 ) ) ) / 12) -
					int(( 3 * ( int(( TimeAndDate.wYear + 4900 + int(( TimeAndDate.wMonth - 14 ) / 12) ) / 100) ) ) / 4) +
					TimeAndDate.wDay - 2415019 - 32075;
	}

    if (dblDate < 60)
    {
        // Because of the 29-02-1900 bug, any serial date 
        // under 60 is one off... Compensate.
        dblDate--;
    }

	  // now add the time component
	dblDate += ((double)TimeAndDate.wHour/24.0) + ((double)TimeAndDate.wMinute/1440.0) +
				((double)TimeAndDate.wSecond/86400.0) + ((double)TimeAndDate.wMilliseconds/86400000.0);

	return (dblDate);
}





void wuSYSTEMTIME::ExtractDateTime(SYSTEMTIME& TimeAndDate, double dblSourceDate)
{
    // Excel/Lotus 123 have a bug with 29-02-1900. 1900 is not a
    // leap year, but Excel/Lotus 123 think it is...
    if (dblSourceDate == 60)
    {
        TimeAndDate.wDay    = 29;
        TimeAndDate.wMonth    = 2;
        TimeAndDate.wYear    = 1900;
    }
    else if (dblSourceDate < 60)
    {
        // Because of the 29-02-1900 bug, any serial date 
        // under 60 is one off... Compensate.
        dblSourceDate++;
    }

	// Modified Julian to DMY calculation with an addition of 2415019
	int l = ((int)dblSourceDate) + 68569 + 2415019;
	int n = int(( 4 * l ) / 146097);
	l = l - int(( 146097 * n + 3 ) / 4);
	int i = int(( 4000 * ( l + 1 ) ) / 1461001);
	l = l - int(( 1461 * i ) / 4) + 31;
	int j = int(( 80 * l ) / 2447);
	TimeAndDate.wDay = l - int(( 2447 * j ) / 80);
	l = int(j / 11);
	TimeAndDate.wMonth = j + 2 - ( 12 * l );
	TimeAndDate.wYear = 100 * ( n - 49 ) + i + l;

	// now do the time component
	double dblTimeComponent = dblSourceDate - (long)dblSourceDate;

	double dblHours = dblTimeComponent * 24.0;
	TimeAndDate.wHour = (DWORD)dblHours;

	double dblMinutes = (dblHours - (double)TimeAndDate.wHour)*60.0;
	TimeAndDate.wMinute = ((DWORD)dblMinutes);

	double dblSeconds = (dblMinutes - (double)TimeAndDate.wMinute) * 60.0;
	TimeAndDate.wSecond = ((DWORD)dblSeconds);

	double dblMilli = (dblSeconds - (double)TimeAndDate.wSecond) * 1000.0;
	TimeAndDate.wMilliseconds = dblMilli;

	TimeAndDate.wDayOfWeek = wuCalendar::GetDayOfWeek(TimeAndDate.wYear, TimeAndDate.wMonth, TimeAndDate.wDay);
}



unsigned int wuSYSTEMTIME::FormatDate(	char*					pcharBuffer, 
									unsigned int			uintBuffLen,
									const char*				strFormatString,
									const wuSYSTEMTIME&	DateTime)
{
	const char			chrSeparator = ' ';
	char				chrLastChar='\0';
	char*				strMonth=NULL;
	const char*			strPlaceHolder = NULL;
	bool				blnFinished = false;
	bool				blnNewSymbol = false;
	char				pcharTempBuffer[1024]={0};
	char*				strDateString = pcharTempBuffer;
	char*				strMonthNames[] = { "Jan", "Feb", "Mar", "Apr", "May",
											"Jun", "Jul", "Aug", "Sep", "Oct",
											"Nov", "Dec" };
	  // intialise the buffer
	memset(pcharBuffer, 0, uintBuffLen);

	  // if we have not supplied a string, default it
	if (strFormatString==NULL || strcmp(strFormatString, "")==0)
	{
		strFormatString = "DD MMM YYYY";
	}

	  // now format the date according to strFormatString
	const char*	strPtr = strFormatString;
	int			intCounter = 0;
	int			intNumber = 0;

	while (!blnFinished)
	{
		switch (*strPtr)
		{
		case 'D':
		case 'd':
		case 'M':
		case 'Y':
		case 'y':
		case 'm':
		case 'h':
		case 's':
			  // check if we are switching between symbols without a separator
			if (intCounter>0 && *strPtr != chrLastChar)
			{
				  // we need to print the characters accumulated up until now,
				  // but also come back to this character later
				blnNewSymbol = true;
				break;
			} else
			{
				intCounter++;
				switch (*strPtr)
				{
				case 'D':
				case 'd':
					intNumber = DateTime.wDay;
					break;
				case 'M':
					intNumber = DateTime.wMonth;
					break;
				case 'Y':
				case 'y':
					intNumber = DateTime.wYear;
					break;
				case 'm':
					intNumber = DateTime.wMinute;
					break;
				case 'h':
					intNumber = DateTime.wHour;
					break;
				case 's':
					intNumber = DateTime.wSecond;
					break;
				}
				break;
			}
		case ' ':
		case '/':
		case '.':
		case '-':
		case ':':
		case '_':
		case '\0':
		case ';':
			  // this is a separator character, print what we've got
			  // since the last separator into the string
			if (intCounter>2 && chrLastChar=='M')
				  // we want to look up the month as a string
				sprintf(strDateString, 
						"%s%c", 
						strMonthNames[DateTime.wMonth-1],
						*strPtr);
			else
			{
				if (intCounter>0)
				{
					if (intCounter>1 && intNumber <10)
						  // we have to pad the number
						sprintf(strDateString,
								"0%d%c",
								intNumber,
								*strPtr);
					else
					{
						  // just print in the number as we have it
						sprintf(strDateString,
								"%d%c",
								intNumber,
								*strPtr);
						  // if this is a year, we may only want 2 digit display
						if (intCounter==2 && chrLastChar=='Y')
							  // reduce to a 2 digit display
							strcpy(strDateString, strDateString+2);
					}
				} else
					  // this is just a repeated separator
					sprintf(strDateString, 
							"%c",
							*strPtr);
			}
			  // reset the counter
			intCounter = 0;
			  // get ready for the next loop
			strDateString += strlen(strDateString);
			if (blnNewSymbol)
			{
				  // if we last switched without a separator, get rid of the
				  // artificial separator that will have been appended to the string
				strDateString--;
				  // this past append was triggered by a change of symbols
				  // not separated by a separator character. We need to get
				  // strPtr back to pointing to the input string
				strPtr = strPlaceHolder;
				blnNewSymbol = false;
				  // step back one as it will be incremented
				strPtr--;
			}
			break;
		}
		if (blnNewSymbol)
		{
			  // let the pointer point to a mock separator
			strPlaceHolder = strPtr;
			strPtr = &chrSeparator;
		} else
		{
			  // make a note of the last character read
			chrLastChar = *strPtr;
			if (*strPtr=='\0')
				blnFinished = true;
			strPtr++;
		}
	}

	  // copy as much of the string as we should into the provided buffer.
	return wu::strlcpy(pcharBuffer, pcharTempBuffer, uintBuffLen);
}




BOOL wuSYSTEMTIME::LocalTime(SYSTEMTIME* GMTime, SYSTEMTIME* LocalTime)
{

	return SystemTimeToTzSpecificLocalTime(NULL, GMTime, LocalTime);
}







bool operator<(const FILETIME& Left, const FILETIME& Right) 
{
	LONG Result = CompareFileTime(&Left, &Right);
 
	return (Result<0);
}




bool operator>(const FILETIME& Left, const FILETIME& Right) 
{
	LONG Result = CompareFileTime(&Left, &Right);
 
	return (Result>0);
}




bool operator<(const SYSTEMTIME& Left, const SYSTEMTIME& Right) 
{
	FILETIME ftLeft;
	FILETIME ftRight;

	SystemTimeToFileTime(&Left, &ftLeft);
	SystemTimeToFileTime(&Right, &ftRight);		

	LONG Result = CompareFileTime(&ftLeft, &ftRight);
 
	return (Result<0);
}



bool operator==(const SYSTEMTIME& Left, const SYSTEMTIME& Right) 
{
	FILETIME ftLeft;
	FILETIME ftRight;

	SystemTimeToFileTime(&Left, &ftLeft);
	SystemTimeToFileTime(&Right, &ftRight);		

	LONG Result = CompareFileTime(&ftLeft, &ftRight);
 
	return (Result==0);
}



bool operator!=(const SYSTEMTIME& Left, const SYSTEMTIME& Right)
{
	FILETIME ftLeft;
	FILETIME ftRight;

	SystemTimeToFileTime(&Left, &ftLeft);
	SystemTimeToFileTime(&Right, &ftRight);		

	LONG Result = CompareFileTime(&ftLeft, &ftRight);
 
	return (Result!=0);
}





bool operator>(const SYSTEMTIME& Left, const SYSTEMTIME& Right) 
{
	FILETIME ftLeft;
	FILETIME ftRight;

	SystemTimeToFileTime(&Left, &ftLeft);
	SystemTimeToFileTime(&Right, &ftRight);		

	LONG Result = CompareFileTime(&ftLeft, &ftRight);

	return (Result>0);
}


SYSTEMTIME operator-(const SYSTEMTIME& Left, const SYSTEMTIME& Right)
{
	FILETIME ftLeft;
	FILETIME ftRight;
	FILETIME ftRes;
	ULARGE_INTEGER ularLeft;
	ULARGE_INTEGER ularRight;
	ULARGE_INTEGER ularRes;
	SYSTEMTIME Res;

	SystemTimeToFileTime(&Left, &ftLeft);
	SystemTimeToFileTime(&Right, &ftRight);	

	ularLeft.LowPart        =   ftLeft.dwLowDateTime; 
	ularLeft.HighPart       =   ftLeft.dwHighDateTime; 
	ularRight.LowPart       =   ftRight.dwLowDateTime; 
	ularRight.HighPart      =   ftRight.dwHighDateTime; 

	// Subtracts the lower and higher parts of the ULARGE_INTEGER structure 
	ularRes.HighPart  =   ularLeft.HighPart - ularRight.HighPart; 
	ularRes.LowPart   =   ularLeft.LowPart  - ularRight.LowPart; 

	ftRes.dwLowDateTime      =   ularRes.LowPart; 
	ftRes.dwHighDateTime     =   ularRes.HighPart;
	
	FileTimeToSystemTime( &ftRes, &Res); 

	return Res;
}





SYSTEMTIME operator+(const SYSTEMTIME& Left, const SYSTEMTIME& Right) 
{
	FILETIME ftLeft;
	FILETIME ftRight;
	FILETIME ftRes;
	ULARGE_INTEGER ularLeft;
	ULARGE_INTEGER ularRight;
	ULARGE_INTEGER ularRes;
	SYSTEMTIME Res;

	SystemTimeToFileTime(&Left, &ftLeft);
	SystemTimeToFileTime(&Right, &ftRight);	

	ularLeft.LowPart        =   ftLeft.dwLowDateTime; 
	ularLeft.HighPart       =   ftLeft.dwHighDateTime; 
	ularRight.LowPart       =   ftRight.dwLowDateTime; 
	ularRight.HighPart      =   ftRight.dwHighDateTime; 

	// adds the lower and higher parts of the ULARGE_INTEGER structure 
	ularRes.HighPart  =   ularLeft.HighPart + ularRight.HighPart; 
	ularRes.LowPart   =   ularLeft.LowPart  + ularRight.LowPart; 

	ftRes.dwLowDateTime      =   ularRes.LowPart; 
	ftRes.dwHighDateTime     =   ularRes.HighPart;
	
	FileTimeToSystemTime( &ftRes, &Res); 

	return Res;
}