#include "winutils/wuSocket.h"



//////////////////////// Socket wrapper class ///////////////

wuSocket::wuSocket()
{
	WSADATA			wsaData;

	mpcharHost=0;  // none (can be used for standard out or in?)
	Sockfd=0;
	Connfd=0;
	muintRemotePort=0;
	muintUsedLocalPort=0;
	muintLocalPort=0;
	mintClilen=0;

	int intReturn = WSAStartup(MAKEWORD(2, 2), &wsaData);

	if (intReturn==NULL)
	{
		RecordSuccessfulAction();
	}
}



// this constructor must be followed by Establish
// Socket constructor mainly used for ServerSocket
wuSocket::wuSocket(int intLocalPortNum)  throw (wuSocketException)
{
	WSADATA			wsaData;

	mpcharHost = 0;
	Sockfd=0;
	Connfd=0;

	muintLocalPort = intLocalPortNum;
	muintUsedLocalPort=0;
	muintRemotePort = 0;

	int intReturn = WSAStartup(MAKEWORD(2, 2), &wsaData);

	if (intReturn==NULL)
	{
		RecordSuccessfulAction();
	}

	SetAddress(&local_addr, intLocalPortNum);

}
/////////////////////




// copy constructor
wuSocket::wuSocket(const wuSocket & other)
{  
	// copy the socket descriptors and make a copy of the host string
	WSADATA			wsaData;
	Sockfd = other.Sockfd;
	Connfd = other.Connfd;
	muintRemotePort   = other.muintRemotePort;
	muintLocalPort	  = other.muintLocalPort;
	muintUsedLocalPort= other.muintLocalPort;

	mintClilen = other.mintClilen;
	
	memset((char*) &local_addr, 0, sizeof(local_addr));

	local_addr.sin_family = other.local_addr.sin_family; 
	local_addr.sin_addr.s_addr = other.local_addr.sin_addr.s_addr; 
	local_addr.sin_port = other.local_addr.sin_port; 

	if (other.mpcharHost!=0)
	{
		int tlen = strlen(other.mpcharHost);
		mpcharHost = new char[tlen+1];
		strcpy(mpcharHost, other.mpcharHost);
	}
	else mpcharHost=0;  
	
	int intReturn = WSAStartup(MAKEWORD(2, 2), &wsaData);

	if (intReturn==NULL)
	{
		RecordSuccessfulAction();
	}
}





wuSocket::~wuSocket()
{
	wuSocket::Close();
	WSACleanup();
}





void wuSocket::SetAddress(struct sockaddr_in* an_addr, int intPortNum)
{
	memset((char*) an_addr, 0, sizeof(*an_addr));
	an_addr->sin_family = AF_INET; 
	an_addr->sin_addr.s_addr=htonl(INADDR_ANY); 
	an_addr->sin_port = htons(intPortNum); 
	RecordSuccessfulAction();
}




void wuSocket::ForceReconnect() throw (wuSocketException)
{
	if (muintRemotePort==0 && muintUsedLocalPort==0)
	{
		throw wuSocketException("ForceReconnect: No previous client connection");
	}
	else
	{
		// first reset all, so to force a restart
		unsigned int uintTempLocalPort = muintLocalPort;
		unsigned int uintTempRemotePort = muintRemotePort;

		// check that the host string has been set up before this call can complete successfully.
		if (mpcharHost==0)
		{
			throw wuSocketException("wuSocket::ForceReconnect: Host has not been set");
		}

		char* strTempHost = 0;
		int len = strlen(mpcharHost) + 1;
		strTempHost = new char[len];
		strcpy(strTempHost, mpcharHost);

		// if the socket is already connected, we must first disconnect
		if (IsConnected())
		{
			Close();
		}

		delete[] mpcharHost;
		mpcharHost = 0;
		muintLocalPort = 0;
		muintUsedLocalPort = 0;
		muintRemotePort = 0;

		try
		{
			// then connect using those same parameters
			Connect(strTempHost, uintTempLocalPort, uintTempRemotePort);
		}
		catch(wuSocketAddressInUse AddrInUse)
		{
			delete[] strTempHost;			
			throw AddrInUse;
		}
		catch(wuSocketException &e)
		{
			delete[] strTempHost;			
			throw e;
		}
		catch(...)
		{
			delete[] strTempHost;
			throw wuSocketException("wuSocket::ForceReconnect:Unknown Error in Connect");
		}

		delete[] strTempHost;
	}
}



void wuSocket::Reconnect() throw (wuSocketException)
{
	if (mpcharHost!=0) // Only do a reconnect if it was once connected before.
	{
		try
		{
			ForceReconnect();	// force a close and then connect again.
		}
		catch(wuSocketException& e)
		{
			throw e;
		}
		catch(...)
		{
			throw wuSocketException("wuSocket::Reconnect: Unknown Error");
		}
	}
	else
	{
		throw wuSocketException("wuSocket::Reconnect: Host never was connected");
	}
}








void wuSocket::Connect(const char* ahost, unsigned int RemotePort) throw (wuSocketException, wuSocketAddressInUse)
{
	Connect(ahost, 0, RemotePort);
}



void wuSocket::Connect(const char* ahost, unsigned int intLocalPort, unsigned int intRemotePort, unsigned int uintReceiveTimeout) throw (wuSocketException, wuSocketAddressInUse)
{
	Connect(ahost, intLocalPort, intRemotePort);
	SetReceiveTimeout(uintReceiveTimeout);
}





void wuSocket::CreateSocket(const char* ahost, unsigned int uintLocalPort, unsigned int uintRemotePort, int intType) throw (wuSocketException, wuSocketAddressInUse)
{
	int NameLen = 0;
	int intReturn = 0;

	WSASetLastError(0); // reset the error flag so that we know if something went wrong

	if (IsConnected())
	{
		if ( (mpcharHost !=0 && (strcmp(ahost, mpcharHost)!=0)) || muintLocalPort!=uintLocalPort || muintRemotePort!=uintRemotePort)
		{
			wuSocket::Close();
		}
		else return;	// else Connect is being called when it is already connected to the same address
	}

	int hlen = strlen(ahost);
	mpcharHost = new char[hlen+1];
	strcpy(mpcharHost, ahost);

	Connfd=0;
	muintRemotePort = uintRemotePort;
	muintLocalPort = uintLocalPort;

	memset((char*) &remote_addr, 0, sizeof(remote_addr));
	
	if ((host_ptr = gethostbyname(mpcharHost)) == 0)
	{
		char errbuff[1024]={0};
		sprintf(errbuff, "Socket::Connect:1: error opening socket - can't gethostbyname because %d", WSAGetLastError ());
		throw wuSocketException(errbuff);
	}

	if (host_ptr->h_addrtype!= AF_INET && host_ptr->h_addrtype!= AF_INET6)
	{
		char errbuff[1024]={0};
		sprintf(errbuff, "Socket::Connect:2: error opening socket - can't bind because %d", WSAGetLastError ());
		throw wuSocketException(errbuff);
	}
	
	if ((Sockfd = socket(host_ptr->h_addrtype, intType, 0))<0)
	{
		char errbuff[1024]={0};
		sprintf(errbuff, "Socket::Connect:3: error opening socket - socket failed because %d", WSAGetLastError ());
		throw wuSocketException(errbuff);
	}

	local_addr.sin_family = host_ptr->h_addrtype; // AF_INET OR AF_INET6
	local_addr.sin_addr.s_addr = htonl(INADDR_ANY);
// should be remote	
	remote_addr.sin_addr.s_addr = ((struct in_addr*) host_ptr->h_addr_list[0])->s_addr;
	remote_addr.sin_family = host_ptr->h_addrtype; // AF_INET OR AF_INET6

	local_addr.sin_port = htons(muintLocalPort);
	remote_addr.sin_port = htons(uintRemotePort);

	intReturn = bind (Sockfd, (struct sockaddr*) &local_addr, sizeof(local_addr));// bind the socket

	if (intReturn!=0)
	{
		int intWSAError = WSAGetLastError();

		if (intWSAError == WSAEADDRINUSE)
		{
			char errbuff[1024]={0};
			sprintf(errbuff, "Socket::Connect:4: error opening socket - can't bind because %d", WSAGetLastError ());
			throw wuSocketAddressInUse(errbuff);
		}
		else
		{
			char errbuff[1024]={0};
			sprintf(errbuff, "Socket::Connect:4: error opening socket - can't bind because %d", WSAGetLastError ());
			throw wuSocketException(errbuff);
		}
	}

	// get the local port
	memset((char*) &local_addr, 0, sizeof(remote_addr));
	NameLen = sizeof(local_addr);
	intReturn = getsockname(Sockfd, (struct sockaddr*) &local_addr, &NameLen);
	
	if (intReturn==0)
	{
		muintUsedLocalPort = ntohs(local_addr.sin_port);// convert the network (big endian) to host (windows = little endian)
	}

	RecordSuccessfulAction();
}




void wuSocket::ConnectOnSocket() throw (wuSocketException, wuSocketAddressInUse)
{
	if (connect(Sockfd , (struct sockaddr*) & remote_addr, sizeof(remote_addr)) <0){

		char errbuff[1024]={0};
		DWORD dwErr = WSAGetLastError();
		sprintf(errbuff, "Socket::Connect:5: error opening socket - can't connect because %d", 
							dwErr,
							wuSocket::TranslateWinsockStatus(dwErr));

		if (dwErr==10048)
		{
			throw wuSocketAddressInUse(errbuff);
		}
		else
		{
			throw wuSocketException(errbuff);
		}
	}

	SetReceiveTimeout(0);

	RecordSuccessfulAction();
}







void wuSocket::Establish() throw (wuSocketException)
{
	int optval=1; //set to turn socket option REUSE_ADDR on=1

		// if this is already opened, we should close the socket first before opening another for 
		// listening.
	if (Connfd!=0)
	{
		CloseConnector();
	}

	if ((Connfd = socket(AF_INET, SOCK_STREAM, 0))<0)
	{
		char errbuffer[1024]={0};
		sprintf(errbuffer, "Socket::Establish: error opening socket");
		throw wuSocketException(errbuffer);
	}

	// set reuse address option so if the application closes uncleanly, we can restart the socket
	// listening on the same port (address)
	setsockopt(Connfd, SOL_SOCKET, SO_REUSEADDR, (const char FAR *)&optval, sizeof(optval ));
	
	// bind the socket to a local address (port) 
	if (bind(Connfd, (struct sockaddr*) &local_addr, sizeof(local_addr))!=0)
	{
		char errbuff[1024]={0};
		sprintf(errbuff, "error opening socket - can't bind because %d", WSAGetLastError ());
		throw wuSocketException(errbuff);
	}

	// set up the listen to the socket
	listen(Connfd, 10); // this call will not fail because all reasons for failure have been controled inside the class

	RecordSuccessfulAction();
}





int wuSocket::Accept() throw(wuSocketException)
{

	mintClilen = sizeof(remote_addr);
	WSASetLastError(0);

	// blocking accept call to wait for a client connection. Returns once a client has connected, 
	// and produces a Socket descriptor to use in communications.
	Sockfd = accept(Connfd, (struct sockaddr FAR*) &remote_addr, (int FAR*)&mintClilen);

	if (Sockfd==INVALID_SOCKET)
	{
		char errbuff[1024]={0};
		int intTempErrorCode = WSAGetLastError();
		sprintf(errbuff, "Socket::Accept %d - %s", intTempErrorCode, TranslateWinsockStatus(intTempErrorCode));

		if (intTempErrorCode==WSAEINTR)
		{
			throw wuSocketInterrupt(errbuff);
		}
		else
		{
			throw wuSocketException(errbuff);
		}
	}

	try
	{
		SetReceiveTimeout(0);
	}
	catch(wuSocketException& e)
	{
	}

	return Sockfd;
}
////////////////////////






int wuSocket::SetReceiveTimeout(int intTimeMilli) throw (wuSocketException){

	int timeout = intTimeMilli;

	int success=setsockopt(Sockfd, SOL_SOCKET, SO_RCVTIMEO, 
							(const char FAR *)&timeout, sizeof(int));

	if (success==SOCKET_ERROR)
	{
		char errbuff[1024]={0};
		sprintf(errbuff, "RCVTIMEO UNSUPPORTED %d", WSAGetLastError ());
		throw wuSocketException(errbuff);
	}
	else RecordSuccessfulAction();	

	return 0;
}



int wuSocket::SetSendTimeout(int intTimeMilli) throw (wuSocketException){

	int timeout = intTimeMilli;

	int success=setsockopt(Sockfd, SOL_SOCKET, SO_SNDTIMEO, 
							(const char FAR *)&timeout, sizeof(int));

	if (success==SOCKET_ERROR)
	{
		char errbuff[1024]={0};
		sprintf(errbuff, "SO_SNDTIMEO UNSUPPORTED %d", WSAGetLastError ());
		throw wuSocketException(errbuff);
	}
	else RecordSuccessfulAction();

	return 0;
}


	/// Set the total per-socket buffer space reserved for receives. (SO_RCVBUF)
void wuSocket::SetReceiveBufferSize(int _intSize)
{
	int intSize = _intSize;

	int success=setsockopt(Sockfd, SOL_SOCKET, SO_RCVBUF, 
							(const char FAR *)&intSize, sizeof(int));

	if (success==SOCKET_ERROR)
	{
		char errbuff[1024]={0};
		sprintf(errbuff, "SO_RCVBUF UNSUPPORTED %d", WSAGetLastError ());
		throw wuSocketException(errbuff);
	}
	else RecordSuccessfulAction();
}




	/// Get the total per-socket buffer space reserved for receives. (SO_RCVBUF)
int wuSocket::GetReceiveBufferSize()
{
	int intSize =0;
	int intLen = sizeof(int);

	int success=getsockopt(Sockfd, SOL_SOCKET, SO_RCVBUF, 
							(char FAR *)&intSize, &intLen);

	if (success==SOCKET_ERROR)
	{
		char errbuff[1024]={0};
		sprintf(errbuff, "SO_RCVBUF UNSUPPORTED %d", WSAGetLastError ());
		throw wuSocketException(errbuff);
	}
	else RecordSuccessfulAction();

	return intSize;
}






wuSocket& wuSocket::operator=(const wuSocket & other)
{ 
	if (this!=&other){

		if (Sockfd!=0 && other.Sockfd!=Sockfd)
		{
			Close();  // if not closed, close now before now taking a new one
		}

		Sockfd = other.Sockfd;
		Connfd = other.Connfd; // leave this alone , might still need to be open for server

		muintRemotePort   = other.muintRemotePort;
		mintClilen = other.mintClilen;
		
		if (other.mpcharHost!=0)
		{
			int tlen = strlen(other.mpcharHost);
			if (mpcharHost!=0) delete mpcharHost;
			mpcharHost = new char[tlen+1];
			strcpy(mpcharHost, other.mpcharHost);
		}
		else mpcharHost=0;
	}

	return *this;
}



int wuSocket::GetListenFD(){
	return Connfd;
}


int wuSocket::GetConnectedFD(){
	return Sockfd;
}

 
bool wuSocket::IsConnected()
{
	return (Sockfd!=0);
}	


int wuSocket::Receive(char FAR* buf, int len)
{
	return Receive(buf, len, 0);
}



int wuSocket::Receive(char FAR* buf, int len, int flags)
{
	int error = 0;
	int ReturnVal = recv(Sockfd, buf, len, flags);
	if (ReturnVal==-1) error = ActOnLastError(); 
	else RecordSuccessfulAction();

	return ReturnVal;
}




int wuSocket::Send(const char FAR* buf, int len)
{
	return this->Send(buf, len, 0);
}



/// Send uses FlushFileBuffers to try and ensure all data is written 
/// to the socket immediately. For justification into this see msdn article 
/// 'Socket handles for Windows sockets 2'. An excerpt reads so:
/// A socket handle can optionally be a file handle In Windows Sockets 2. 
/// It is possible to use socket handles withReadFile,WriteFile,ReadFileEx,
/// WriteFileEx,DuplicateHandle, and other Win32 functions.
int wuSocket::Send(const char FAR* buf, int len, int flags)
{
	int error = 0;
	int ReturnVal = send(Sockfd, buf, len, flags);
	if (ReturnVal==-1) error = ActOnLastError(); 
	else 
	{	
		FlushFileBuffers((HANDLE)Sockfd);
		RecordSuccessfulAction();
	}
	return ReturnVal;
}



int wuSocket::Close()
{
	int error = 0;
	int ReturnVal = -1;
		// first shutdown the socket before closing. This should be 
		// done for a graceful closing of the socket. 

	WSASetLastError(0); // reset the error flag so that we know if something went wrong

	if (Sockfd!=0)
	{
		ReturnVal = shutdown(Sockfd, SD_SEND); // SD_BOTH and SD_RECEIVE are considered unsafe in winsock

		ReturnVal = closesocket(Sockfd);
		Sockfd=0;	
	}

	if (ReturnVal!=-1)
	{
		RecordSuccessfulAction();
	}
	else
	{
		error = ActOnLastError(); 
	}

	return ReturnVal;
}
/////////////////



int wuSocket::CloseConnector(){

	int er = 0;

	if (Connfd!=0)
	{
		er = shutdown(Connfd, SD_SEND);// SD_BOTH and SD_RECEIVE are considered unsafe in winsock
		er = closesocket(Connfd);
	}

	Connfd=0;

	return er;
}
////////////////




/*
	WSAENETDOWN	10050
	WSAENETRESET	10052
	WSAECONNABORTED	10053
	WSAENOTCONN	10057
*/

// returns the error code that it acted upon
int wuSocket::ActOnLastError()
{
	int error = 0;

	error = WSAGetLastError(); 

	if (error == WSAENETDOWN ||		// recoverable? Unknown
		error == WSAENETRESET ||
		error == WSAECONNABORTED ||
		error == WSAENOTCONN	||
		error == WSAECONNRESET	||
	//	error == WSAEINTR			// recoverable
	//	error == WSAETIMEDOUT	||	// recoverable don't make this close socket  
		error == WSAECONNREFUSED)

	{
		mblnLastActionSuccessful = false;
		if (Sockfd !=0)
		{
			Close();
		}
	}
	else if (	error == WSAEINTR	||	// recoverable errors	
				error == WSAETIMEDOUT)
	{
		mblnLastActionSuccessful = false;
	}

	return error;
}


void wuSocket::RecordSuccessfulAction()
{
	mblnLastActionSuccessful = true;
}



bool wuSocket::WasLastActionSuccessful()
{
	return mblnLastActionSuccessful;
}


	// returns the name of the publishing machine
char* wuSocket::GetPeerName(char* pcharPeerNameBuffer, int intSizePeerName) throw(wuSocketException)
{
	struct sockaddr_in		objNameStruct;
	char					strMessage[1024];
	int						intStatus		= 0;
	int						intNameLen		= 0;
	char FAR *				strHostBuffer	= 0;

	objNameStruct.sin_family		= AF_INET;
	intNameLen = sizeof(objNameStruct);

	intStatus = getpeername(Sockfd, (struct sockaddr FAR*)&objNameStruct, &intNameLen);

	if (intStatus != 0)
	{
		sprintf(strMessage, "GetPeerName - Error, %s", TranslateWinsockStatus(WSAGetLastError()));
		throw wuSocketException(strMessage);
	} 
	else
	{
		strHostBuffer = inet_ntoa(objNameStruct.sin_addr);

		if (strHostBuffer!=0 && pcharPeerNameBuffer!=0)
		{
			strncpy(pcharPeerNameBuffer, inet_ntoa(objNameStruct.sin_addr), intSizePeerName);
			return pcharPeerNameBuffer;
		}
		else if (pcharPeerNameBuffer!=0)
		{
			pcharPeerNameBuffer[0]=0;
			return 0;
		}
		else
		{
			throw wuSocketException("Null Pointer provided to GetPeerName!");
		}
	}
}



	// returns the port of the publishing machine
unsigned short wuSocket::GetPeerPortAddress() throw(wuSocketException)
{
	struct sockaddr_in		objNameStruct;
	char					strMessage[1024];
	int						intStatus		= 0;
	int						intNameLen		= 0;
	char FAR *				strHostBuffer	= 0;

	objNameStruct.sin_family		= AF_INET;
	intNameLen = sizeof(objNameStruct);

	intStatus = getpeername(Sockfd, (struct sockaddr FAR*)&objNameStruct, &intNameLen);

	if (intStatus != 0)
	{
		sprintf(strMessage, "GetPeerPortAddress - Error, %s", TranslateWinsockStatus(WSAGetLastError()));
		throw wuSocketException(strMessage);
	} 
	else
	{
		strHostBuffer = inet_ntoa(objNameStruct.sin_addr);

		if (strHostBuffer!=0)
		{
			return ntohs(objNameStruct.sin_port);
		}
		else
		{
			throw wuSocketException("Null Pointer provided by winsock getpeername!");
		}
	}
}






const int wuSocket::smintErrorCodes[] =	{	
									0,
									WSAEINTR,              
									WSAEBADF,              
									WSAEACCES,             
									WSAEFAULT,             
									WSAEINVAL,             
									WSAEMFILE,             
									WSAEWOULDBLOCK,
									WSAEINPROGRESS,
									WSAEALREADY,   
									WSAENOTSOCK,   
									WSAEDESTADDRREQ,
									WSAEMSGSIZE,    
									WSAEPROTOTYPE,  
									WSAENOPROTOOPT, 
									WSAEPROTONOSUPPORT,
									WSAESOCKTNOSUPPORT,
									WSAEOPNOTSUPP,     
									WSAEPFNOSUPPORT,   
									WSAEAFNOSUPPORT,   
									WSAEADDRINUSE,   
									WSAEADDRNOTAVAIL,  
									WSAENETDOWN,       
									WSAENETUNREACH,    
									WSAENETRESET,      
									WSAECONNABORTED,   
									WSAECONNRESET,     
									WSAENOBUFS,        
									WSAEISCONN,        
									WSAENOTCONN,       
									WSAESHUTDOWN,      
									WSAETOOMANYREFS,   
									WSAETIMEDOUT,      
									WSAECONNREFUSED,   
									WSAELOOP,          
									WSAENAMETOOLONG,   
									WSAEHOSTDOWN,      
									WSAEHOSTUNREACH,   
									WSAENOTEMPTY,      
									WSAEPROCLIM,       
									WSAEUSERS,         
									WSAEDQUOT,         
									WSAESTALE,         
									WSAEREMOTE,
									WSASYSNOTREADY,        
									WSAVERNOTSUPPORTED,
									WSANOTINITIALISED,
									WSAEDISCON
};

const char* wuSocket::smstrErrorMessages[] = {	
							"Publishing was successful",
							"WSAEINTR",              
							"WSAEBADF",              
							"WSAEACCES",             
							"WSAEFAULT",             
							"WSAEINVAL",             
							"WSAEMFILE",
							"WSAEWOULDBLOCK",
							"WSAEINPROGRESS",
							"WSAEALREADY",   
							"WSAENOTSOCK",   
							"WSAEDESTADDRREQ",
							"WSAEMSGSIZE",    
							"WSAEPROTOTYPE",  
							"WSAENOPROTOOPT", 
							"WSAEPROTONOSUPPORT",
							"WSAESOCKTNOSUPPORT",
							"WSAEOPNOTSUPP - - MSG_OOB was specified, but socket is not stream style (SOCK_STREAM), out of band data not supported, or socket supports only send operations.",     
							"WSAEPFNOSUPPORT",   
							"WSAEAFNOSUPPORT",   
							"WSAEADDRINUSE",   
							"WSAEADDRNOTAVAIL",  
							"WSAENETDOWN - The network subsystem or the associated service provider has failed",       
							"WSAENETUNREACH",    
							"WSAENETRESET",      
							"WSAECONNABORTED - The connection has been aborted",   
							"WSAECONNRESET - The connection has been reset",     
							"WSAENOBUFS",        
							"WSAEISCONN",        
							"WSAENOTCONN - No connection is established",       
							"WSAESHUTDOWN",      
							"WSAETOOMANYREFS",   
							"WSAETIMEDOUT - The operation timed out",      
							"WSAECONNREFUSED - The attempt to connect was forcefully rejected",   
							"WSAELOOP",          
							"WSAENAMETOOLONG",   
							"WSAEHOSTDOWN",      
							"WSAEHOSTUNREACH",   
							"WSAENOTEMPTY",      
							"WSAEPROCLIM",       
							"WSAEUSERS",         
							"WSAEDQUOT",         
							"WSAESTALE",         
							"WSAEREMOTE",
							"WSASYSNOTREADY",     
							"WSAVERNOTSUPPORTED",
							"WSANOTINITIALISED - A successful WSAStartup must occur before using this function",
							"WSAEDISCON"
};


	// returns a string-based error message or success message 
	// based on the code given
const char*	wuSocket::TranslateWinsockStatus(int intCode)
{
	const char*	strMessage = "Unknown Error"; 
	int			intCount=0;

	for (intCount=0; intCount<_WU_SOCKETS_NOOFERRORCODES; intCount++)
	{
		if (intCode == smintErrorCodes[intCount])
		{
			strMessage = smstrErrorMessages[intCount];
			intCount = _WU_SOCKETS_NOOFERRORCODES;
		}
	}
	return (strMessage);
}





