//LOCAL INCLUDES
//
#include "winutils/wuSysLogFile.h"
#include <stdio.h>
#include "winutils/wuSYSTEMTIME.h"

using namespace std;


const char* LogFileSuffix=".log";


//--------------------------------------------------------------


wuSysLogFile::wuSysLogFile(void) : wuFile ()
{
	
}


//--------------------------------------------------------------


wuSysLogFile::~wuSysLogFile(void)
{
	// its ok. Close is performed by the parent class.
}


//--------------------------------------------------------------


wuSysLogFile::wuSysLogFile(	std::string			strAppName,
								std::string			strLogFileUniqueName,
								std::string			strFilePath,
								SysLogSeverityCode	objSeverityCode,
								unsigned long		lngMaxFileSize,
								unsigned int		intMaxNumFiles,
								unsigned int		uintWinsockSignificantVersNum,
								unsigned int		uintWinsockInsignificantVersNum)
{
	InitialiseSettings(	strAppName,
						strLogFileUniqueName,
						strFilePath,
						objSeverityCode,
						lngMaxFileSize,
						intMaxNumFiles,
						uintWinsockSignificantVersNum,
						uintWinsockInsignificantVersNum);
}


//--------------------------------------------------------------


void wuSysLogFile::InitialiseSettings(	std::string			strAppName,
										std::string			strLogFileUniqueName,
										std::string			strFilePath,
										SysLogSeverityCode	objSeverityCode,
										unsigned long		lngMaxFileSize,
										unsigned int		intMaxNumFiles,
										unsigned int		uintWinsockSignificantVersNum,
										unsigned int		uintWinsockInsignificantVersNum)
{
	wuFile::Initialise(strFilePath, "", 1000, true, false);
	wuSysLogger::Initialise(strAppName.c_str(), uintWinsockSignificantVersNum, uintWinsockInsignificantVersNum);
	mstrAppName = strAppName;
	mstrLogFileUniqueName = strLogFileUniqueName;
	GetLocalTime(&mSYSTIMELastWriteDate);
	mobjSeverityCode = objSeverityCode;
	mlngMaxFileSize = lngMaxFileSize;
	mintMaxNumFiles = intMaxNumFiles;
	mintFileCount = 1;
}



SysLogSeverityCode wuSysLogFile::GetLogSeverityLevel()
{
	return mobjSeverityCode;
}


//--------------------------------------------------------------


bool wuSysLogFile::Open(bool blnNewFile)
{
	bool	blnSuccess = true;
	int		intFileCount = 0;
	char	achrDateBuffer[10] = {0};
	string	strSearch = "";
	string	strLatestFile = "";

	WIN32_FIND_DATA objFindFileData;
	HANDLE hdlFind;

	
		// if creating file but continuing form an old log file, search for the lastast log file
	if (!blnNewFile)
	{
			// get date time as string
		wuSYSTEMTIME::FormatDate(achrDateBuffer, 10, "YYYYMMDD", mSYSTIMELastWriteDate);

			// create search string
		if (mstrLogFileUniqueName == "")
		{
			strSearch = GetFilePath() + "\\" + achrDateBuffer + "_" + mstrAppName + "_*" +LogFileSuffix;
		}
		else
		{
			strSearch = GetFilePath() + "\\" + achrDateBuffer + "_" + mstrAppName + "_" + mstrLogFileUniqueName + "_*"+LogFileSuffix;
		}

		hdlFind = FindFirstFile(strSearch.c_str(), &objFindFileData);
		if (hdlFind != INVALID_HANDLE_VALUE)
		{
			while (blnSuccess)
			{
				strLatestFile = objFindFileData.cFileName;
				blnSuccess = FindNextFile(hdlFind, &objFindFileData);
				mintFileCount++;
			}
			mintFileCount--;
		}
		FindClose(hdlFind);
	}
	
		// reset the file name before opening
	SetFileName();
	blnSuccess = wuFile::Open();

	return (blnSuccess);
}


//--------------------------------------------------------------


bool wuSysLogFile::Log(const char* LogString)
{
	return WriteLine(LogString);
}


//--------------------------------------------------------------


bool wuSysLogFile::Log(SysLogFacilityCode Facility, 
						SysLogSeverityCode Severity, 
						const char* cpcharMessageContent)
{
	bool		blnReturn = false;
	string		strOutput = "";
	SYSTEMTIME	lCurrentTime;

	GetLocalTime(&lCurrentTime);

		// construct the log message if seve
	if (Severity <= mobjSeverityCode)
	{
			// construct the message
		strOutput = string("Facility:") + cpcharFacility[Facility] +
					string(";Severity:") + cpcharSeverity[Severity] + ";" + cpcharMessageContent;

			// if file has reach max size or if new day, roll to the next file
		if (GetFileSize() + strOutput.size() > mlngMaxFileSize ||
			lCurrentTime.wDay != mSYSTIMELastWriteDate.wDay)
		{
			RollToNextFile();
		}
		blnReturn = wuSysLogger::Log(Facility, Severity, strOutput.c_str());
		mSYSTIMELastWriteDate = lCurrentTime;
	}

	return blnReturn;
}


//--------------------------------------------------------------


void wuSysLogFile::RollToNextFile(void)
{
	Close();
	LimitNumLogFiles();
	SYSTEMTIME	lCurrentTime;

	GetLocalTime(&lCurrentTime);

		// reset the file counter if we have reached a new day or rearch the maximum 
		// number of files per day (not max number of files)
	if (lCurrentTime.wDay != mSYSTIMELastWriteDate.wDay || mintFileCount >= 999)
	{
		mintFileCount = 1;
	}
	else
	{
		mintFileCount++;
	}

		// open the new file without seaching for the latest
	Open(true);
}


//--------------------------------------------------------------


void wuSysLogFile::SetFileName(void)
{
	char	achrDateBuffer[10] = {0};
	char	achrBuffer[1024] = {0};
	SYSTEMTIME	lCurrentTime;

	GetLocalTime(&lCurrentTime);

		// get date time as string
	wuSYSTEMTIME::FormatDate(achrDateBuffer, 10, "YYYYMMDD", lCurrentTime);

		// construct log file name
	if (mstrLogFileUniqueName == "")
	{
		sprintf(achrBuffer, "%s_%s_%03u%s", achrDateBuffer, mstrAppName.c_str(), mintFileCount, LogFileSuffix);
	}
	else
	{
		sprintf(achrBuffer, "%s_%s_%s_%03u%s", achrDateBuffer, mstrAppName.c_str(), mstrLogFileUniqueName.c_str(), mintFileCount, LogFileSuffix);
	}
	wuFile::SetFileName(achrBuffer);
}


//--------------------------------------------------------------


void wuSysLogFile::LimitNumLogFiles(void)
{
	bool	blnSuccess = true;
	int		intFileCount = 0;
	string	strSearch = "";
	string	strOldestFile = "";
	string	strOldestPathFile = "";
	WIN32_FIND_DATA objFindFileData;
	HANDLE hdlFind;
	FILETIME objFileOldestCreationTime;
	
	
		// create search string
	if (mstrLogFileUniqueName == "")
	{
		strSearch = GetFilePath() + "\\*_" + mstrAppName + "_*" + LogFileSuffix;
	}
	else
	{
		strSearch = GetFilePath() + "\\*_" + mstrAppName + "_" + mstrLogFileUniqueName + "_*" + LogFileSuffix;
	}

	hdlFind = FindFirstFile(strSearch.c_str(), &objFindFileData);
	objFileOldestCreationTime = objFindFileData.ftCreationTime;
	strOldestFile = objFindFileData.cFileName;
	if (hdlFind != INVALID_HANDLE_VALUE)
	{
		while (blnSuccess)
		{
			blnSuccess = FindNextFile(hdlFind, &objFindFileData);
			intFileCount++;

			if (CompareFileTime(&objFileOldestCreationTime, &objFindFileData.ftCreationTime) == 1)
			{
				strOldestFile = objFindFileData.cFileName;
				objFileOldestCreationTime = objFindFileData.ftCreationTime;
			}
		}
	}
	FindClose(hdlFind);

		// delete oldest file
	if (intFileCount >= mintMaxNumFiles)
	{
		strOldestPathFile = GetFilePath() + "\\" + strOldestFile;
		remove(strOldestPathFile.c_str());
	}
}


//--------------------------------------------------------------
