#ifndef _WINSOCKAPI_
	#define _WINSOCKAPI_
#endif//_WINSOCKAPI_

#ifndef STRICT
	#define STRICT
#endif
#ifndef _WIN32_WINNT 
	#define _WIN32_WINNT 0x0400
#endif//_WIN32_WINNT

// fix test for mfc
#if defined(WIN32)
	#ifdef _AFXDLL
		#include <afx.h>
	#else
		#include<windows.h>
	#endif
#endif

#include <stdio.h>
#include "winutils/wuStdUtils.h"
#include "winutils/wuStokenizer.h"
#include "winutils/wuSysLogStringGenerator.h"


static const char* scpcharUnknown = "Unknown";




wuSysLogStringGenerator::wuSysLogStringGenerator()
{
	strcpy(mpcharLocalHostName, scpcharUnknown);
	blnWSAInitialised = false;
}



wuSysLogStringGenerator::~wuSysLogStringGenerator()
{
	UnInitialise();
}



void wuSysLogStringGenerator::UnInitialise()
{
	if (blnWSAInitialised)
	{
		WSACleanup();
		blnWSAInitialised = false;		
	}
}



bool wuSysLogStringGenerator::IsInitialised()
{
	return blnWSAInitialised;
}



void wuSysLogStringGenerator::Initialise(const char* cpcharProcessName, unsigned int uintWinsockSignifVersNum, unsigned int uintWinsockInsigVersNum) throw (wuSocketException)
{
	WSADATA wsaData;

	if (blnWSAInitialised)
	{
		WSACleanup();
		blnWSAInitialised = false;
	}

	wu::strlcpy(mpcharProcessName, cpcharProcessName, 31);

	int intReturn = WSAStartup(MAKEWORD(uintWinsockSignifVersNum, uintWinsockInsigVersNum), &wsaData);

	if (intReturn==NULL)
	{
		blnWSAInitialised = true;

		hostent* he;

		he = gethostbyname("localhost");
			
		if (he==NULL)
		{
			throw wuSocketException(wuSocketException::UNDEF_ERROR);
		}
		else
		{
			strcpy(mpcharLocalHostName, he->h_name);
		}
	}
	else
	{
		throw wuSocketException(wuSocketException::WINSOCK_NOT_STARTED);
	}
}









size_t wuSysLogStringGenerator::CreatelogString (char* pcharLogStringBuffer, 
											unsigned int uintLogStringBufferLen, 
											SysLogFacilityCode Facility, 
											SysLogSeverityCode Severity,
											const char* cpcharMessageContent)
{
	unsigned int uintPriority = 0;
	SYSTEMTIME LocalTime;
	memset((void*)&LocalTime, 0, sizeof(SYSTEMTIME));
	char pcharTempBuffer[1024];
	char pcharAdditionalSpace[2]={0};
	char pcharHour[3]={0};
	char pcharMinute[3]={0};
	char pcharSecond[3]={0};

	GetLocalTime(&LocalTime);

	uintPriority = (Facility * 8) + Severity;
	
	// maybe add extra formatting for day of month
	if (LocalTime.wDay<10)
	{
		pcharAdditionalSpace[0]=' ';// add an additional space to make up the lost character for the single digit number
	}

	// format the hour
	if (LocalTime.wHour<10)
	{
		pcharHour[0]='0';
	}

	sprintf(pcharHour, "%s%d", pcharHour, LocalTime.wHour);

	// format the minute
	if (LocalTime.wMinute<10)
	{
		pcharMinute[0]='0';
	}

	sprintf(pcharMinute, "%s%d", pcharMinute, LocalTime.wMinute);

	// format the second
	if (LocalTime.wSecond<10)
	{
		pcharSecond[0]='0';
	}

	sprintf(pcharSecond, "%s%d", pcharSecond, LocalTime.wSecond);

	sprintf(pcharTempBuffer, "<%d>%s %s%d %s:%s:%s %s %.*s:%.*s",
								uintPriority, 
								scpcharSysLogMonth[LocalTime.wMonth-1],
								pcharAdditionalSpace,
								LocalTime.wDay,
								pcharHour,
								pcharMinute,
								pcharSecond,
								mpcharLocalHostName,
								_SYSLOG_MAX_TAG,
								mpcharProcessName,
								_SYSLOG_MAX_MESSAGE_CONTENT,
								cpcharMessageContent);

	// now copy to the specified buffer of size uintLogStringBufferLen
	return wu::strlcpy(pcharLogStringBuffer, pcharTempBuffer, uintLogStringBufferLen);
}




SysLogSeverityCode wuSysLogStringGenerator::StringToSeverityCode(const char* cpcharSeverity)
{
	bool blnFound = false;
	unsigned int uintIndex = 0;
	int intRes = 1;
	SysLogSeverityCode RetCode;

	while (!blnFound && *cpcharSeverityCodeString[uintIndex]!=0 )
	{
		intRes = strcmp(wuStokenizer::toLowerCase(cpcharSeverityCodeString[uintIndex]).c_str(), 
							wuStokenizer::toLowerCase(cpcharSeverity).c_str());

		if (intRes==0)
		{
			blnFound = true;
		}
		else
		{
			uintIndex++;		
		}
	}
	
	if (blnFound)
	{
		RetCode = (SysLogSeverityCode)uintIndex;
	}
	else
	{
		RetCode = syslgEmergency;
	}

	return RetCode;
}

