
#include "winutils/wuWinService.h"
#include "winutils/wuStdUtils.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <exception>




using namespace std;

// *** Some usefull defines
#define _YES_   0x01
#define _NO_    0x00
#define _OK_    0x00
#define _ERROR_ 0x01


typedef BOOL (__stdcall *ChangeServiceConfig2Type) (SC_HANDLE hService,
													DWORD dwInfoLevel,
													LPVOID lpInfo
													);



wuWinService* wuWinService::mThis=0;


wuWinService::wuWinService()
{
	if (wuWinService::mThis!=0)
	{
		throw exception ("Service Instance Already exists");
	}
	else
	{
		wuWinService::mThis = this;
	}

	mSrvTable[0].lpServiceName = 0;
	mSrvTable[0].lpServiceProc = 0;

	mpcharDisplayName = 0;
	mpcharServiceDescription = 0;
}



void wuWinService::Initialise(	const char* pcharServiceName, 
								const char* pcharDisplayName, 
								const char* pcharDescription) throw ()
{
	if (pcharServiceName!=0)
	{
		int intLen = strlen(pcharServiceName) +1;

		mSrvTable[0].lpServiceName = new char[intLen];
		strcpy(mSrvTable[0].lpServiceName, pcharServiceName);

		// set up the display name
		if (pcharDisplayName!=0)
		{
			intLen = strlen(pcharDisplayName)+1;
			mpcharDisplayName = new char[intLen];
			strcpy(mpcharDisplayName, pcharDisplayName);
		}

		// setup the description (not available in NT)
		if (pcharDescription!=0)
		{
			intLen = strlen(pcharDescription)+1;
			mpcharServiceDescription = new char[intLen];
			strcpy(mpcharServiceDescription, pcharDescription);
		}

		mSrvTable[0].lpServiceProc = wuWinService::XServiceMain;  // Service EntryPoint
		mSrvTable[1].lpServiceName = NULL;         // Nothing
		mSrvTable[1].lpServiceProc = NULL;         // Nothing
	}
	else
	{
		throw exception("Error! Service must have a name");
	}
}



void wuWinService::StartServiceCtrlDispatcher()
{
	if (!::StartServiceCtrlDispatcher(mSrvTable) )
	{
		throw exception("Unable to start service");
	}
}




wuWinService::~wuWinService()
{
	if (mSrvTable[0].lpServiceName!=0)
	{
		delete[] mSrvTable[0].lpServiceName;
		mSrvTable[0].lpServiceName = 0;
	}

	if (mpcharDisplayName!=0)
	{
		delete[] mpcharDisplayName;
	}

	if (mpcharServiceDescription!=0)
	{
		delete[] mpcharServiceDescription;
	}
}





int wuWinService::QueryServiceStatus() throw()
{
	SC_HANDLE hService = NULL;	
	SC_HANDLE hSCManager = NULL;
	SERVICE_STATUS ServiceStatus;
	BOOL blnResult = FALSE;

	hSCManager = OpenSCManager( NULL, NULL, SC_MANAGER_ALL_ACCESS );

	if ( hSCManager == NULL ) 
	{
		throw exception("Can't Open SCManager");
	}

	hService = OpenService(hSCManager, mSrvTable[0].lpServiceName, SERVICE_ALL_ACCESS);
	blnResult = ::QueryServiceStatus(hService, &ServiceStatus);

	CloseServiceHandle(hService);
	CloseServiceHandle(hSCManager);

	if (blnResult==NULL)
	{
		throw exception("QueryServiceStatus Failed");
	}

	return ServiceStatus.dwCurrentState;
}





void wuWinService::InstallService(void) throw()
{
	SC_HANDLE scHndl;
	SC_HANDLE scServ;
	char	strQualifiedExeName[256];
	char	temp[258]={0};

	GetModuleFileName(NULL, temp, 256);

	sprintf(strQualifiedExeName, "\"%s\"", temp);
	// fix, path needs to be fully quoted in case of a space
	scHndl = OpenSCManager( NULL, NULL, SC_MANAGER_CREATE_SERVICE );
	if ( scHndl == NULL ) 
	{
		throw exception("Can't open SCManager");
	}

	// Install the XService to Service Control Manager Database
	scServ = CreateService(scHndl,
                           mSrvTable[0].lpServiceName,
                           mpcharDisplayName,// display name
                           SERVICE_ALL_ACCESS |SERVICE_CHANGE_CONFIG,
                           SERVICE_WIN32_OWN_PROCESS | SERVICE_INTERACTIVE_PROCESS,
                           SERVICE_AUTO_START,
                           SERVICE_ERROR_NORMAL,
                           strQualifiedExeName,
                           NULL,
                           NULL,
                           NULL,
                           NULL,
                           NULL );

	if ( scServ == NULL )
	{
		CloseServiceHandle( scServ );

		DWORD dwErr = GetLastError();
		char pcharErrBuff[1024]={0};
		sprintf(pcharErrBuff, "Error Creating Service. Error Code %d",  dwErr);
		throw exception(pcharErrBuff);
	}
	else
	{
		HINSTANCE DesLib = LoadLibrary("advapi32.dll");	// must first test that library exists

		if (DesLib)
		{
			ChangeServiceConfig2Type ServDescFunc = (ChangeServiceConfig2Type)GetProcAddress(DesLib, "ChangeServiceConfig2A");

			if (ServDescFunc)
			{
				SERVICE_DESCRIPTION ServDesc;
				ServDesc.lpDescription = mpcharServiceDescription!=0?mpcharServiceDescription:"No Description Available";
				ServDescFunc(scServ, SERVICE_CONFIG_DESCRIPTION, &ServDesc);
			}

			FreeLibrary(DesLib);
		}
	}

	CloseServiceHandle( scServ );
	CloseServiceHandle( scHndl );
}






void wuWinService::UninstallService(void) throw()
{

	SC_HANDLE scHndl;
	SC_HANDLE scServ;
	DWORD _err = 0;

	// UnInstall the XService from Service Control Manager Database
	scHndl = OpenSCManager( NULL, NULL, SC_MANAGER_CREATE_SERVICE );
	if ( scHndl == NULL ) 
	{
		throw exception("Error in Opening SCManager");
	}

	scServ = OpenService( scHndl, mSrvTable[0].lpServiceName, SERVICE_ALL_ACCESS  );

	_err = GetLastError();

	if ( scServ == NULL )
	{
		if  (  _err == ERROR_SERVICE_DOES_NOT_EXIST )
		{
			(void) CloseHandle( scServ );
		}

		throw exception("error, Service does not exist");
	}
   
	if ( !DeleteService( scServ ) )
	{
		throw exception("Error in uninstallService-DeleteService !");
	}

	(void) CloseHandle(scServ);
	(void) CloseHandle(scHndl);
}





bool wuWinService::ServiceInstalled(void)
{
	SC_HANDLE scHndl;
	SC_HANDLE scServ;
	DWORD          _err = 0;
	bool blnReturn = false;

	scHndl = OpenSCManager( NULL,  NULL, SC_MANAGER_CREATE_SERVICE );
	if ( scHndl == NULL )
	{
		throw exception("Error in isServiceInstalled-OpenSCManager !");
	}

	scServ = OpenService( scHndl, mSrvTable[0].lpServiceName, SERVICE_ALL_ACCESS  );

	_err = GetLastError();

	if ( scServ == NULL )
	{
		if  (  _err == ERROR_SERVICE_DOES_NOT_EXIST )
		{
			CloseServiceHandle( scHndl );
			blnReturn = false;
		}
		else
		{
			throw exception("Error in isServiceInstalled-OpenService !");
		}
	}
	else
	{
		blnReturn = true;
		CloseServiceHandle( scServ );
	}

	(void) CloseServiceHandle( scHndl );

	return blnReturn;
}







void WINAPI wuWinService::XServiceMain(DWORD dwNumServiceArgs, LPTSTR *lpServiceArgs)
{
	if (wuWinService::mThis!=0)
	{
		mThis->ServiceMain(dwNumServiceArgs, lpServiceArgs);
	}
}



void wuWinService::ServiceMain(DWORD dwNumServiceArgs, LPTSTR *lpServiceArgs)
{
	DWORD					rc=0;
	DWORD					wrdBytesNeeded=0;
	int						blnResult=0;
	const int				cintConfigBufferSize = 4096;
	char					strBuffer1[1024]={0};
	QUERY_SERVICE_CONFIG*	lpServiceConfig;

	SC_HANDLE				scHndl;
	SC_HANDLE				scServ;
	DWORD					_err = 0;
	DWORD					wrdError=0;

	lpServiceConfig = (LPQUERY_SERVICE_CONFIG)LocalAlloc(LPTR, cintConfigBufferSize);
	 
    if (lpServiceConfig == NULL) 
    {
		MessageBox( NULL, "Error allocating memory for Service Query !", mSrvTable[0].lpServiceName, MB_ICONERROR );
		DebugBreak(); // Force an error 
    }

	// Create a semaphore to keep service alive
	HANDLE hControlEvent = CreateEvent( NULL, FALSE, FALSE, mSrvTable[0].lpServiceName);

	// Register function that will receive messages from service manager
	// Must be the first action in Service Main
	mhCtrlStatus = RegisterServiceCtrlHandler( mSrvTable[0].lpServiceName, (LPHANDLER_FUNCTION)XControlHandler );

	if ( mhCtrlStatus == (SERVICE_STATUS_HANDLE)NULL )
	{
		MessageBox( NULL, "Error in XServiceMain-RegisterServiceCtrlHandler !", mSrvTable[0].lpServiceName, MB_ICONERROR );
		DebugBreak(); // Force an error 
    }

	mStatus.dwServiceType              = SERVICE_WIN32_OWN_PROCESS | SERVICE_INTERACTIVE_PROCESS;
	mStatus.dwCurrentState             = SERVICE_RUNNING;
	mStatus.dwControlsAccepted         = SERVICE_ACCEPT_STOP | SERVICE_ACCEPT_SHUTDOWN;
	mStatus.dwWin32ExitCode            = NO_ERROR;
	mStatus.dwServiceSpecificExitCode  = NO_ERROR;
	mStatus.dwCheckPoint               = 0;
	mStatus.dwWaitHint                 = 0;

	// Make the first Status update
	rc = SetServiceStatus( mhCtrlStatus, &mStatus );
	if (!rc)
	{
		MessageBox( NULL, "Error in XServiceMain-SetServiceStatus !", mSrvTable[0].lpServiceName, MB_ICONERROR );
		DebugBreak();
    }

	// After this point the XControlHandler function will receive control requests.
	// You can start your processes, threads and any other task that you want

	// find the path name of the service binary (exe)
	scHndl = OpenSCManager( NULL,  NULL, SC_MANAGER_CREATE_SERVICE );
	scServ = OpenService( scHndl, mSrvTable[0].lpServiceName, SERVICE_ALL_ACCESS  );
	if (scServ)
	{
		blnResult = QueryServiceConfig(	scServ,
										lpServiceConfig,
										cintConfigBufferSize,
										&wrdBytesNeeded);
		if (!blnResult)
		{
			wrdError = GetLastError();
			switch (wrdError)
			{
				case ERROR_ACCESS_DENIED:
					strcpy (strBuffer1, 
							"Could not retrieve binary name - The specified handle was not opened with SERVICE_QUERY_CONFIG access");
					break;
					case ERROR_INSUFFICIENT_BUFFER:
					strcpy (strBuffer1, 
							"Could not retrieve binary name - There is more service configuration information than would fit into the lpServiceConfig buffer. The number of bytes required to get all the information is returned in the pcbBytesNeeded parameter. Nothing is written to the lpServiceConfig buffer");
					break;
				case ERROR_INVALID_HANDLE:
					strcpy (strBuffer1, 
							"Could not retrieve binary name - The specified handle is invalid");
					break;
				case ERROR_NOACCESS:
					sprintf (strBuffer1, 
							"Could not retrieve binary name - Invalid access to memory location. Bytes needed %ld",
							wrdBytesNeeded);
					break;
				default:
					sprintf (strBuffer1, "Could not retrieve binary name - Error code %ld", wrdError);
			}
			MessageBox( NULL, strBuffer1, "Error", MB_ICONERROR );
		}
	} 
	else
	{
		wrdError = GetLastError();
		sprintf (strBuffer1, "Error while opening service - Error code %ld", wrdError);
		MessageBox( NULL, strBuffer1, "Error", MB_ICONERROR );
	}

	  // finished finding the path name

	// start the Service
	Start();

	// end of your code

	// Keep service running
   WaitForSingleObject( hControlEvent, INFINITE );

	// Service received a termination command (see XControlHandler function )
   mStatus.dwCurrentState = SERVICE_STOPPED;
   (void) SetServiceStatus( mhCtrlStatus, &mStatus );

	CloseHandle(hControlEvent);

   return;
}









void WINAPI wuWinService::XControlHandler(DWORD dwControl)
{
	if (wuWinService::mThis!=0)
	{
		wuWinService::mThis->ControlHandler(dwControl);
	}
}



void wuWinService::ControlHandler(DWORD dwControl)
{
	HANDLE hControlEvent = CreateEvent( NULL, FALSE, FALSE, mSrvTable[0].lpServiceName);
	DWORD rc;

	// Receive messages from service control dispatcher
	switch (dwControl)
	{
	   case SERVICE_CONTROL_PAUSE :
	   case SERVICE_CONTROL_CONTINUE :
	   case SERVICE_CONTROL_INTERROGATE :
		    mStatus.dwWaitHint     = 0;
		    break;
	   case SERVICE_CONTROL_SHUTDOWN:
	   case SERVICE_CONTROL_STOP :
		    mStatus.dwCurrentState = SERVICE_STOP_PENDING;
		    mStatus.dwWaitHint     = 5000;
		    break;
	}


	// Update service status
	rc = SetServiceStatus( mhCtrlStatus, &mStatus );
	if (!rc)
    {
		MessageBox( NULL, "Error in XControlHandler-SetServiceStatus !", mSrvTable[0].lpServiceName, MB_ICONERROR );
		DebugBreak();
    }

	// If message is a stop or shutdown so, service is going to be stoped
   if ( (dwControl == SERVICE_CONTROL_STOP) || (dwControl == SERVICE_CONTROL_SHUTDOWN) )
		(void) SetEvent( hControlEvent );


	CloseHandle(hControlEvent);
}





unsigned int wuWinService::GetServiceName(char* pcharBuffer, unsigned int len)
{
	unsigned int uintCopied = 0;

	if (pcharBuffer!=0 && len>0 && mSrvTable[0].lpServiceName!=0)
	{
		uintCopied = wu::strlcpy(pcharBuffer, mSrvTable[0].lpServiceName, len);
	}
	
	return uintCopied;
}



unsigned int wuWinService::GetDisplayName(char* pcharBuffer, unsigned int len)
{
	unsigned int uintCopied = 0;

	if (pcharBuffer!=0 && len>0 && mpcharDisplayName!=0)
	{
		uintCopied = wu::strlcpy(pcharBuffer, mpcharDisplayName, len);
	}
	
	return uintCopied;
}



unsigned int wuWinService::GetServiceDescription(char* pcharBuffer, unsigned int len)
{
	unsigned int uintCopied = 0;

	if (pcharBuffer!=0 && len>0 && mpcharServiceDescription!=0)
	{
		uintCopied = wu::strlcpy(pcharBuffer, mpcharServiceDescription, len);
	}
	
	return uintCopied;
}











